@@ -64,8 +64,6 @@ int main(int, char* []) {
          *********************/
 
         Calendar calendar = TARGET();
-        // uncommenting the following line generates an error
-        // calendar = Tokyo();
         Date settlementDate(22, September, 2004);
         // must be a business day
         settlementDate = calendar.adjust(settlementDate);
@@ -163,100 +161,100 @@ int main(int, char* []) {
             Handle<Quote>(d1wRate),
             1*Weeks, fixingDays,
             calendar, ModifiedFollowing,
-            true, fixingDays, depositDayCounter));
+            true, depositDayCounter));
         boost::shared_ptr<RateHelper> d1m(new DepositRateHelper(
             Handle<Quote>(d1mRate),
             1*Months, fixingDays,
             calendar, ModifiedFollowing,
-            true, fixingDays, depositDayCounter));
+            true, depositDayCounter));
         boost::shared_ptr<RateHelper> d3m(new DepositRateHelper(
             Handle<Quote>(d3mRate),
             3*Months, fixingDays,
             calendar, ModifiedFollowing,
-            true, fixingDays, depositDayCounter));
+            true, depositDayCounter));
         boost::shared_ptr<RateHelper> d6m(new DepositRateHelper(
             Handle<Quote>(d6mRate),
             6*Months, fixingDays,
             calendar, ModifiedFollowing,
-            true, fixingDays, depositDayCounter));
+            true, depositDayCounter));
         boost::shared_ptr<RateHelper> d9m(new DepositRateHelper(
             Handle<Quote>(d9mRate),
             9*Months, fixingDays,
             calendar, ModifiedFollowing,
-            true, fixingDays, depositDayCounter));
+            true, depositDayCounter));
         boost::shared_ptr<RateHelper> d1y(new DepositRateHelper(
             Handle<Quote>(d1yRate),
             1*Years, fixingDays,
             calendar, ModifiedFollowing,
-            true, fixingDays, depositDayCounter));
+            true, depositDayCounter));
 
 
         // setup FRAs
         boost::shared_ptr<RateHelper> fra3x6(new FraRateHelper(
             Handle<Quote>(fra3x6Rate),
             3, 6, fixingDays, calendar, ModifiedFollowing,
-            true, fixingDays, depositDayCounter));
+            true, depositDayCounter));
         boost::shared_ptr<RateHelper> fra6x9(new FraRateHelper(
             Handle<Quote>(fra6x9Rate),
             6, 9, fixingDays, calendar, ModifiedFollowing,
-            true, fixingDays, depositDayCounter));
+            true, depositDayCounter));
         boost::shared_ptr<RateHelper> fra6x12(new FraRateHelper(
             Handle<Quote>(fra6x12Rate),
             6, 12, fixingDays, calendar, ModifiedFollowing,
-            true, fixingDays, depositDayCounter));
+            true, depositDayCounter));
 
 
         // setup futures
-        Rate convexityAdjustment = 0.0;
+        // Rate convexityAdjustment = 0.0;
         Integer futMonths = 3;
         Date imm = IMM::nextDate(settlementDate);
         boost::shared_ptr<RateHelper> fut1(new FuturesRateHelper(
             Handle<Quote>(fut1Price),
             imm,
             futMonths, calendar, ModifiedFollowing,
-            depositDayCounter, convexityAdjustment));
+            true, depositDayCounter));
         imm = IMM::nextDate(imm+1);
         boost::shared_ptr<RateHelper> fut2(new FuturesRateHelper(
-            Handle<Quote>(fut1Price),
+            Handle<Quote>(fut2Price),
             imm,
             futMonths, calendar, ModifiedFollowing,
-            depositDayCounter, convexityAdjustment));
+            true, depositDayCounter));
         imm = IMM::nextDate(imm+1);
         boost::shared_ptr<RateHelper> fut3(new FuturesRateHelper(
-            Handle<Quote>(fut1Price),
+            Handle<Quote>(fut3Price),
             imm,
             futMonths, calendar, ModifiedFollowing,
-            depositDayCounter, convexityAdjustment));
+            true, depositDayCounter));
         imm = IMM::nextDate(imm+1);
         boost::shared_ptr<RateHelper> fut4(new FuturesRateHelper(
-            Handle<Quote>(fut1Price),
+            Handle<Quote>(fut4Price),
             imm,
             futMonths, calendar, ModifiedFollowing,
-            depositDayCounter, convexityAdjustment));
+            true, depositDayCounter));
         imm = IMM::nextDate(imm+1);
         boost::shared_ptr<RateHelper> fut5(new FuturesRateHelper(
-            Handle<Quote>(fut1Price),
+            Handle<Quote>(fut5Price),
             imm,
             futMonths, calendar, ModifiedFollowing,
-            depositDayCounter, convexityAdjustment));
+            true, depositDayCounter));
         imm = IMM::nextDate(imm+1);
         boost::shared_ptr<RateHelper> fut6(new FuturesRateHelper(
-            Handle<Quote>(fut1Price),
+            Handle<Quote>(fut6Price),
             imm,
             futMonths, calendar, ModifiedFollowing,
-            depositDayCounter, convexityAdjustment));
+            true, depositDayCounter));
         imm = IMM::nextDate(imm+1);
         boost::shared_ptr<RateHelper> fut7(new FuturesRateHelper(
-            Handle<Quote>(fut1Price),
+            Handle<Quote>(fut7Price),
             imm,
             futMonths, calendar, ModifiedFollowing,
-            depositDayCounter, convexityAdjustment));
+            true, depositDayCounter));
         imm = IMM::nextDate(imm+1);
         boost::shared_ptr<RateHelper> fut8(new FuturesRateHelper(
-            Handle<Quote>(fut1Price),
+            Handle<Quote>(fut8Price),
             imm,
             futMonths, calendar, ModifiedFollowing,
-            depositDayCounter, convexityAdjustment));
+            true, depositDayCounter));
 
 
         // setup swaps
@@ -320,7 +318,10 @@ int main(int, char* []) {
         boost::shared_ptr<YieldTermStructure> depoSwapTermStructure(
             new PiecewiseYieldCurve<Discount,LogLinear>(
                                           settlementDate, depoSwapInstruments,
-                                          termStructureDayCounter, tolerance));
+                                          termStructureDayCounter,
+                                          std::vector<Handle<Quote> >(),
+                                          std::vector<Date>(),
+                                          tolerance));
 
 
         // A depo-futures-swap curve
@@ -342,7 +343,10 @@ int main(int, char* []) {
         boost::shared_ptr<YieldTermStructure> depoFutSwapTermStructure(
             new PiecewiseYieldCurve<Discount,LogLinear>(
                                        settlementDate, depoFutSwapInstruments,
-                                       termStructureDayCounter, tolerance));
+                                       termStructureDayCounter,
+                                       std::vector<Handle<Quote> >(),
+                                       std::vector<Date>(),
+                                       tolerance));
 
 
         // A depo-FRA-swap curve
@@ -361,7 +365,10 @@ int main(int, char* []) {
         boost::shared_ptr<YieldTermStructure> depoFRASwapTermStructure(
             new PiecewiseYieldCurve<Discount,LogLinear>(
                                        settlementDate, depoFRASwapInstruments,
-                                       termStructureDayCounter, tolerance));
+                                       termStructureDayCounter,
+                                       std::vector<Handle<Quote> >(),
+                                       std::vector<Date>(),
+                                       tolerance));
 
 
         // Term structures that will be used for pricing:
