@@ -1,9 +1,10 @@
 /* -*- mode: c++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
 /*
- Copyright (C) 2007 Ferdinando Ametrano
  Copyright (C) 2005, 2006 StatPro Italia srl
  Copyright (C) 2005 Charles Whitmore
+ Copyright (C) 2007, 2008 Ferdinando Ametrano
+ Copyright (C) 2008 Toyin Akin
 
  This file is part of QuantLib, a free-software/open-source library
  for financial quantitative analysts and developers - http://quantlib.org/
@@ -22,15 +23,52 @@
 #include <ql/cashflows/cashflows.hpp>
 #include <ql/cashflows/coupon.hpp>
 #include <ql/termstructures/yield/flatforward.hpp>
-#include <ql/math/solvers1d/brent.hpp>
+//#include <ql/math/solvers1d/brent.hpp>
+#include <ql/math/solvers1d/newtonsafe.hpp>
 #include <ql/cashflows/couponpricer.hpp>
 
+using boost::shared_ptr;
+using boost::dynamic_pointer_cast;
+
 namespace QuantLib {
 
     namespace {
 
         // utility functions
 
+        Rate couponRate(const Leg& leg,
+                        Leg::const_iterator cf) {
+            if (cf==leg.end()) return 0.0;
+
+            Date paymentDate = (*cf)->date();
+            bool firstCouponFound = false;
+            Real nominal = Null<Real>();
+            Time accrualPeriod = Null<Time>();
+            DayCounter dc;
+            Rate result = 0.0;
+            for (; cf<leg.end() && (*cf)->date()==paymentDate; ++cf) {
+                shared_ptr<Coupon> cp = dynamic_pointer_cast<Coupon>(*cf);
+                if (cp) {
+                    if (firstCouponFound) {
+                        QL_REQUIRE(nominal       == cp->nominal() &&
+                                   accrualPeriod == cp->accrualPeriod() &&
+                                   dc            == cp->dayCounter(),
+                                   "cannot aggregate two different coupons on "
+                                   << paymentDate);
+                    } else {
+                        firstCouponFound = true;
+                        nominal = cp->nominal();
+                        accrualPeriod = cp->accrualPeriod();
+                        dc = cp->dayCounter();
+                    }
+                    result += cp->rate();
+                }
+            }
+            QL_ENSURE(firstCouponFound,
+                      "next cashflow (" << paymentDate << ") is not a coupon");
+            return result;
+        }
+
         template <class T>
         Integer sign(T x) {
             static T zero = T();
@@ -42,11 +80,86 @@ namespace QuantLib {
                 return -1;
         }
 
-        const Spread basisPoint_ = 1.0e-4;
 
-        class irrFinder : public std::unary_function<Rate,Real> {
+        Real simpleDuration(const Leg& cashflows,
+                            const InterestRate& y,
+                            const Date& settlementDate) {
+            Real P = 0.0;
+            Real dPdy = 0.0;
+            const DayCounter& dc = y.dayCounter();
+            for (Size i=0; i<cashflows.size(); ++i) {
+                if (!cashflows[i]->hasOccurred(settlementDate)) {
+                    Time t = dc.yearFraction(settlementDate,
+                                             cashflows[i]->date());
+                    Real c = cashflows[i]->amount();
+                    DiscountFactor B = y.discountFactor(t);
+                    P += c * B;
+                    dPdy += t * c * B;
+                }
+            }
+            if (P == 0.0) // no cashflows
+                return 0.0;
+            return dPdy/P;
+        }
+
+        Real modifiedDuration(const Leg& cashflows,
+                              const InterestRate& y,
+                              const Date& settlementDate) {
+            Real P = 0.0;
+            Real dPdy = 0.0;
+            Rate r = y.rate();
+            Natural N = y.frequency();
+            const DayCounter& dc = y.dayCounter();
+            for (Size i=0; i<cashflows.size(); ++i) {
+                if (!cashflows[i]->hasOccurred(settlementDate)) {
+                    Time t = dc.yearFraction(settlementDate,
+                                             cashflows[i]->date());
+                    Real c = cashflows[i]->amount();
+                    DiscountFactor B = y.discountFactor(t);
+
+                    P += c * B;
+                    switch (y.compounding()) {
+                      case Simple:
+                        dPdy -= c * B*B * t;
+                        break;
+                      case Compounded:
+                        dPdy -= c * t * B/(1+r/N);
+                        break;
+                      case Continuous:
+                        dPdy -= c * B * t;
+                        break;
+                      case SimpleThenCompounded:
+                        if (t<=1.0/N)
+                            dPdy -= c * B*B * t;
+                        else
+                            dPdy -= c * t * B/(1+r/N);
+                        break;
+                      default:
+                        QL_FAIL("unknown compounding convention (" <<
+                                Integer(y.compounding()) << ")");
+                    }
+                }
+            }
+
+            if (P == 0.0) // no cashflows
+                return 0.0;
+            return -dPdy/P; // reverse derivative sign
+        }
+
+        Real macaulayDuration(const Leg& cashflows,
+                              const InterestRate& y,
+                              const Date& settlementDate) {
+
+            QL_REQUIRE(y.compounding() == Compounded,
+                       "compounded rate required");
+
+            return (1.0+y.rate()/y.frequency()) *
+                modifiedDuration(cashflows, y, settlementDate);
+        }
+
+        class IrrFinder : public std::unary_function<Rate, Real> {
           public:
-            irrFinder(const Leg& cashflows,
+            IrrFinder(const Leg& cashflows,
                       Real marketPrice,
                       const DayCounter& dayCounter,
                       Compounding compounding,
@@ -55,11 +168,15 @@ namespace QuantLib {
             : cashflows_(cashflows), marketPrice_(marketPrice),
               dayCounter_(dayCounter), compounding_(compounding),
               frequency_(frequency), settlementDate_(settlementDate) {}
-            Real operator()(Rate guess) const {
-                InterestRate y(guess, dayCounter_, compounding_, frequency_);
-                Real NPV = CashFlows::npv(cashflows_,y,settlementDate_);
+            Real operator()(Rate x) const {
+                InterestRate y(x, dayCounter_, compounding_, frequency_);
+                Real NPV = CashFlows::npv(cashflows_, y, settlementDate_);
                 return marketPrice_ - NPV;
             }
+            Real derivative(Rate x) const {
+                InterestRate y(x, dayCounter_, compounding_, frequency_);
+                return modifiedDuration(cashflows_, y, settlementDate_);
+            }
           private:
             const Leg& cashflows_;
             Real marketPrice_;
@@ -94,118 +211,31 @@ namespace QuantLib {
             Real result_;
         };
 
-        Real simpleDuration(const Leg& cashflows,
-                            const InterestRate& rate,
-                            Date settlementDate) {
-
-            Real P = 0.0;
-            Real tP = 0.0;
-
-            for (Size i=0; i<cashflows.size(); ++i) {
-                if (!cashflows[i]->hasOccurred(settlementDate)) {
-                    Time t =
-                        rate.dayCounter().yearFraction(settlementDate,
-                                                       cashflows[i]->date());
-                    Real c = cashflows[i]->amount();
-                    DiscountFactor B = rate.discountFactor(t);
-
-                    P += c * B;
-                    tP += t * c * B;
-                }
-            }
-
-            if (P == 0.0)
-                // no cashflows
-                return 0.0;
-
-            return tP/P;
-        }
-
-        Real modifiedDuration(const Leg& cashflows,
-                              const InterestRate& rate,
-                              Date settlementDate) {
-
-            Real P = 0.0;
-            Real dPdy = 0.0;
-            Rate y = Rate(rate);
-            Integer N = rate.frequency();
-
-            for (Size i=0; i<cashflows.size(); ++i) {
-                if (!cashflows[i]->hasOccurred(settlementDate)) {
-                    Time t =
-                        rate.dayCounter().yearFraction(settlementDate,
-                                                       cashflows[i]->date());
-                    Real c = cashflows[i]->amount();
-                    DiscountFactor B = rate.discountFactor(t);
-
-                    P += c * B;
-                    switch (rate.compounding()) {
-                      case Simple:
-                        dPdy -= c * B*B*t;
-                        break;
-                      case Compounded:
-                        dPdy -= c * B*t/(1+y/N);
-                        break;
-                      case Continuous:
-                        dPdy -= c * B*t;
-                        break;
-                      case SimpleThenCompounded:
-                      default:
-                        QL_FAIL("unsupported compounding type");
-                    }
-                }
-            }
-
-            if (P == 0.0)
-                // no cashflows
-                return 0.0;
-
-            return -dPdy/P;
-        }
-
-        Real macaulayDuration(const Leg& cashflows,
-                              const InterestRate& rate,
-                              Date settlementDate) {
-
-            Rate y = Rate(rate);
-            Integer N = rate.frequency();
-
-            QL_REQUIRE(rate.compounding() == Compounded,
-                       "compounded rate required");
-            QL_REQUIRE(N >= 1, "unsupported frequency");
-
-            return (1+y/N)*modifiedDuration(cashflows,rate,settlementDate);
-        }
-
+        const Spread basisPoint_ = 1.0e-4;
     }
 
-    Leg::const_iterator CashFlows::lastCashFlow(const Leg& leg,
-                                                const Date& refDate) {
-        Date d = (refDate==Date() ?
-                  Settings::instance().evaluationDate() :
-                  refDate);
+    Leg::const_iterator CashFlows::previousCashFlow(const Leg& leg,
+                                                    Date refDate) {
+        if (refDate==Date())
+            refDate = Settings::instance().evaluationDate();
 
-        if ( ! (*leg.begin())->hasOccurred(d) )
+        if ( ! (*leg.begin())->hasOccurred(refDate) )
             return leg.end();
 
-        Leg::const_iterator i;
-        for (i = leg.begin()+1; i<leg.end(); ++i) {
-            if ( ! (*i)->hasOccurred(d) )
-                return --i;
-        }
-        return leg.end();
+        Leg::const_iterator i = nextCashFlow(leg, refDate);
+        Date beforeLastPaymentDate = (*--i)->date()-1;
+        return nextCashFlow(leg, beforeLastPaymentDate);
     }
 
     Leg::const_iterator CashFlows::nextCashFlow(const Leg& leg,
-                                                const Date& refDate) {
-        Date d = (refDate==Date() ?
-                  Settings::instance().evaluationDate() :
-                  refDate);
+                                                Date refDate) {
+        if (refDate==Date())
+            refDate = Settings::instance().evaluationDate();
+
         Leg::const_iterator i;
         for (i = leg.begin(); i<leg.end(); ++i) {
             // the first coupon paying after d is the one we're after
-            if ( ! (*i)->hasOccurred(d) )
-                //return boost::dynamic_pointer_cast<Coupon>(leg[i]);
+            if ( ! (*i)->hasOccurred(refDate) )
                 return i;
         }
         return leg.end();
@@ -213,28 +243,20 @@ namespace QuantLib {
 
     Rate CashFlows::previousCouponRate(const Leg& leg,
                                        const Date& refDate) {
-        Leg::const_iterator cf = lastCashFlow(leg, refDate);
-        if (cf==leg.end()) return 0.0;
-
-        boost::shared_ptr<Coupon> cp=boost::dynamic_pointer_cast<Coupon>(*cf);
-        if (cp) return cp->rate();
-        else    return 0.0;
+        Leg::const_iterator cf = previousCashFlow(leg, refDate);
+        return couponRate(leg, cf);
     }
 
-    Rate CashFlows::currentCouponRate(const Leg& leg,
-                                      const Date& refDate) {
+    Rate CashFlows::nextCouponRate(const Leg& leg,
+                                   const Date& refDate) {
         Leg::const_iterator cf = nextCashFlow(leg, refDate);
-        if (cf==leg.end()) return 0.0;
-
-        boost::shared_ptr<Coupon> cp=boost::dynamic_pointer_cast<Coupon>(*cf);
-        if (cp) return cp->rate();
-        else    return 0.0;
+        return couponRate(leg, cf);
     }
 
     Date CashFlows::startDate(const Leg& cashflows) {
         Date d = Date::maxDate();
         for (Size i=0; i<cashflows.size(); ++i) {
-            boost::shared_ptr<Coupon> c =
+            shared_ptr<Coupon> c =
                 boost::dynamic_pointer_cast<Coupon>(cashflows[i]);
             if (c)
                 d = std::min(d, c->accrualStartDate());
@@ -254,16 +276,15 @@ namespace QuantLib {
 
     Real CashFlows::npv(const Leg& cashflows,
                         const YieldTermStructure& discountCurve,
-                        const Date& settlementDate,
+                        Date settlementDate,
                         const Date& npvDate,
-                        Integer exDividendDays) {
-        Date d = settlementDate != Date() ?
-                 settlementDate :
-                 discountCurve.referenceDate();
+                        Natural exDividendDays) {
+        if (settlementDate==Date())
+            settlementDate = discountCurve.referenceDate();
 
         Real totalNPV = 0.0;
         for (Size i=0; i<cashflows.size(); ++i) {
-            if (!cashflows[i]->hasOccurred(d+exDividendDays))
+            if (!cashflows[i]->hasOccurred(settlementDate+exDividendDays))
                 totalNPV += cashflows[i]->amount() *
                             discountCurve.discount(cashflows[i]->date());
         }
@@ -277,26 +298,25 @@ namespace QuantLib {
     Real CashFlows::npv(const Leg& cashflows,
                         const InterestRate& irr,
                         Date settlementDate) {
-        if (settlementDate == Date())
+        if (settlementDate==Date())
             settlementDate = Settings::instance().evaluationDate();
+
         FlatForward flatRate(settlementDate, irr.rate(), irr.dayCounter(),
-                                 irr.compounding(), irr.frequency());
+                             irr.compounding(), irr.frequency());
         return npv(cashflows, flatRate, settlementDate, settlementDate);
     }
 
     Real CashFlows::bps(const Leg& cashflows,
                         const YieldTermStructure& discountCurve,
-                        const Date& settlementDate,
+                        Date settlementDate,
                         const Date& npvDate,
-                        Integer exDividendDays) {
-
-        Date d = settlementDate;
-        if (d==Date())
-            d = discountCurve.referenceDate();
+                        Natural exDividendDays) {
+        if (settlementDate==Date())
+            settlementDate = discountCurve.referenceDate();
 
         BPSCalculator calc(discountCurve, npvDate);
         for (Size i=0; i<cashflows.size(); ++i) {
-            if (!cashflows[i]->hasOccurred(d+exDividendDays))
+            if (!cashflows[i]->hasOccurred(settlementDate+exDividendDays))
                 cashflows[i]->accept(calc);
         }
         return basisPoint_*calc.result();
@@ -305,24 +325,39 @@ namespace QuantLib {
     Real CashFlows::bps(const Leg& cashflows,
                         const InterestRate& irr,
                         Date settlementDate) {
-        if (settlementDate == Date())
+        if (settlementDate==Date())
             settlementDate = Settings::instance().evaluationDate();
+
         FlatForward flatRate(settlementDate, irr.rate(), irr.dayCounter(),
-                                 irr.compounding(), irr.frequency());
+                             irr.compounding(), irr.frequency());
         return bps(cashflows, flatRate, settlementDate, settlementDate);
     }
 
+    Rate CashFlows::atmRate(const Leg& cashFlows,
+                            const YieldTermStructure& discountCurve,
+                            const Date& settlementDate,
+                            const Date& npvDate,
+                            Natural exDividendDays,
+                            Real npv) {
+        Real bps = CashFlows::bps(cashFlows, discountCurve, settlementDate,
+                                  npvDate, exDividendDays);
+        if (npv==Null<Real>())
+            npv = CashFlows::npv(cashFlows, discountCurve, settlementDate,
+                                 npvDate, exDividendDays);
+        return basisPoint_*npv/bps;
+    }
+
     Rate CashFlows::irr(const Leg& cashflows,
                         Real marketPrice,
                         const DayCounter& dayCounter,
                         Compounding compounding,
                         Frequency frequency,
                         Date settlementDate,
-                        Real tolerance,
+                        Real accuracy,
                         Size maxIterations,
                         Rate guess) {
 
-        if (settlementDate == Date())
+        if (settlementDate==Date())
             settlementDate = Settings::instance().evaluationDate();
 
         // depending on the sign of the market price, check that cash
@@ -342,7 +377,7 @@ namespace QuantLib {
             }
         }
         QL_REQUIRE(signChanges > 0,
-                  "the given cash flows cannot result in the given market "
+                   "the given cash flows cannot result in the given market "
                    "price due to their sign");
 
         /* The following is commented out due to the lack of a QL_WARN macro
@@ -364,68 +399,71 @@ namespace QuantLib {
         };
         */
 
-        Brent solver;
+        //Brent solver;
+        NewtonSafe solver;
         solver.setMaxEvaluations(maxIterations);
-        return solver.solve(irrFinder(cashflows, marketPrice, dayCounter,
+        return solver.solve(IrrFinder(cashflows, marketPrice, dayCounter,
                                       compounding, frequency, settlementDate),
-                            tolerance, guess, guess/10.0);
+                            accuracy, guess, guess/10.0);
     }
 
     Time CashFlows::duration(const Leg& cashflows,
                              const InterestRate& rate,
                              Duration::Type type,
                              Date settlementDate) {
-
-        if (settlementDate == Date())
+        if (settlementDate==Date())
             settlementDate = Settings::instance().evaluationDate();
 
         switch (type) {
           case Duration::Simple:
-            return simpleDuration(cashflows,rate,settlementDate);
+            return simpleDuration(cashflows, rate, settlementDate);
           case Duration::Modified:
-            return modifiedDuration(cashflows,rate,settlementDate);
+            return modifiedDuration(cashflows, rate, settlementDate);
           case Duration::Macaulay:
-            return macaulayDuration(cashflows,rate,settlementDate);
+            return macaulayDuration(cashflows, rate, settlementDate);
           default:
             QL_FAIL("unknown duration type");
         }
     }
 
     Real CashFlows::convexity(const Leg& cashflows,
-                              const InterestRate& rate,
+                              const InterestRate& y,
                               Date settlementDate) {
-
-        if (settlementDate == Date())
+        if (settlementDate==Date())
             settlementDate = Settings::instance().evaluationDate();
 
-        DayCounter dayCounter = rate.dayCounter();
+        const DayCounter& dc = y.dayCounter();
 
         Real P = 0.0;
         Real d2Pdy2 = 0.0;
-        Rate y = Rate(rate);
-        Integer N = rate.frequency();
-
+        Rate r = y.rate();
+        Natural N = y.frequency();
         for (Size i=0; i<cashflows.size(); ++i) {
             if (!cashflows[i]->hasOccurred(settlementDate)) {
-                Time t = dayCounter.yearFraction(settlementDate,
-                                                 cashflows[i]->date());
+                Time t = dc.yearFraction(settlementDate,
+                                         cashflows[i]->date());
                 Real c = cashflows[i]->amount();
-                DiscountFactor B = rate.discountFactor(t);
-
+                DiscountFactor B = y.discountFactor(t);
                 P += c * B;
-                switch (rate.compounding()) {
+                switch (y.compounding()) {
                   case Simple:
                     d2Pdy2 += c * 2.0*B*B*B*t*t;
                     break;
                   case Compounded:
-                    d2Pdy2 += c * B*t*(N*t+1)/(N*(1+y/N)*(1+y/N));
+                    d2Pdy2 += c * B*t*(N*t+1)/(N*(1+r/N)*(1+r/N));
                     break;
                   case Continuous:
                     d2Pdy2 += c * B*t*t;
                     break;
                   case SimpleThenCompounded:
+                    if (t<=1.0/N)
+                        d2Pdy2 += c * 2.0*B*B*B*t*t;
+                    else
+                        d2Pdy2 += c * B*t*(N*t+1)/(N*(1+r/N)*(1+r/N));
+                    break;
                   default:
-                    QL_FAIL("unsupported compounding type");
+                    QL_FAIL("unknown compounding convention (" <<
+                            Integer(y.compounding()) << ")");
                 }
             }
         }
@@ -437,18 +475,38 @@ namespace QuantLib {
         return d2Pdy2/P;
     }
 
-    Rate CashFlows::atmRate(const Leg& cashFlows,
-                            const YieldTermStructure& discountCurve,
-                            const Date& settlementDate,
-                            const Date& npvDate,
-                            Integer exDividendDays,
-                            Real npv) {
-        Real bps = CashFlows::bps(cashFlows, discountCurve, settlementDate,
-                                  npvDate, exDividendDays);
-        if (npv==Null<Real>())
-            npv = CashFlows::npv(cashFlows, discountCurve, settlementDate,
-                                  npvDate, exDividendDays);
-        return basisPoint_*npv/bps;
+
+    Real CashFlows::basisPointValue(const Leg& leg,
+                                    const InterestRate& y,
+                                    Date settlementDate) {
+        Real shift = 0.0001;
+        Real dirtyPrice = CashFlows::npv(leg, y, settlementDate);
+        Real modifiedDuration = CashFlows::duration(leg, y,
+                                                    Duration::Modified,
+                                                    settlementDate);
+        Real convexity = CashFlows::convexity(leg, y, settlementDate);
+
+        Real delta = -modifiedDuration*dirtyPrice;
+
+        Real gamma = (convexity/100.0)*dirtyPrice;
+
+        delta *= shift;
+        gamma *= shift*shift;
+
+        return delta + 0.5*gamma;
+    }
+
+    Real CashFlows::yieldValueBasisPoint(const Leg& leg,
+                                         const InterestRate& y,
+                                         Date settlementDate) {
+        Real shift = 0.01;
+
+        Real dirtyPrice = CashFlows::npv(leg, y, settlementDate);
+        Real modifiedDuration = CashFlows::duration(leg, y,
+                                                    Duration::Modified,
+                                                    settlementDate);
+
+        return (1.0/(-dirtyPrice*modifiedDuration))*shift;
     }
 
 }
