@@ -35,12 +35,10 @@
 #include <ql/utilities/null.hpp>
 #include <ql/utilities/vectors.hpp>
 #include <ql/position.hpp>
+#include <ql/indexes/swapindex.hpp>
 
 namespace QuantLib {
 
-    class SwapIndex;
-    class IborIndex;
-
     namespace detail {
 
         Rate effectiveFixedRate(const std::vector<Spread>& spreads,
@@ -55,12 +53,12 @@ namespace QuantLib {
     }
 
 
-    template <typename IndexType,
+    template <typename InterestRateIndexType,
               typename FloatingCouponType,
               typename CappedFlooredCouponType>
     Leg FloatingLeg(const std::vector<Real>& nominals,
                     const Schedule& schedule,
-                    const boost::shared_ptr<IndexType>& index,
+                    const boost::shared_ptr<InterestRateIndexType>& index,
                     const DayCounter& paymentDayCounter,
                     BusinessDayConvention paymentAdj,
                     const std::vector<Natural>& fixingDays,
@@ -71,8 +69,6 @@ namespace QuantLib {
                     bool isInArrears,
                     bool isZero) {
 
-        QL_REQUIRE(!nominals.empty(), "no nominal given");
-
         Size n = schedule.size()-1;
         QL_REQUIRE(nominals.size() <= n,
                    "too many nominals (" << nominals.size() <<
@@ -115,7 +111,7 @@ namespace QuantLib {
             }
             if (detail::get(gearings, i, 1.0) == 0.0) { // fixed coupon
                 leg.push_back(boost::shared_ptr<CashFlow>(new
-                    FixedRateCoupon(detail::get(nominals, i, Null<Real>()),
+                    FixedRateCoupon(detail::get(nominals, i, 1.0),
                                     paymentDate,
                                     detail::effectiveFixedRate(spreads,caps,
                                                                floors,i),
@@ -125,27 +121,29 @@ namespace QuantLib {
                 if (detail::noOption(caps, floors, i))
                     leg.push_back(boost::shared_ptr<CashFlow>(new
                         FloatingCouponType(paymentDate,
-                                   detail::get(nominals, i, Null<Real>()),
-                                   start, end,
-                                   detail::get(fixingDays, i, 2), index,
-                                   detail::get(gearings, i, 1.0),
-                                   detail::get(spreads, i, 0.0),
-                                   refStart, refEnd,
-                                   paymentDayCounter, isInArrears)));
+                               detail::get(nominals, i, 1.0),
+                               start, end,
+                               detail::get(fixingDays, i, index->fixingDays()),
+                               index,
+                               detail::get(gearings, i, 1.0),
+                               detail::get(spreads, i, 0.0),
+                               refStart, refEnd,
+                               paymentDayCounter, isInArrears)));
                 else {
                     leg.push_back(boost::shared_ptr<CashFlow>(new
                         CappedFlooredCouponType(
-                                       paymentDate,
-                                       detail::get(nominals, i, Null<Real>()),
-                                       start, end,
-                                       detail::get(fixingDays, i, 2), index,
-                                       detail::get(gearings, i, 1.0),
-                                       detail::get(spreads, i, 0.0),
-                                       detail::get(caps,   i, Null<Rate>()),
-                                       detail::get(floors, i, Null<Rate>()),
-                                       refStart, refEnd,
-                                       paymentDayCounter,
-                                       isInArrears)));
+                               paymentDate,
+                               detail::get(nominals, i, 1.0),
+                               start, end,
+                               detail::get(fixingDays, i, index->fixingDays()),
+                               index,
+                               detail::get(gearings, i, 1.0),
+                               detail::get(spreads, i, 0.0),
+                               detail::get(caps,   i, Null<Rate>()),
+                               detail::get(floors, i, Null<Rate>()),
+                               refStart, refEnd,
+                               paymentDayCounter,
+                               isInArrears)));
                 }
             }
         }
@@ -153,12 +151,12 @@ namespace QuantLib {
     }
 
 
-    template <typename IndexType,
+    template <typename InterestRateIndexType,
               typename FloatingCouponType,
               typename DigitalCouponType>
     Leg FloatingDigitalLeg(const std::vector<Real>& nominals,
                    const Schedule& schedule,
-                   const boost::shared_ptr<IndexType>& index,
+                   const boost::shared_ptr<InterestRateIndexType>& index,
                    const DayCounter& paymentDayCounter,
                    BusinessDayConvention paymentAdj,
                    const std::vector<Natural>& fixingDays,
@@ -175,8 +173,6 @@ namespace QuantLib {
                    const std::vector<Rate>& putDigitalPayoffs,
                    const boost::shared_ptr<DigitalReplication>& replication) {
 
-        QL_REQUIRE(!nominals.empty(), "no nominal given");
-
         Size n = schedule.size()-1;
         QL_REQUIRE(nominals.size() <= n,
                    "too many nominals (" << nominals.size() <<
@@ -216,18 +212,18 @@ namespace QuantLib {
             }
             if (detail::get(gearings, i, 1.0) == 0.0) { // fixed coupon
                 leg.push_back(boost::shared_ptr<CashFlow>(new
-                    FixedRateCoupon(detail::get(nominals, i, Null<Real>()),
+                    FixedRateCoupon(detail::get(nominals, i, 1.0),
                                     paymentDate,
                                     detail::get(spreads, i, 1.0),
                                     paymentDayCounter,
                                     start, end, refStart, refEnd)));
             } else { // floating digital coupon
-                boost::shared_ptr<FloatingCouponType> underlying =
-                    (boost::shared_ptr<FloatingCouponType>)(new
-                        FloatingCouponType(paymentDate,
-                                       detail::get(nominals, i, Null<Real>()),
+                boost::shared_ptr<FloatingCouponType> underlying(new
+                    FloatingCouponType(paymentDate,
+                                       detail::get(nominals, i, 1.0),
                                        start, end,
-                                       detail::get(fixingDays, i, 2), index,
+                                       detail::get(fixingDays, i, index->fixingDays()),
+                                       index,
                                        detail::get(gearings, i, 1.0),
                                        detail::get(spreads, i, 0.0),
                                        refStart, refEnd,
@@ -251,5 +247,4 @@ namespace QuantLib {
 
 }
 
-
 #endif
