@@ -24,6 +24,7 @@
 #include <ql/cashflows/digitalcmscoupon.hpp>
 #include <ql/cashflows/digitaliborcoupon.hpp>
 #include <ql/cashflows/rangeaccrual.hpp>
+#include <ql/experimental/coupons/subperiodcoupons.hpp>
 #include <ql/pricingengines/blackformula.hpp>
 #include <ql/indexes/interestrateindex.hpp>
 #include <ql/termstructures/yieldtermstructure.hpp>
@@ -35,7 +36,7 @@ namespace QuantLib {
 //===========================================================================//
 
     void BlackIborCouponPricer::initialize(const FloatingRateCoupon& coupon) {
-        coupon_ =  dynamic_cast<const IborCoupon*>(&coupon);
+        coupon_ = dynamic_cast<const IborCoupon*>(&coupon);
         gearing_ = coupon_->gearing();
         spread_ = coupon_->spread();
         Date paymentDate = coupon_->date();
@@ -44,7 +45,7 @@ namespace QuantLib {
 
         Date today = Settings::instance().evaluationDate();
 
-        if(paymentDate > today)
+        if (paymentDate > today)
             discount_ = rateCurve->discount(paymentDate);
         else
             discount_ = 1.0;
@@ -56,7 +57,7 @@ namespace QuantLib {
         // past or future fixing is managed in InterestRateIndex::fixing()
 
         Real swapletPrice =
-           adjustedFixing()* coupon_->accrualPeriod()* discount_;
+           adjustedFixing()* coupon_->accrualPeriod() * discount_;
         return gearing_ * swapletPrice + spreadLegValue_;
     }
 
@@ -101,22 +102,23 @@ namespace QuantLib {
             QL_REQUIRE(!capletVolatility().empty(),
                        "missing optionlet volatility");
             // not yet determined, use Black model
-            Rate fixing =
-                 blackFormula(
-                       optionType,
-                       effStrike,
-                       adjustedFixing(),
-                       std::sqrt(capletVolatility()->blackVariance(fixingDate,
-                                                                   effStrike)));
-            return fixing* coupon_->accrualPeriod()*discount_; ;
+            Real variance =
+                std::sqrt(capletVolatility()->blackVariance(fixingDate,
+                                                            effStrike));
+            Rate fixing = blackFormula(optionType,
+                                       effStrike,
+                                       adjustedFixing(),
+                                       variance);
+            return fixing * coupon_->accrualPeriod() * discount_;
         }
     }
 
-    Rate BlackIborCouponPricer::adjustedFixing() const {
+    Rate BlackIborCouponPricer::adjustedFixing(Rate fixing) const {
 
         Real adjustement = 0.0;
 
-        Rate fixing = coupon_->indexFixing();
+        if (fixing == Null<Rate>())
+            fixing = coupon_->indexFixing();
 
         if (!coupon_->isInArrears()) {
             adjustement = 0.0;
@@ -153,7 +155,8 @@ namespace QuantLib {
                              public Visitor<CappedFlooredCmsCoupon>,
                              public Visitor<DigitalIborCoupon>,
                              public Visitor<DigitalCmsCoupon>,
-                             public Visitor<RangeAccrualFloatersCoupon>{
+                             public Visitor<RangeAccrualFloatersCoupon>,
+                             public Visitor<SubPeriodsCoupon> {
           private:
             const boost::shared_ptr<FloatingRateCouponPricer> pricer_;
           public:
@@ -170,6 +173,7 @@ namespace QuantLib {
             void visit(CappedFlooredCmsCoupon& c);
             void visit(DigitalCmsCoupon& c);
             void visit(RangeAccrualFloatersCoupon& c);
+            void visit(SubPeriodsCoupon& c);
         };
 
         void PricerSetter::visit(CashFlow&) {
@@ -236,6 +240,14 @@ namespace QuantLib {
             c.setPricer(rangeAccrualPricer);
         }
 
+        void PricerSetter::visit(SubPeriodsCoupon& c) {
+            const boost::shared_ptr<SubPeriodsPricer> subPeriodsPricer =
+                boost::dynamic_pointer_cast<SubPeriodsPricer>(pricer_);
+            QL_REQUIRE(subPeriodsPricer,
+                       "pricer not compatible with sub-period coupon");
+            c.setPricer(subPeriodsPricer);
+        }
+
     }
 
     void setCouponPricer(
@@ -266,4 +278,3 @@ namespace QuantLib {
     }
 
 }
-
