@@ -2,7 +2,7 @@
 
 /*
  Copyright (C) 2000, 2001, 2002, 2003 RiskMap srl
- Copyright (C) 2003, 2004, 2005, 2006 StatPro Italia srl
+ Copyright (C) 2003, 2004, 2005, 2006, 2008 StatPro Italia srl
 
  This file is part of QuantLib, a free-software/open-source library
  for financial quantitative analysts and developers - http://quantlib.org/
@@ -59,5 +59,19 @@ namespace QuantLib {
                                         endOfMonth_);
     }
 
+    boost::shared_ptr<IborIndex> IborIndex::clone(
+                               const Handle<YieldTermStructure>& h) const {
+        return boost::shared_ptr<IborIndex>(
+                                        new IborIndex(familyName(),
+                                                      tenor(),
+                                                      fixingDays(),
+                                                      currency(),
+                                                      fixingCalendar(),
+                                                      businessDayConvention(),
+                                                      endOfMonth(),
+                                                      dayCounter(),
+                                                      h));
+    }
+
 }
 
