@@ -2,7 +2,7 @@
 
 /*
  Copyright (C) 2000, 2001, 2002, 2003 RiskMap srl
- Copyright (C) 2003, 2004, 2005, 2006, 2007 StatPro Italia srl
+ Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008 StatPro Italia srl
 
  This file is part of QuantLib, a free-software/open-source library
  for financial quantitative analysts and developers - http://quantlib.org/
@@ -56,6 +56,12 @@ namespace QuantLib {
         //@{
         Date maturityDate(const Date& valueDate) const;
         // @}
+        //! \name Other methods
+        //@{
+        //! returns a copy of itself linked to a different forecast curve
+        virtual boost::shared_ptr<IborIndex> clone(
+                                   const Handle<YieldTermStructure>& h) const;
+        // @}
       protected:
         Rate forecastFixing(const Date& fixingDate) const;
         BusinessDayConvention convention_;
