@@ -34,14 +34,27 @@ namespace QuantLib {
     : familyName_(familyName), tenor_(tenor), fixingDays_(fixingDays),
       fixingCalendar_(fixingCalendar), currency_(currency),
       dayCounter_(dayCounter) {
+        tenor_.normalize();
         registerWith(Settings::instance().evaluationDate());
         registerWith(IndexManager::instance().notifier(name()));
     }
 
     std::string InterestRateIndex::name() const {
         std::ostringstream out;
-        out << familyName_ << io::short_period(tenor_)
-            << " " << dayCounter_.name();
+        out << familyName_;
+        if (tenor_ == 1*Days) {
+            if (fixingDays_==0)
+                out << "ON";
+            else if (fixingDays_==1)
+                out << "TN";
+            else if (fixingDays_==2)
+                out << "SN";
+            else
+                out << io::short_period(tenor_);
+        } else {
+            out << io::short_period(tenor_);
+        }
+        out << " " << dayCounter_.name();
         return out.str();
     }
 
