@@ -1,11 +1,12 @@
 /* -*- mode: c++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
 /*
- Copyright (C) 2007 Ferdinando Ametrano
- Copyright (C) 2007 Chiara Fornarola
  Copyright (C) 2004 Jeff Yu
  Copyright (C) 2004 M-Dimension Consulting Inc.
  Copyright (C) 2005, 2006, 2007 StatPro Italia srl
+ Copyright (C) 2007, 2008 Ferdinando Ametrano
+ Copyright (C) 2007 Chiara Fornarola
+ Copyright (C) 2008 Simon Ibbotson
 
  This file is part of QuantLib, a free-software/open-source library
  for financial quantitative analysts and developers - http://quantlib.org/
@@ -30,8 +31,10 @@
 
 #include <ql/instrument.hpp>
 #include <ql/cashflow.hpp>
+#include <ql/cashflows/coupon.hpp>
 #include <ql/time/calendar.hpp>
 #include <ql/time/daycounter.hpp>
+#include <ql/time/schedule.hpp>
 #include <ql/types.hpp>
 #include <ql/handle.hpp>
 #include <ql/termstructures/yieldtermstructure.hpp>
@@ -39,14 +42,17 @@
 
 namespace QuantLib {
 
-     class CashFlow;
+    class CashFlow;
+
+    typedef std::vector<boost::shared_ptr<CashFlow> > Leg;
 
-     typedef std::vector<boost::shared_ptr<CashFlow> > Leg;
     //! Base bond class
-    /*! Derived classes must fill the unitialized data members.
+    /*! Derived classes must fill the uninitialized data members.
 
-        \warning Most methods assume that the cashflows are stored
-                 sorted by date, the redemption being the last one.
+        \warning Most methods assume that the cash flows are stored
+                 sorted by date, the redemption(s) being after any
+                 cash flow at the same date. In particular, if there's
+                 one single redemption, it must be the last cash flow,
 
         \ingroup instruments
 
@@ -57,24 +63,46 @@ namespace QuantLib {
     */
     class Bond : public Instrument {
       public:
+        //! constructor for amortizing or non-amortizing bonds.
+        /*! Redemptions and maturity are calculated from the coupon
+            data, if available.  Therefore, redemptions must not be
+            included in the passed cash flows.
+        */
+        Bond(Natural settlementDays,
+             const Calendar& calendar,
+             const Date& issueDate = Date(),
+             const Leg& coupons = Leg());
+
+        //! old constructor for non amortizing bonds.
+        /*! \warning The last passed cash flow must be the bond
+                     redemption. No other cash flow can have a date
+                     later than the redemption date.
+        */
         Bond(Natural settlementDays,
              const Calendar& calendar,
              Real faceAmount,
              const Date& maturityDate,
-             const Date& issueDate = Null<Date>(),
-             const Leg& leg = std::vector<boost::shared_ptr<CashFlow> >());
+             const Date& issueDate = Date(),
+             const Leg& cashflows = Leg());
+
         class arguments;
         class results;
         class engine;
+
         //! \name Inspectors
         //@{
         Natural settlementDays() const;
         const Calendar& calendar() const;
         Real faceAmount() const;
-        /*! \warning the returned vector includes the redemption as
-                     the last cash flow.
-        */
+
+        const std::vector<Real>& notionals() const;
+        virtual Real notional(Date d = Date()) const;
+
+        /*! \note returns all the cashflows, including the redemptions. */
         const Leg& cashflows() const;
+        /*! returns just the redemption flows (not interest payments) */
+        const Leg& redemptions() const;
+        /*! returns the redemption, if only one is defined */
         const boost::shared_ptr<CashFlow>& redemption() const;
 
         Date maturityDate() const;
@@ -82,8 +110,10 @@ namespace QuantLib {
 
         Date settlementDate(const Date& d = Date()) const;
         //@}
+
         //! \name Calculations
         //@{
+
         //! theoretical clean price
         /*! The default bond settlement is used for calculation.
 
@@ -95,6 +125,7 @@ namespace QuantLib {
                      advisable to use such other overload.
         */
         Real cleanPrice() const;
+
         //! theoretical dirty price
         /*! The default bond settlement is used for calculation.
 
@@ -106,6 +137,11 @@ namespace QuantLib {
                      advisable to use such other overload.
         */
         Real dirtyPrice() const;
+
+        //! theoretical settlement value
+        /*! The default bond settlement date is used for calculation. */
+        Real settlementValue() const;
+
         //! theoretical bond yield
         /*! The default bond settlement and theoretical price are used
             for calculation.
@@ -123,6 +159,7 @@ namespace QuantLib {
                         Compounding comp,
                         Frequency freq,
                         Date settlementDate = Date()) const;
+
         //! dirty price given a yield and settlement date
         /*! The default bond settlement is used if no date is given. */
         Real dirtyPrice(Rate yield,
@@ -130,6 +167,11 @@ namespace QuantLib {
                         Compounding comp,
                         Frequency freq,
                         Date settlementDate = Date()) const;
+
+        //! settlement value as a function of the clean price
+        /*! The default bond settlement date is used for calculation. */
+        Real settlementValue(Real cleanPrice) const;
+
         //! yield given a (clean) price and settlement date
         /*! The default bond settlement is used if no date is given. */
         Rate yield(Real cleanPrice,
@@ -151,6 +193,7 @@ namespace QuantLib {
                                    Compounding comp,
                                    Frequency freq,
                                    Date settlementDate = Date()) const;
+
         //! dirty price given Z-spread
         /*! Z-spread compounding, frequency, daycount are taken into account
             The default bond settlement is used if no date is given.
@@ -166,27 +209,72 @@ namespace QuantLib {
         //! accrued amount at a given date
         /*! The default bond settlement is used if no date is given. */
         virtual Real accruedAmount(Date d = Date()) const;
+
         bool isExpired() const;
         //@}
 
-        //! current coupon at a given date
-        /*! The default bond settlement is used if no date is given. */
-        Real currentCoupon(Date d = Date()) const;
+        /*! Expected next coupon: depending on (the bond and) the given date
+            the coupon can be historic, deterministic or expected in a
+            stochastic sense. When the bond settlement date is used the coupon
+            is the already-fixed not-yet-paid one.
 
-        //! previous coupon already paid at a given date
-        /*! The default bond settlement is used if no date is given. */
-        Real previousCoupon(Date d = Date()) const;
+            The current bond settlement is used if no date is given.
+        */
+        virtual Rate nextCoupon(Date d = Date()) const;
+
+        //! Previous coupon already paid at a given date
+        /*! Expected previous coupon: depending on (the bond and) the given
+            date the coupon can be historic, deterministic or expected in a
+            stochastic sense. When the bond settlement date is used the coupon
+            is the last paid one.
+
+            The current bond settlement is used if no date is given.
+        */
+        Rate previousCoupon(Date d = Date()) const;
         //@}
 
       protected:
+        void setupExpired() const;
         void setupArguments(PricingEngine::arguments*) const;
+        void fetchResults(const PricingEngine::results*) const;
+
+        /*! This method can be called by derived classes in order to
+            build redemption payments from the existing cash flows.
+            It must be called after setting up the cashflows_ vector
+            and will fill the notionalSchedule_, notionals_, and
+            redemptions_ data members.
+
+            If given, the elements of the redemptions vector will
+            multiply the amount of the redemption cash flow.  The
+            elements will be taken in base 100, i.e., a redemption
+            equal to 100 does not modify the amount.
+
+            \pre The cashflows_ vector must contain at least one
+                 coupon and must be sorted by date.
+        */
+        void addRedemptionsToCashflows(const std::vector<Real>& redemptions
+                                                       = std::vector<Real>());
+
+        void calculateNotionalsFromCashflows();
+
+        /*! This method can be called by derived classes in order to
+            build a bond with a single redemption payment.  It will
+            fill the notionalSchedule_, notionals_, and redemptions_
+            data members.
+        */
+        void setSingleRedemption(Real notional,
+                                 Real redemption,
+                                 const Date& date);
 
         Natural settlementDays_;
         Calendar calendar_;
-        Real faceAmount_;
-        Leg cashflows_;
+        std::vector<Date> notionalSchedule_;
+        std::vector<Real> notionals_;
+        Leg cashflows_; // all cashflows
+        Leg redemptions_; // the redemptions
 
         Date maturityDate_, issueDate_;
+        mutable Real settlementValue_;
     };
 
     class Bond::arguments : public PricingEngine::arguments {
@@ -197,7 +285,14 @@ namespace QuantLib {
         void validate() const;
     };
 
-    class Bond::results : public Instrument::results {};
+    class Bond::results : public Instrument::results {
+      public:
+        Real settlementValue;
+        void reset() {
+            settlementValue = Null<Real>();
+            Instrument::results::reset();
+        }
+    };
 
     class Bond::engine : public GenericEngine<Bond::arguments,
                                               Bond::results> {};
@@ -214,15 +309,19 @@ namespace QuantLib {
     }
 
     inline Real Bond::faceAmount() const {
-        return faceAmount_;
+        return notionals_.front();
+    }
+
+    inline const std::vector<Real>& Bond::notionals() const {
+        return notionals_;
     }
 
     inline const Leg& Bond::cashflows() const {
         return cashflows_;
     }
 
-    inline const boost::shared_ptr<CashFlow>& Bond::redemption() const {
-        return cashflows_.back();
+    inline const Leg& Bond::redemptions() const {
+        return redemptions_;
     }
 
     inline Date Bond::maturityDate() const {
