@@ -1,7 +1,7 @@
 /* -*- mode: c++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
 /*
- Copyright (C) 2007 Ferdinando Ametrano
+ Copyright (C) 2007, 2008 Ferdinando Ametrano
  Copyright (C) 2004 Jeff Yu
  Copyright (C) 2004 M-Dimension Consulting Inc.
  Copyright (C) 2005, 2006, 2007 StatPro Italia srl
@@ -35,20 +35,21 @@ namespace QuantLib {
                                  BusinessDayConvention paymentConvention,
                                  Real redemption,
                                  const Date& issueDate)
-    : Bond(settlementDays, schedule.calendar(), faceAmount,
-           schedule.endDate(), issueDate) {
+    : Bond(settlementDays, schedule.calendar(), issueDate),
+      frequency_(schedule.tenor().frequency()),
+      dayCounter_(accrualDayCounter) {
 
-        cashflows_ = FixedRateLeg(schedule,accrualDayCounter)
-            .withNotionals(faceAmount_)
+        maturityDate_ = schedule.endDate();
+
+        cashflows_ = FixedRateLeg(schedule, accrualDayCounter)
+            .withNotionals(faceAmount)
             .withCouponRates(coupons)
             .withPaymentAdjustment(paymentConvention);
 
-        Date redemptionDate = calendar_.adjust(maturityDate_,
-                                               paymentConvention);
-        cashflows_.push_back(boost::shared_ptr<CashFlow>(new
-            SimpleCashFlow(faceAmount_*redemption/100.0, redemptionDate)));
+        addRedemptionsToCashflows(std::vector<Real>(1, redemption));
 
         QL_ENSURE(!cashflows().empty(), "bond with no cashflows!");
+        QL_ENSURE(redemptions_.size() == 1, "multiple redemptions created");
     }
 
     FixedRateBond::FixedRateBond(Natural settlementDays,
@@ -66,9 +67,10 @@ namespace QuantLib {
                                  const Date& stubDate,
                                  DateGeneration::Rule rule,
                                  bool endOfMonth)
-    : Bond(settlementDays, calendar, faceAmount, maturityDate, issueDate) {
+    : Bond(settlementDays, calendar, issueDate),
+      frequency_(tenor.frequency()), dayCounter_(accrualDayCounter) {
 
-        maturityDate_     = maturityDate;
+        maturityDate_ = maturityDate;
 
         Date firstDate, nextToLastDate;
         switch (rule) {
@@ -82,6 +84,8 @@ namespace QuantLib {
             break;
           case DateGeneration::Zero:
           case DateGeneration::ThirdWednesday:
+          case DateGeneration::Twentieth:
+          case DateGeneration::TwentiethIMM:
             QL_FAIL("stub date (" << stubDate << ") not allowed with " <<
                     rule << " DateGeneration::Rule");
           default:
@@ -94,16 +98,14 @@ namespace QuantLib {
                           firstDate, nextToLastDate);
 
         cashflows_ = FixedRateLeg(schedule, accrualDayCounter)
-            .withNotionals(faceAmount_)
+            .withNotionals(faceAmount)
             .withCouponRates(coupons)
             .withPaymentAdjustment(paymentConvention);
 
-        Date redemptionDate = calendar_.adjust(maturityDate_,
-                                               paymentConvention);
-        cashflows_.push_back(boost::shared_ptr<CashFlow>(new
-            SimpleCashFlow(faceAmount_*redemption/100.0, redemptionDate)));
+        addRedemptionsToCashflows(std::vector<Real>(1, redemption));
 
         QL_ENSURE(!cashflows().empty(), "bond with no cashflows!");
+        QL_ENSURE(redemptions_.size() == 1, "multiple redemptions created");
     }
 
 }
