@@ -20,12 +20,14 @@
  FOR A PARTICULAR PURPOSE.  See the license for more details.
 */
 
+#include <ql/instruments/capfloor.hpp>
 #include <ql/pricingengines/capfloor/blackcapfloorengine.hpp>
-#include <ql/math/solvers1d/brent.hpp>
-#include <ql/cashflows/cashflows.hpp>
-#include <ql/termstructures/yieldtermstructure.hpp>
+#include <ql/math/solvers1d/newtonsafe.hpp>
 #include <ql/quotes/simplequote.hpp>
+#include <ql/cashflows/cashflows.hpp>
+#include <ql/utilities/dataformatters.hpp>
 
+using boost::shared_ptr;
 
 namespace QuantLib {
 
@@ -34,7 +36,7 @@ namespace QuantLib {
         class ImpliedVolHelper {
           public:
             ImpliedVolHelper(const CapFloor&,
-                             const Handle<YieldTermStructure>&,
+                             const Handle<YieldTermStructure>& discountCurve,
                              Real targetValue);
             Real operator()(Volatility x) const;
             Real derivative(Volatility x) const;
@@ -52,10 +54,10 @@ namespace QuantLib {
                               Real targetValue)
         : discountCurve_(discountCurve), targetValue_(targetValue) {
 
-            vol_ = boost::shared_ptr<SimpleQuote>(new SimpleQuote(0.0));
+            vol_ = boost::shared_ptr<SimpleQuote>(new SimpleQuote(-1.0));
             Handle<Quote> h(vol_);
-            engine_ = boost::shared_ptr<PricingEngine>(
-                                  new BlackCapFloorEngine(discountCurve_, h));
+            engine_ = boost::shared_ptr<PricingEngine>(new
+                                    BlackCapFloorEngine(discountCurve_, h));
             cap.setupArguments(engine_->getArguments());
 
             results_ =
@@ -63,26 +65,44 @@ namespace QuantLib {
         }
 
         Real ImpliedVolHelper::operator()(Volatility x) const {
-            vol_->setValue(x);
-            engine_->calculate();
+            if (x!=vol_->value()) {
+                vol_->setValue(x);
+                engine_->calculate();
+            }
             return results_->value-targetValue_;
         }
 
         Real ImpliedVolHelper::derivative(Volatility x) const {
-            vol_->setValue(x);
-            engine_->calculate();
-            return 0.0;
-            //return results_->vega;
+            if (x!=vol_->value()) {
+                vol_->setValue(x);
+                engine_->calculate();
+            }
+            std::map<std::string,boost::any>::const_iterator vega_ =
+                results_->additionalResults.find("vega");
+            QL_REQUIRE(vega_ != results_->additionalResults.end(),
+                       "vega not provided");
+            return boost::any_cast<Real>(vega_->second);
         }
-
     }
 
+    std::ostream& operator<<(std::ostream& out,
+                             CapFloor::Type t) {
+        switch (t) {
+          case CapFloor::Cap:
+            return out << "Cap";
+          case CapFloor::Floor:
+            return out << "Floor";
+          case CapFloor::Collar:
+            return out << "Collar";
+          default:
+            QL_FAIL("unknown CapFloor::Type (" << Integer(t) << ")");
+        }
+    }
 
-    CapFloor::CapFloor(
-                 CapFloor::Type type,
-                 const Leg& floatingLeg,
-                 const std::vector<Rate>& capRates,
-                 const std::vector<Rate>& floorRates)
+    CapFloor::CapFloor(CapFloor::Type type,
+                       const Leg& floatingLeg,
+                       const std::vector<Rate>& capRates,
+                       const std::vector<Rate>& floorRates)
     : type_(type), floatingLeg_(floatingLeg),
       capRates_(capRates), floorRates_(floorRates) {
         if (type_ == Cap || type_ == Collar) {
@@ -104,10 +124,9 @@ namespace QuantLib {
         registerWith(Settings::instance().evaluationDate());
     }
 
-    CapFloor::CapFloor(
-                 CapFloor::Type type,
-                 const Leg& floatingLeg,
-                 const std::vector<Rate>& strikes)
+    CapFloor::CapFloor(CapFloor::Type type,
+                       const Leg& floatingLeg,
+                       const std::vector<Rate>& strikes)
     : type_(type), floatingLeg_(floatingLeg) {
         QL_REQUIRE(!strikes.empty(), "no strikes given");
         if (type_ == Cap) {
@@ -130,13 +149,9 @@ namespace QuantLib {
         registerWith(Settings::instance().evaluationDate());
     }
 
-    Rate CapFloor::atmRate(const YieldTermStructure& discountCurve) const {
-        return CashFlows::atmRate(floatingLeg_, discountCurve);
-    }
-
     bool CapFloor::isExpired() const {
         Date today = Settings::instance().evaluationDate();
-        for (Size i=0; i<floatingLeg_.size(); i++)
+        for (Size i=0; i<floatingLeg_.size(); ++i)
             if (!floatingLeg_[i]->hasOccurred(today))
                 return false;
         return true;
@@ -150,14 +165,29 @@ namespace QuantLib {
         return CashFlows::maturityDate(floatingLeg_);
     }
 
-    boost::shared_ptr<FloatingRateCoupon>
+    shared_ptr<FloatingRateCoupon>
     CapFloor::lastFloatingRateCoupon() const {
-        boost::shared_ptr<CashFlow> lastCF(floatingLeg_.back());
-        boost::shared_ptr<FloatingRateCoupon> lastFloatingCoupon =
+        shared_ptr<CashFlow> lastCF(floatingLeg_.back());
+        shared_ptr<FloatingRateCoupon> lastFloatingCoupon =
             boost::dynamic_pointer_cast<FloatingRateCoupon>(lastCF);
         return lastFloatingCoupon;
     }
 
+    shared_ptr<CapFloor> CapFloor::optionlet(const Size i) const {
+        QL_REQUIRE(i < floatingLeg().size(),
+                   io::ordinal(i+1) << " optionlet does not exist, only " <<
+                   floatingLeg().size());
+        Leg cf(1, floatingLeg()[i]);
+
+        std::vector<Rate> cap, floor;
+        if (type() == Cap || type() == Collar)
+            cap.push_back(capRates()[i]);
+        if (type() == Floor || type() == Collar)
+            floor.push_back(floorRates()[i]);
+
+        return shared_ptr<CapFloor>(new CapFloor(type(), cf, cap, floor));
+    }
+
     void CapFloor::setupArguments(PricingEngine::arguments* args) const {
         CapFloor::arguments* arguments =
             dynamic_cast<CapFloor::arguments*>(args);
@@ -180,11 +210,11 @@ namespace QuantLib {
 
         Date today = Settings::instance().evaluationDate();
 
-        for (Size i=0; i<n; i++) {
-            boost::shared_ptr<FloatingRateCoupon> coupon =
+        for (Size i=0; i<n; ++i) {
+            shared_ptr<FloatingRateCoupon> coupon =
                 boost::dynamic_pointer_cast<FloatingRateCoupon>(
                                                              floatingLeg_[i]);
-            QL_REQUIRE(coupon, "non-iborCoupon given");
+            QL_REQUIRE(coupon, "non-FloatingRateCoupon given");
             arguments->startDates[i] = coupon->accrualStartDate();
             arguments->fixingDates[i] = coupon->fixingDate();
             arguments->endDates[i] = coupon->date();
@@ -254,38 +284,26 @@ namespace QuantLib {
                    << forwards.size() << ")");
     }
 
+    Rate CapFloor::atmRate(const YieldTermStructure& discountCurve) const {
+        return CashFlows::atmRate(floatingLeg_, discountCurve);
+    }
+
     Volatility CapFloor::impliedVolatility(
                               Real targetValue,
                               const Handle<YieldTermStructure>& discountCurve,
+                              Volatility guess,
                               Real accuracy,
-                              Size maxEvaluations,
+                              Natural maxEvaluations,
                               Volatility minVol,
                               Volatility maxVol) const {
         calculate();
         QL_REQUIRE(!isExpired(), "instrument expired");
 
-        Volatility guess = 0.10;   // no way we can get a more accurate one
-
         ImpliedVolHelper f(*this, discountCurve, targetValue);
-        Brent solver;
+        //Brent solver;
+        NewtonSafe solver;
         solver.setMaxEvaluations(maxEvaluations);
         return solver.solve(f, accuracy, guess, minVol, maxVol);
     }
 
-
-
-
-    std::ostream& operator<<(std::ostream& out, CapFloor::Type t) {
-        switch (t) {
-          case CapFloor::Cap:
-            return out << "Cap";
-          case CapFloor::Floor:
-            return out << "Floor";
-          case CapFloor::Collar:
-            return out << "Collar";
-          default:
-            QL_FAIL("unknown CapFloor::Type (" << Integer(t) << ")");
-        }
-    }
-
 }
