@@ -1,7 +1,7 @@
 /* -*- mode: c++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
 /*
- Copyright (C) 2006 Ferdinando Ametrano
+ Copyright (C) 2006, 2007 Ferdinando Ametrano
  Copyright (C) 2007 StatPro Italia srl
 
  This file is part of QuantLib, a free-software/open-source library
@@ -38,7 +38,7 @@ namespace QuantLib {
       public:
         MakeCapFloor(CapFloor::Type capFloorType,
                      const Period& capFloorTenor,
-                     const boost::shared_ptr<IborIndex>& index,
+                     const boost::shared_ptr<IborIndex>& iborIndex,
                      Rate strike = Null<Rate>(),
                      const Period& forwardStart = 0*Days);
 
@@ -48,20 +48,16 @@ namespace QuantLib {
         MakeCapFloor& withNominal(Real n);
         MakeCapFloor& withEffectiveDate(const Date& effectiveDate,
                                         bool firstCapletExcluded);
-
         MakeCapFloor& withTenor(const Period& t);
         MakeCapFloor& withCalendar(const Calendar& cal);
         MakeCapFloor& withConvention(BusinessDayConvention bdc);
         MakeCapFloor& withTerminationDateConvention(BusinessDayConvention bdc);
-        MakeCapFloor& withRule(DateGeneration::Rule r =
-                                                     DateGeneration::Forward);
+        MakeCapFloor& withRule(DateGeneration::Rule r);
         MakeCapFloor& withEndOfMonth(bool flag = true);
         MakeCapFloor& withFirstDate(const Date& d);
         MakeCapFloor& withNextToLastDate(const Date& d);
         MakeCapFloor& withDayCount(const DayCounter& dc);
 
-        MakeCapFloor& withDiscountingTermStructure(
-                  const Handle<YieldTermStructure>& discountingTermStructure);
         MakeCapFloor& withPricingEngine(
                               const boost::shared_ptr<PricingEngine>& engine);
       private:
@@ -69,9 +65,9 @@ namespace QuantLib {
         Rate strike_;
         bool firstCapletExcluded_;
 
-        Handle<YieldTermStructure> discountCurve_;
-        boost::shared_ptr<PricingEngine> engine_;
         MakeVanillaSwap makeVanillaSwap_;
+
+        boost::shared_ptr<PricingEngine> engine_;
     };
 
 }
