@@ -21,22 +21,77 @@
 */
 
 #include <ql/instruments/swaption.hpp>
-#include <ql/math/solvers1d/brent.hpp>
 #include <ql/pricingengines/swaption/blackswaptionengine.hpp>
-#include <ql/termstructures/yieldtermstructure.hpp>
+//#include <ql/math/solvers1d/brent.hpp>
+#include <ql/math/solvers1d/newtonsafe.hpp>
+#include <ql/quotes/simplequote.hpp>
 #include <ql/exercise.hpp>
 
 namespace QuantLib {
 
+    namespace {
+
+        class ImpliedVolHelper {
+          public:
+            ImpliedVolHelper(const Swaption&,
+                             const Handle<YieldTermStructure>& discountCurve,
+                             Real targetValue);
+            Real operator()(Volatility x) const;
+            Real derivative(Volatility x) const;
+          private:
+            boost::shared_ptr<PricingEngine> engine_;
+            Handle<YieldTermStructure> discountCurve_;
+            Real targetValue_;
+            boost::shared_ptr<SimpleQuote> vol_;
+            const Instrument::results* results_;
+        };
+
+        ImpliedVolHelper::ImpliedVolHelper(
+                              const Swaption& swaption,
+                              const Handle<YieldTermStructure>& discountCurve,
+                              Real targetValue)
+        : discountCurve_(discountCurve), targetValue_(targetValue) {
+
+            vol_ = boost::shared_ptr<SimpleQuote>(new SimpleQuote(-1.0));
+            Handle<Quote> h(vol_);
+            engine_ = boost::shared_ptr<PricingEngine>(new
+                                    BlackSwaptionEngine(discountCurve_, h));
+            swaption.setupArguments(engine_->getArguments());
+
+            results_ =
+                dynamic_cast<const Instrument::results*>(engine_->getResults());
+        }
+
+        Real ImpliedVolHelper::operator()(Volatility x) const {
+            if (x!=vol_->value()) {
+                vol_->setValue(x);
+                engine_->calculate();
+            }
+            return results_->value-targetValue_;
+        }
+
+        Real ImpliedVolHelper::derivative(Volatility x) const {
+            if (x!=vol_->value()) {
+                vol_->setValue(x);
+                engine_->calculate();
+            }
+            std::map<std::string,boost::any>::const_iterator vega_ =
+                results_->additionalResults.find("vega");
+            QL_REQUIRE(vega_ != results_->additionalResults.end(),
+                       "vega not provided");
+            return boost::any_cast<Real>(vega_->second);
+        }
+    }
+
     std::ostream& operator<<(std::ostream& out,
-                             Settlement::Type type) {
-        switch (type) {
+                             Settlement::Type t) {
+        switch (t) {
           case Settlement::Physical:
-            return out << "delivery";
+            return out << "Delivery";
           case Settlement::Cash:
-            return out << "cash";
+            return out << "Cash";
           default:
-            QL_FAIL("unknown settlement type");
+            QL_FAIL("unknown Settlement::Type(" << Integer(t) << ")");
         }
     }
 
@@ -71,62 +126,24 @@ namespace QuantLib {
         VanillaSwap::arguments::validate();
         QL_REQUIRE(swap, "vanilla swap not set");
         QL_REQUIRE(exercise, "exercise not set");
-        //QL_REQUIRE(fixedRate != Null<Real>(),
-        //           "fixed swap rate null or not set");
-        //QL_REQUIRE(fairRate != Null<Real>(),
-        //           "fair swap rate null or not set");
-        //QL_REQUIRE(fixedBPS != Null<Real>(),
-        //           "fixed swap BPS null or not set");
-        //QL_REQUIRE(forecastingDiscount != Null<Real>(),
-        //           "forecasting discount null or not set");
-        //if(settlementType == Settlement::Cash) {
-        //    QL_REQUIRE(fixedCashBPS != Null<Real>(),
-        //               "fixed swap cash BPS null or not set "
-        //               "for cash-settled swaption");
-        //}
     }
 
     Volatility Swaption::impliedVolatility(
                               Real targetValue,
-                              const Handle<YieldTermStructure>& termStructure,
+                              const Handle<YieldTermStructure>& discountCurve,
+                              Volatility guess,
                               Real accuracy,
-                              Size maxEvaluations,
+                              Natural maxEvaluations,
                               Volatility minVol,
                               Volatility maxVol) const {
         calculate();
         QL_REQUIRE(!isExpired(), "instrument expired");
 
-        Volatility guess = 0.10; // improve
-
-        ImpliedVolHelper f(*this, termStructure, targetValue);
-        Brent solver;
+        ImpliedVolHelper f(*this, discountCurve, targetValue);
+        //Brent solver;
+        NewtonSafe solver;
         solver.setMaxEvaluations(maxEvaluations);
         return solver.solve(f, accuracy, guess, minVol, maxVol);
     }
 
-    Swaption::ImpliedVolHelper::ImpliedVolHelper(
-                              const Swaption& swaption,
-                              const Handle<YieldTermStructure>& termStructure,
-                              Real targetValue)
-    : termStructure_(termStructure), targetValue_(targetValue) {
-
-        vol_ = boost::shared_ptr<SimpleQuote>(new SimpleQuote(0.0));
-        Handle<Quote> h(vol_);
-        engine_ = boost::shared_ptr<PricingEngine>(new
-                                        BlackSwaptionEngine(termStructure, h));
-        swaption.setupArguments(engine_->getArguments());
-
-        results_ =
-            dynamic_cast<const Instrument::results*>(engine_->getResults());
-    }
-
-    Real Swaption::ImpliedVolHelper::operator()(Volatility x) const {
-        vol_->setValue(x);
-        engine_->calculate();
-        return results_->value-targetValue_;
-    }
-
-    Rate Swaption::atmRate() const{
-        return swap_->fairRate();
-    }
 }
