@@ -26,7 +26,6 @@
 #include <ql/cashflows/cashflows.hpp>
 #include <ql/cashflows/couponpricer.hpp>
 #include <ql/indexes/iborindex.hpp>
-#include <ql/time/schedule.hpp>
 #include <ql/termstructures/yieldtermstructure.hpp>
 
 namespace QuantLib {
@@ -37,27 +36,28 @@ namespace QuantLib {
                              Rate fixedRate,
                              const DayCounter& fixedDayCount,
                              const Schedule& floatSchedule,
-                             const boost::shared_ptr<IborIndex>& index,
+                             const boost::shared_ptr<IborIndex>& iborIndex,
                              Spread spread,
-                             const DayCounter& floatingDayCount)
-    : Swap(Leg(), Leg()),
-      type_(type), fixedRate_(fixedRate), spread_(spread),
-      nominal_(nominal) {
+                             const DayCounter& floatingDayCount,
+                             BusinessDayConvention paymentConvention)
+    : Swap(2), type_(type), nominal_(nominal),
+      fixedSchedule_(fixedSchedule), fixedRate_(fixedRate),
+      fixedDayCount_(fixedDayCount),
+      floatingSchedule_(floatSchedule), iborIndex_(iborIndex), spread_(spread),
+      floatingDayCount_(floatingDayCount),
+      paymentConvention_(paymentConvention) {
 
-        BusinessDayConvention convention =
-            floatSchedule.businessDayConvention();
+        Leg fixedLeg = FixedRateLeg(fixedSchedule_, fixedDayCount_)
+            .withNotionals(nominal_)
+            .withCouponRates(fixedRate_)
+            .withPaymentAdjustment(paymentConvention_);
 
-        Leg fixedLeg = FixedRateLeg(fixedSchedule,fixedDayCount)
-            .withNotionals(nominal)
-            .withCouponRates(fixedRate)
-            .withPaymentAdjustment(convention);
-
-        Leg floatingLeg = IborLeg(floatSchedule, index)
-            .withNotionals(nominal)
-            .withPaymentDayCounter(floatingDayCount)
-            .withPaymentAdjustment(convention)
-            .withFixingDays(index->fixingDays())
-            .withSpreads(spread);
+        Leg floatingLeg = IborLeg(floatingSchedule_, iborIndex_)
+            .withNotionals(nominal_)
+            .withPaymentDayCounter(floatingDayCount_)
+            .withPaymentAdjustment(paymentConvention_)
+            //.withFixingDays(iborIndex->fixingDays())
+            .withSpreads(spread_);
 
         Leg::const_iterator i;
         for (i = floatingLeg.begin(); i < floatingLeg.end(); ++i)
@@ -66,11 +66,11 @@ namespace QuantLib {
         legs_[0] = fixedLeg;
         legs_[1] = floatingLeg;
         if (type_==Payer) {
-            payer_[0]=-1.0;
-            payer_[1]=+1.0;
+            payer_[0] = -1.0;
+            payer_[1] = +1.0;
         } else {
-            payer_[0]=+1.0;
-            payer_[1]=-1.0;
+            payer_[0] = +1.0;
+            payer_[1] = -1.0;
         }
     }
 
@@ -232,4 +232,16 @@ namespace QuantLib {
         fairSpread = Null<Spread>();
     }
 
+    std::ostream& operator<<(std::ostream& out,
+                             VanillaSwap::Type t) {
+        switch (t) {
+          case VanillaSwap::Payer:
+            return out << "Payer";
+          case VanillaSwap::Receiver:
+            return out << "Receiver";
+          default:
+            QL_FAIL("unknown VanillaSwap::Type(" << Integer(t) << ")");
+        }
+    }
+
 }
