@@ -1,7 +1,7 @@
 /* -*- mode: c++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
 /*
- Copyright (C) 2006 Ferdinando Ametrano
+ Copyright (C) 2006, 2008 Ferdinando Ametrano
  Copyright (C) 2000, 2001, 2002, 2003 RiskMap srl
  Copyright (C) 2003, 2004, 2005, 2006, 2007 StatPro Italia srl
 
@@ -28,6 +28,7 @@
 
 #include <ql/instruments/swap.hpp>
 #include <ql/time/daycounter.hpp>
+#include <ql/time/schedule.hpp>
 
 namespace QuantLib {
 
@@ -37,6 +38,13 @@ namespace QuantLib {
     //! Plain-vanilla swap
     /*! \ingroup instruments
 
+        \warning if QL_TODAYS_PAYMENTS was defined (in userconfig.hpp
+                 or when calling ./configure; it is undefined by
+                 default) payments occurring at the settlement date of
+                 the swap are included in the NPV, and therefore
+                 affect the fair-rate and fair-spread
+                 calculation. This might not be what you want.
+
         \test
         - the correctness of the returned value is tested by checking
           that the price of a swap paying the fair fixed rate is null.
@@ -63,9 +71,10 @@ namespace QuantLib {
                     Rate fixedRate,
                     const DayCounter& fixedDayCount,
                     const Schedule& floatSchedule,
-                    const boost::shared_ptr<IborIndex>& index,
+                    const boost::shared_ptr<IborIndex>& iborIndex,
                     Spread spread,
-                    const DayCounter& floatingDayCount);
+                    const DayCounter& floatingDayCount,
+                    BusinessDayConvention paymentConvention = Following);
         // results
         Real fixedLegBPS() const;
         Real fixedLegNPV() const;
@@ -75,25 +84,37 @@ namespace QuantLib {
         Real floatingLegNPV() const;
         Spread fairSpread() const;
         // inspectors
-        Rate fixedRate() const;
-        Spread spread() const;
-        Real nominal() const;
         Type type() const;
-        const Leg& fixedLeg() const {
-            return legs_[0];
-        }
-        const Leg& floatingLeg() const {
-            return legs_[1];
-        }
+        Real nominal() const;
+
+        const Schedule& fixedSchedule() const;
+        Rate fixedRate() const;
+        const DayCounter& fixedDayCount() const;
+
+        const Schedule& floatingSchedule() const;
+        const boost::shared_ptr<IborIndex>& iborIndex() const;
+        Spread spread() const;
+        const DayCounter& floatingDayCount() const;
+
+        BusinessDayConvention paymentConvention() const;
+
+        const Leg& fixedLeg() const;
+        const Leg& floatingLeg() const;
         // other
         void setupArguments(PricingEngine::arguments* args) const;
         void fetchResults(const PricingEngine::results*) const;
       private:
         void setupExpired() const;
         Type type_;
-        Rate fixedRate_;
-        Spread spread_;
         Real nominal_;
+        Schedule fixedSchedule_;
+        Rate fixedRate_;
+        DayCounter fixedDayCount_;
+        Schedule floatingSchedule_;
+        boost::shared_ptr<IborIndex> iborIndex_;
+        Spread spread_;
+        DayCounter floatingDayCount_;
+        BusinessDayConvention paymentConvention_;
         // results
         mutable Rate fairRate_;
         mutable Spread fairSpread_;
@@ -135,33 +156,57 @@ namespace QuantLib {
 
     // inline definitions
 
-    inline Rate VanillaSwap::fixedRate() const {
-        return fixedRate_;
-    }
-
-    inline Spread VanillaSwap::spread() const {
-        return spread_;
+    inline VanillaSwap::Type VanillaSwap::type() const {
+        return type_;
     }
 
     inline Real VanillaSwap::nominal() const {
         return nominal_;
     }
 
-    inline VanillaSwap::Type VanillaSwap::type() const {
-        return type_;
+    inline const Schedule& VanillaSwap::fixedSchedule() const {
+        return fixedSchedule_;
     }
 
-    inline std::ostream& operator<<(std::ostream& out, VanillaSwap::Type type) {
-        switch (type) {
-          case VanillaSwap::Payer:
-            return out << "payer";
-          case VanillaSwap::Receiver:
-            return out << "receiver";
-          default:
-            QL_FAIL("unknown option type");
-        }
+    inline Rate VanillaSwap::fixedRate() const {
+        return fixedRate_;
     }
 
+    inline const DayCounter& VanillaSwap::fixedDayCount() const {
+        return fixedDayCount_;
+    }
+
+    inline const Schedule& VanillaSwap::floatingSchedule() const {
+        return floatingSchedule_;
+    }
+
+    inline const boost::shared_ptr<IborIndex>& VanillaSwap::iborIndex() const {
+        return iborIndex_;
+    }
+
+    inline Spread VanillaSwap::spread() const {
+        return spread_;
+    }
+
+    inline const DayCounter& VanillaSwap::floatingDayCount() const {
+        return floatingDayCount_;
+    }
+
+    inline BusinessDayConvention VanillaSwap::paymentConvention() const {
+        return paymentConvention_;
+    }
+
+    inline const Leg& VanillaSwap::fixedLeg() const {
+        return legs_[0];
+    }
+
+    inline const Leg& VanillaSwap::floatingLeg() const {
+        return legs_[1];
+    }
+
+    std::ostream& operator<<(std::ostream& out,
+                             VanillaSwap::Type t);
+
 }
 
 #endif
