@@ -1,7 +1,7 @@
 /* -*- mode: c++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
 /*
- Copyright (C) 2007 Klaus Spanderen
+ Copyright (C) 2007, 2008 Klaus Spanderen
 
  This file is part of QuantLib, a free-software/open-source library
  for financial quantitative analysts and developers - http://quantlib.org/
@@ -22,19 +22,17 @@
 */
 
 #include <ql/math/matrix.hpp>
-#if defined(QL_PATCH_MSVC71) || defined(QL_PATCH_MSVC80)
+#if defined(QL_PATCH_MSVC)
 #pragma warning(push)
 #pragma warning(disable:4180)
 #endif
-#if !defined(__GNUC__) || __GNUC__ > 3 || __GNUC_MINOR__ > 4
-#define QL_MATRIX_BLAS
-#endif
 
-#if defined(QL_MATRIX_BLAS)
+#if !defined(QL_NO_UBLAS_SUPPORT)
 #include <boost/numeric/ublas/vector_proxy.hpp>
 #include <boost/numeric/ublas/triangular.hpp>
 #include <boost/numeric/ublas/lu.hpp>
 #endif
+
 #if defined(QL_PATCH_MSVC71) || defined(QL_PATCH_MSVC80)
 #pragma warning(pop)
 #endif
@@ -42,7 +40,7 @@
 namespace QuantLib {
 
     Disposable<Matrix> inverse(const Matrix& m) {
-        #if defined(QL_MATRIX_BLAS)
+        #if !defined(QL_NO_UBLAS_SUPPORT)
 
         QL_REQUIRE(m.rows() == m.columns(), "matrix is not square");
 
@@ -69,8 +67,36 @@ namespace QuantLib {
         return retVal;
 
         #else
-        QL_FAIL("this version of gcc does not support the Boost uBlas library");
+        QL_FAIL("this version of gcc does not support "
+                "the Boost uBLAS library");
         #endif
     }
 
+    Real determinant(const Matrix& m) {
+        #if !defined(QL_NO_UBLAS_SUPPORT)
+        QL_REQUIRE(m.rows() == m.columns(), "matrix is not square");
+
+        boost::numeric::ublas::matrix<Real> a(m.rows(), m.columns());
+        std::copy(m.begin(), m.end(), a.data().begin());
+
+
+        // lu decomposition
+        boost::numeric::ublas::permutation_matrix<Size> pert(m.rows());
+        /* const Size singular = */ lu_factorize(a, pert);
+
+        Real retVal = 1.0;
+
+        for (Size i=0; i < m.rows(); ++i) {
+            if (pert[i] != i)
+                retVal *= -a(i,i);
+            else
+                retVal *=  a(i,i);
+        }
+        return retVal;
+
+        #else
+        QL_FAIL("this version of gcc does not support "
+                "the Boost uBLAS library");
+        #endif
+    }
 }
