@@ -26,11 +26,15 @@
 
 #include <ql/models/calibrationhelper.hpp>
 #include <ql/instruments/capfloor.hpp>
-#include <ql/indexes/iborindex.hpp>
 
 namespace QuantLib {
 
     //! calibration helper for ATM cap
+    /*! \bug This helper does not register with the passed IBOR index
+             and with the evaluation date. Furthermore, the ATM strike
+             rate is not recalculated when any of its observables
+             change.
+    */
     class CapHelper : public CalibrationHelper {
       public:
         CapHelper(const Period& length,
@@ -42,18 +46,13 @@ namespace QuantLib {
                   bool includeFirstSwaplet,
                   const Handle<YieldTermStructure>& termStructure,
                   bool calibrateVolatility = false);
-
         virtual void addTimesTo(std::list<Time>& times) const;
-
         virtual Real modelValue() const;
-
         virtual Real blackPrice(Volatility volatility) const;
-
       private:
         boost::shared_ptr<Cap> cap_;
     };
 
 }
 
-
 #endif
