@@ -1,12 +1,12 @@
 /* -*- mode: c++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
 /*
+ Copyright (C) 2001, 2002, 2003 Sadruddin Rejeb
+ Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008 Ferdinando Ametrano
+ Copyright (C) 2006 Mark Joshi
+ Copyright (C) 2006 StatPro Italia srl
  Copyright (C) 2007 Cristina Duminuco
  Copyright (C) 2007 Chiara Fornarola
- Copyright (C) 2003, 2004, 2005, 2006, 2007 Ferdinando Ametrano
- Copyright (C) 2006 Mark Joshi
- Copyright (C) 2001, 2002, 2003 Sadruddin Rejeb
- Copyright (C) 2006 StatPro Italia srl
 
  This file is part of QuantLib, a free-software/open-source library
  for financial quantitative analysts and developers - http://quantlib.org/
@@ -62,7 +62,7 @@ namespace QuantLib {
         strike = strike + displacement;
 
         // since displacement is non-negative strike==0 iff displacement==0
-        // so returning forward*discount is OK 
+        // so returning forward*discount is OK
         if (strike==0.0)
             return (optionType==Option::Call ? forward*discount : 0.0);
 
@@ -191,15 +191,16 @@ namespace QuantLib {
         CumulativeNormalDistribution N_;
     };
 
-   
+
     Real blackFormulaImpliedStdDev(Option::Type optionType,
                                    Real strike,
                                    Real forward,
                                    Real blackPrice,
                                    Real discount,
+                                   Real displacement,
                                    Real guess,
                                    Real accuracy,
-                                   Real displacement)
+                                   Natural maxIterations)
     {
         checkParameters(strike, forward, displacement);
         QL_REQUIRE(blackPrice>=0.0,
@@ -218,7 +219,7 @@ namespace QuantLib {
         BlackImpliedStdDevHelper f(optionType, strike, forward,
                                    blackPrice/discount);
         NewtonSafe solver;
-        solver.setMaxEvaluations(100);
+        solver.setMaxEvaluations(maxIterations);
         Real minSdtDev = 0.0, maxStdDev = 3.0;
         Real stdDev = solver.solve(f, accuracy, guess, minSdtDev, maxStdDev);
         QL_ENSURE(stdDev>=0.0,
@@ -231,11 +232,12 @@ namespace QuantLib {
                         Real forward,
                         Real blackPrice,
                         Real discount,
+                        Real displacement,
                         Real guess,
                         Real accuracy,
-                        Real displacement) {
+                        Natural maxIterations) {
         return blackFormulaImpliedStdDev(payoff->optionType(), payoff->strike(),
-            forward, blackPrice, discount, guess, accuracy, displacement);
+            forward, blackPrice, discount, displacement, guess, accuracy, maxIterations);
     }
 
     Real blackFormulaCashItmProbability(Option::Type optionType,
@@ -256,7 +258,7 @@ namespace QuantLib {
         return phi(optionType*d2);
     }
 
-   
+
     Real blackFormulaCashItmProbability(
                         const boost::shared_ptr<PlainVanillaPayoff>& payoff,
                         Real forward,
@@ -267,6 +269,20 @@ namespace QuantLib {
     }
 
 
+    Real blackFormulaVolDerivative(Rate strike,
+                                      Rate forward,
+                                      Real stdDev,
+                                      Real expiry,
+                                      Real discount,
+                                      Real displacement)
+    {
+        return  blackFormulaStdDevDerivative(strike,
+                                     forward,
+                                     stdDev,
+                                     discount,
+                                     displacement)*sqrt(expiry);
+    }
+
     Real blackFormulaStdDevDerivative(Rate strike,
                                       Rate forward,
                                       Real stdDev,
@@ -282,6 +298,13 @@ namespace QuantLib {
         forward = forward + displacement;
         strike = strike + displacement;
 
+        if (stdDev==0.0) {
+            if (forward>strike)
+                return discount * forward;
+            else
+                return 0.0;
+        }
+
         Real d1 = std::log(forward/strike)/stdDev + .5*stdDev;
         return discount * forward *
             CumulativeNormalDistribution().derivative(d1);
