@@ -1,7 +1,7 @@
 /* -*- mode: c++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
 /*
- Copyright (C) 2007 Ferdinando Ametrano
+ Copyright (C) 2007, 2008 Ferdinando Ametrano
  Copyright (C) 2006 Cristina Duminuco
  Copyright (C) 2001, 2002, 2003 Sadruddin Rejeb
  Copyright (C) 2006, 2007 StatPro Italia srl
@@ -21,26 +21,35 @@
 */
 
 #include <ql/pricingengines/swaption/blackswaptionengine.hpp>
-#include <ql/pricingengines/swap/discountingswapengine.hpp>
 #include <ql/pricingengines/blackformula.hpp>
 #include <ql/termstructures/volatility/swaption/swaptionconstantvol.hpp>
+#include <ql/time/calendars/nullcalendar.hpp>
+
+#include <ql/pricingengines/swap/discountingswapengine.hpp>
+#include <ql/indexes/iborindex.hpp>
 #include <ql/cashflows/fixedratecoupon.hpp>
 #include <ql/cashflows/cashflows.hpp>
-#include <ql/time/daycounters/actual365fixed.hpp>
-#include <ql/time/calendars/nullcalendar.hpp>
-#include <ql/termstructures/yieldtermstructure.hpp>
 #include <ql/exercise.hpp>
 
 namespace QuantLib {
 
+    BlackSwaptionEngine::BlackSwaptionEngine(
+                              const Handle<YieldTermStructure>& termStructure,
+                              Volatility vol,
+                              const DayCounter& dc)
+    : termStructure_(termStructure),
+      volatility_(boost::shared_ptr<SwaptionVolatilityStructure>(new
+          ConstantSwaptionVolatility(0, NullCalendar(), Following, vol, dc))) {
+        registerWith(termStructure_);
+    }
+
     BlackSwaptionEngine::BlackSwaptionEngine(
                             const Handle<YieldTermStructure>& termStructure,
-                            const Handle<Quote>& volatility,
+                            const Handle<Quote>& vol,
                             const DayCounter& dc)
     : termStructure_(termStructure),
       volatility_(boost::shared_ptr<SwaptionVolatilityStructure>(new
-                  SwaptionConstantVolatility(0, NullCalendar(),
-                                             volatility, dc))) {
+          ConstantSwaptionVolatility(0, NullCalendar(), Following, vol, dc))) {
         registerWith(termStructure_);
         registerWith(volatility_);
     }
@@ -57,21 +66,37 @@ namespace QuantLib {
     void BlackSwaptionEngine::calculate() const {
         static const Spread basisPoint = 1.0e-4;
 
+        Date exerciseDate = arguments_.exercise->date(0);
+
+        // the part of the swap preceding exerciseDate should be truncated
+        // to avoid taking into account unwanted cashflows
         VanillaSwap swap = *arguments_.swap;
+
+        Rate strike = swap.fixedRate();
+
+        // using the forecasting curve
+        swap.setPricingEngine(boost::shared_ptr<PricingEngine>(new
+                    DiscountingSwapEngine(swap.iborIndex()->termStructure())));
+        Rate atmForward = swap.fairRate();
+
+        // Volatilities are quoted for zero-spreaded swaps.
+        // Therefore, any spread on the floating leg must be removed
+        // with a corresponding correction on the fixed leg.
+        if (swap.spread()!=0.0) {
+            Spread correction = swap.spread() *
+                std::fabs(swap.floatingLegBPS()/swap.fixedLegBPS());
+            strike -= correction;
+            atmForward -= correction;
+            results_.additionalResults["spreadCorrection"] = correction;
+        } else {
+            results_.additionalResults["spreadCorrection"] = 0.0;
+        }
+        results_.additionalResults["strike"] = strike;
+        results_.additionalResults["atmForward"] = atmForward;
+
+        // using the discounting curve
         swap.setPricingEngine(boost::shared_ptr<PricingEngine>(new
                                     DiscountingSwapEngine(termStructure_)));
-
-        // Volatilities are calculated for zero-spreaded swaps.
-        // Therefore, the spread on the floating leg is removed
-        // and a corresponding correction is made on the fixed leg.
-        Spread correction = swap.spread() *
-            std::fabs(swap.floatingLegBPS()/swap.fixedLegBPS());
-        Rate strike = swap.fixedRate() - correction;
-        Rate forward = swap.fairRate() - correction;
-
-        Date settlement = termStructure_->referenceDate();
-        Date maturityDate = arguments_.floatingPayDates.back();
-        Date exerciseDate = arguments_.exercise->date(0);
         Real annuity;
         switch(arguments_.settlementType) {
           case Settlement::Physical: {
@@ -83,38 +108,36 @@ namespace QuantLib {
               boost::shared_ptr<FixedRateCoupon> firstCoupon =
                   boost::dynamic_pointer_cast<FixedRateCoupon>(fixedLeg[0]);
               DayCounter dayCount = firstCoupon->dayCounter();
-              Real fixedCashBPS =
+              Real fixedLegCashBPS =
                   CashFlows::bps(fixedLeg,
-                                 InterestRate(forward, dayCount, Compounded),
-                                 settlement) ;
-              annuity = fixedCashBPS/basisPoint;
+                                 InterestRate(atmForward, dayCount, Compounded),
+                                 termStructure_->referenceDate()) ;
+              annuity = std::fabs(fixedLegCashBPS/basisPoint);
               break;
           }
           default:
             QL_FAIL("unknown settlement type");
         }
+        results_.additionalResults["annuity"] = annuity;
 
-        Time exerciseTime =
-            termStructure_->dayCounter().yearFraction(settlement,
-                                                      exerciseDate);
-        Time maturityTime =
-            termStructure_->dayCounter().yearFraction(settlement,
-                                                      maturityDate);
-        Time swapLength = maturityTime - exerciseTime;
+        // the swap length calculation might be improved using the value date
+        // of the exercise date
+        Time swapLength =  volatility_->swapLength(exerciseDate,
+                                                   arguments_.floatingPayDates.back());
+        results_.additionalResults["swapLength"] = swapLength;
 
-        Volatility vol = volatility_->volatility(exerciseTime,
-                                                 swapLength,
-                                                 strike);
-        Option::Type w = (arguments_.type==VanillaSwap::Payer) ?
-                                                Option::Call : Option::Put;
-        results_.value = annuity * blackFormula(w, strike, forward,
-                                                vol*std::sqrt(exerciseTime));
-        Real variance = volatility_->blackVariance(exerciseTime,
+        Real variance = volatility_->blackVariance(exerciseDate,
                                                    swapLength,
                                                    strike);
         Real stdDev = std::sqrt(variance);
+        results_.additionalResults["stdDev"] = stdDev;
+        Option::Type w = (arguments_.type==VanillaSwap::Payer) ?
+                                                Option::Call : Option::Put;
+        results_.value = blackFormula(w, strike, atmForward, stdDev, annuity);
+
+        Time exerciseTime = volatility_->timeFromReference(exerciseDate);
         results_.additionalResults["vega"] = std::sqrt(exerciseTime) *
-            blackFormulaStdDevDerivative(strike, forward, stdDev, annuity);
+            blackFormulaStdDevDerivative(strike, atmForward, stdDev, annuity);
     }
 
     Handle<YieldTermStructure> BlackSwaptionEngine::termStructure() {
