@@ -27,6 +27,10 @@ namespace QuantLib {
                              Size integrationOrder)
     : AnalyticHestonEngine(model, integrationOrder) { }
 
+    BatesEngine::BatesEngine(const boost::shared_ptr<BatesModel>& model,
+                             Real relTolerance, Size maxEvaluations)
+    : AnalyticHestonEngine(model, relTolerance, maxEvaluations) { }
+
     std::complex<Real> BatesEngine::addOnTerm(Real phi,
                                                       Time t, Size j) const {
         boost::shared_ptr<BatesModel> batesModel =
@@ -49,6 +53,11 @@ namespace QuantLib {
         Size integrationOrder)
     : BatesEngine(model, integrationOrder) { }
 
+    BatesDetJumpEngine::BatesDetJumpEngine(
+        const boost::shared_ptr<BatesDetJumpModel>& model,
+        Real relTolerance, Size maxEvaluations)
+    : BatesEngine(model, relTolerance, maxEvaluations) { }
+
     std::complex<Real> BatesDetJumpEngine::addOnTerm(
         Real phi, Time t, Size j) const {
 
@@ -73,6 +82,10 @@ namespace QuantLib {
         Size integrationOrder)
     : AnalyticHestonEngine(model, integrationOrder) { }
 
+    BatesDoubleExpEngine::BatesDoubleExpEngine(
+        const boost::shared_ptr<BatesDoubleExpModel>& model,
+        Real relTolerance, Size maxEvaluations)
+    : AnalyticHestonEngine(model, relTolerance, maxEvaluations) { }
 
     std::complex<Real> BatesDoubleExpEngine::addOnTerm(
         Real phi, Time t, Size j) const {
@@ -96,6 +109,11 @@ namespace QuantLib {
         Size integrationOrder)
     : BatesDoubleExpEngine(model, integrationOrder) { }
 
+    BatesDoubleExpDetJumpEngine::BatesDoubleExpDetJumpEngine(
+        const boost::shared_ptr<BatesDoubleExpDetJumpModel>& model,
+        Real relTolerance, Size maxEvaluations)
+    : BatesDoubleExpEngine(model, relTolerance, maxEvaluations) { }
+
     std::complex<Real> BatesDoubleExpDetJumpEngine::addOnTerm(
         Real phi, Time t, Size j) const {
         const std::complex<Real> l =
