@@ -2,7 +2,7 @@
 
 /*
  Copyright (C) 2007 Chris Kenyon
- Copyright (C) 2007 StatPro Italia srl
+ Copyright (C) 2007, 2008 StatPro Italia srl
 
  This file is part of QuantLib, a free-software/open-source library
  for financial quantitative analysts and developers - http://quantlib.org/
@@ -26,7 +26,7 @@
 #define quantlib_piecewise_zero_inflation_curve_hpp
 
 #include <ql/termstructures/inflation/interpolatedzeroinflationcurve.hpp>
-#include <ql/termstructures/bootstrapper.hpp>
+#include <ql/termstructures/iterativebootstrap.hpp>
 #include <ql/patterns/lazyobject.hpp>
 
 namespace QuantLib {
@@ -68,14 +68,20 @@ namespace QuantLib {
 
     //! Piecewise zero-inflation term structure
     template <class Interpolator,
-              template <class,class,class> class Bootstrap = IterativeBootstrap,
+              template <class> class Bootstrap = IterativeBootstrap,
               class Traits = ZeroInflationTraits>
     class PiecewiseZeroInflationCurve
         : public InterpolatedZeroInflationCurve<Interpolator>,
           public LazyObject {
       private:
         typedef InterpolatedZeroInflationCurve<Interpolator> base_curve;
+        typedef PiecewiseZeroInflationCurve<Interpolator,Bootstrap,Traits>
+                                                                   this_curve;
       public:
+        typedef Traits traits_type;
+        typedef Interpolator interpolator_type;
+        //! \name Constructors
+        //@{
         PiecewiseZeroInflationCurve(
                const Date& referenceDate,
                const Calendar& calendar,
@@ -94,7 +100,7 @@ namespace QuantLib {
           instruments_(instruments), accuracy_(accuracy) {
             bootstrap_.setup(this);
         }
-
+        //@}
         //! \name Inflation interface
         //@{
         Date baseDate() const;
@@ -118,62 +124,57 @@ namespace QuantLib {
         std::vector<boost::shared_ptr<typename Traits::helper> > instruments_;
         Real accuracy_;
 
-        friend class Bootstrap<
-            PiecewiseZeroInflationCurve<Interpolator, Bootstrap, Traits>,
-            Traits, Interpolator>;
-        friend class BootstrapError<
-            PiecewiseZeroInflationCurve<Interpolator, Bootstrap, Traits>,
-            Traits, Interpolator>;
-        Bootstrap<PiecewiseZeroInflationCurve<Interpolator, Bootstrap,Traits>,
-                  Traits, Interpolator> bootstrap_;
+        friend class Bootstrap<this_curve>;
+        friend class BootstrapError<this_curve>;
+        Bootstrap<this_curve> bootstrap_;
     };
 
 
     // inline and template definitions
 
-    template <class I, template <class, class, class> class B, class T>
+    template <class I, template <class> class B, class T>
     inline Date PiecewiseZeroInflationCurve<I,B,T>::baseDate() const {
         this->calculate();
         return base_curve::baseDate();
     }
 
-    template <class I, template <class, class, class> class B, class T>
+    template <class I, template <class> class B, class T>
     inline Date PiecewiseZeroInflationCurve<I,B,T>::maxDate() const {
         this->calculate();
         return base_curve::maxDate();
     }
 
-    template <class I, template <class, class, class> class B, class T>
+    template <class I, template <class> class B, class T>
     const std::vector<Time>& PiecewiseZeroInflationCurve<I,B,T>::times() const {
         calculate();
         return base_curve::times();
     }
 
-    template <class I, template <class, class, class> class B, class T>
+    template <class I, template <class> class B, class T>
     const std::vector<Date>& PiecewiseZeroInflationCurve<I,B,T>::dates() const {
         calculate();
         return base_curve::dates();
     }
 
-    template <class I, template <class, class, class> class B, class T>
+    template <class I, template <class> class B, class T>
     const std::vector<Real>& PiecewiseZeroInflationCurve<I,B,T>::data() const {
         calculate();
         return base_curve::data();
     }
 
-    template <class I, template <class, class, class> class B, class T>
+    template <class I, template <class> class B, class T>
     std::vector<std::pair<Date, Real> >
     PiecewiseZeroInflationCurve<I,B,T>::nodes() const {
         calculate();
         return base_curve::nodes();
     }
 
-    template <class I, template <class, class, class> class B, class T>
+    template <class I, template <class> class B, class T>
     void PiecewiseZeroInflationCurve<I,B,T>::performCalculations() const {
         bootstrap_.calculate();
     }
 
-    template <class I, template<class, class, class> class B, class T>
+    template <class I, template<class> class B, class T>
     void PiecewiseZeroInflationCurve<I,B,T>::update() {
         base_curve::update();
         LazyObject::update();
