@@ -19,12 +19,14 @@
 */
 
 /*! \file optionletstripper.hpp
+    \brief optionlet (caplet/floorlet) volatility stripper
 */
 
 #ifndef quantlib_optionletstripper_hpp
 #define quantlib_optionletstripper_hpp
 
 #include <ql/termstructures/volatility/optionlet/strippedoptionletbase.hpp>
+#include <ql/termstructures/volatility/capfloor/capfloortermvolsurface.hpp>
 
 namespace QuantLib {
 
@@ -33,15 +35,16 @@ namespace QuantLib {
     class OptionletStripper : public StrippedOptionletBase {
       public:
         OptionletStripper(const boost::shared_ptr<CapFloorTermVolSurface>&,
-                          const boost::shared_ptr<IborIndex>& index);
+                          const boost::shared_ptr<IborIndex>& iborIndex_);
 
         //! \name StrippedOptionletBase interface
         //@{
         const std::vector<Rate>& optionletStrikes(Size i) const;
         const std::vector<Volatility>& optionletVolatilities(Size i) const;
 
-        const std::vector<Date>& optionletDates() const;
-        const std::vector<Time>& optionletTimes() const;
+        const std::vector<Date>& optionletFixingDates() const;
+        const std::vector<Time>& optionletFixingTimes() const;
+        Size optionletMaturities() const;
 
         const std::vector<Rate>& atmOptionletRates() const;
 
@@ -49,19 +52,18 @@ namespace QuantLib {
         Calendar calendar() const;
         Natural settlementDays() const;
         BusinessDayConvention businessDayConvention() const;
-        const Date& referenceDate() const;
         //@}
 
-        boost::shared_ptr<CapFloorTermVolSurface> termVolSurface() const;
-        const std::vector<Period>& optionletTenors() const;
-        boost::shared_ptr<IborIndex> index() const;
+        const std::vector<Period>& optionletFixingTenors() const;
         const std::vector<Date>& optionletPaymentDates() const;
         const std::vector<Time>& optionletAccrualPeriods() const;
+        boost::shared_ptr<CapFloorTermVolSurface> termVolSurface() const;
+        boost::shared_ptr<IborIndex> iborIndex() const;
 
       protected:
         const boost::shared_ptr<CapFloorTermVolSurface> termVolSurface_;
-        const boost::shared_ptr<IborIndex> index_; 
-        Size nStrikes_; 
+        const boost::shared_ptr<IborIndex> iborIndex_;
+        Size nStrikes_;
         Size nOptionletTenors_;
 
         mutable std::vector<std::vector<Rate> > optionletStrikes_;
@@ -71,9 +73,9 @@ namespace QuantLib {
         mutable std::vector<Date> optionletDates_;
         std::vector<Period> optionletTenors_;
         mutable std::vector<Rate> atmOptionletRate_;
-        mutable std::vector<Date> optionletPaymentDates_; 
+        mutable std::vector<Date> optionletPaymentDates_;
         mutable std::vector<Time> optionletAccrualPeriods_;
-        
+
         std::vector<Period> capFloorLengths_;
 
     };
