@@ -18,9 +18,10 @@
 */
 
 #include <ql/termstructures/volatility/swaption/swaptionvolcube1.hpp>
-#include <ql/math/interpolations/sabrinterpolation.hpp>
-#include <ql/termstructures/volatility/swaption/swaptionvolmatrix.hpp>
 #include <ql/math/interpolations/flatextrapolation2d.hpp>
+#include <ql/math/interpolations/bilinearinterpolation.hpp>
+#include <ql/math/interpolations/sabrinterpolation.hpp>
+#include <ql/quote.hpp>
 
 #ifndef SWAPTIONVOLCUBE_VEGAWEIGHTED_TOL
     #define SWAPTIONVOLCUBE_VEGAWEIGHTED_TOL 15.0e-4
@@ -42,6 +43,7 @@ namespace QuantLib {
                 const std::vector<Spread>& strikeSpreads,
                 const std::vector<std::vector<Handle<Quote> > >& volSpreads,
                 const boost::shared_ptr<SwapIndex>& swapIndexBase,
+                const boost::shared_ptr<SwapIndex>& shortSwapIndexBase,
                 bool vegaWeightedSmileFit,
                 const std::vector<std::vector<Handle<Quote> > >& parametersGuess,
                 const std::vector<bool>& isParameterFixed,
@@ -51,12 +53,13 @@ namespace QuantLib {
                 const boost::shared_ptr<OptimizationMethod>& optMethod)
     : SwaptionVolatilityCube(atmVolStructure, optionTenors, swapTenors,
                              strikeSpreads, volSpreads, swapIndexBase,
+                             shortSwapIndexBase,
                              vegaWeightedSmileFit),
       parametersGuessQuotes_(parametersGuess),
       isParameterFixed_(isParameterFixed), isAtmCalibrated_(isAtmCalibrated),
       endCriteria_(endCriteria), optMethod_(optMethod)
     {
-        if(maxErrorTolerance != Null<Rate>()){
+        if (maxErrorTolerance != Null<Rate>()) {
             maxErrorTolerance_ = maxErrorTolerance;
         } else{
             maxErrorTolerance_ = SWAPTIONVOLCUBE_VEGAWEIGHTED_TOL;
@@ -310,15 +313,15 @@ namespace QuantLib {
 
     }
 
-    void SwaptionVolCube1::fillVolatilityCube() const{
+    void SwaptionVolCube1::fillVolatilityCube() const {
 
-        const boost::shared_ptr<SwaptionVolatilityMatrix> atmVolStructure =
-            boost::dynamic_pointer_cast<SwaptionVolatilityMatrix>(*atmVol_);
+        const boost::shared_ptr<SwaptionVolatilityDiscrete> atmVolStructure =
+            boost::dynamic_pointer_cast<SwaptionVolatilityDiscrete>(*atmVol_);
 
         std::vector<Time> atmOptionTimes(atmVolStructure->optionTimes());
         std::vector<Time> optionTimes(volCubeAtmCalibrated_.optionTimes());
         atmOptionTimes.insert(atmOptionTimes.end(),
-                                optionTimes.begin(), optionTimes.end());
+                              optionTimes.begin(), optionTimes.end());
         std::sort(atmOptionTimes.begin(),atmOptionTimes.end());
         std::vector<Time>::iterator new_end =
             unique(atmOptionTimes.begin(), atmOptionTimes.end());
@@ -408,9 +411,8 @@ namespace QuantLib {
     std::vector<Real> SwaptionVolCube1::spreadVolInterpolation(
         const Date& atmOptionDate, const Period& atmSwapTenor) const {
 
-        const std::pair<Time, Time> p =
-            convertDates(atmOptionDate, atmSwapTenor);
-        Time atmOptionTime = p.first, atmTimeLength = p.second;
+        Time atmOptionTime = timeFromReference(atmOptionDate);
+        Time atmTimeLength = swapLength(atmSwapTenor);
 
         std::vector<Real> result;
         const std::vector<Time>& optionTimes(sparseParameters_.optionTimes());
@@ -423,16 +425,16 @@ namespace QuantLib {
                                           swapLengthsPreviousNode;
 
         optionTimesPreviousNode = std::lower_bound(optionTimes.begin(),
-                                                optionTimes.end(),
-                                                atmOptionTime);
-        Size optionTimesPreviousIndex
-            = optionTimesPreviousNode - optionTimes.begin();
+                                                   optionTimes.end(),
+                                                   atmOptionTime);
+        Size optionTimesPreviousIndex =
+            optionTimesPreviousNode - optionTimes.begin();
         if (optionTimesPreviousIndex >0)
             optionTimesPreviousIndex --;
 
         swapLengthsPreviousNode = std::lower_bound(swapLengths.begin(),
-                                               swapLengths.end(),
-                                               atmTimeLength);
+                                                   swapLengths.end(),
+                                                   atmTimeLength);
         Size swapLengthsPreviousIndex = swapLengthsPreviousNode - swapLengths.begin();
         if (swapLengthsPreviousIndex >0)
             swapLengthsPreviousIndex --;
@@ -473,7 +475,7 @@ namespace QuantLib {
         swapTenorNodes[0] = swapTenors[swapLengthsPreviousIndex];
         swapTenorNodes[1] = swapTenors[swapLengthsPreviousIndex+1];
 
-        const Rate atmForward = atmStrike(atmOptionDate, atmSwapTenor);
+        Rate atmForward = atmStrike(atmOptionDate, atmSwapTenor);
 
         Matrix atmForwards(2, 2, 0.0);
         Matrix atmVols(2, 2, 0.0);
@@ -521,8 +523,7 @@ namespace QuantLib {
            localInterpolator.setLayer(0, spreadVols);
            localInterpolator.updateInterpolators();
 
-           result.push_back(
-                        localInterpolator(atmOptionTime, atmTimeLength)[0]);
+           result.push_back(localInterpolator(atmOptionTime, atmTimeLength)[0]);
         }
         return result;
     }
@@ -610,10 +611,10 @@ namespace QuantLib {
         for (Size k=0;k<nLayers_;k++) {
             transposedPoints_.push_back(transpose(points[k]));
 
-            boost::shared_ptr<Interpolation2D> interpolation (
-                new BilinearInterpolation (optionTimes_.begin(), optionTimes_.end(),
-                                           swapLengths_.begin(), swapLengths_.end(),
-                                           transposedPoints_[k]));
+            boost::shared_ptr<Interpolation2D> interpolation (new
+                BilinearInterpolation (optionTimes_.begin(), optionTimes_.end(),
+                                       swapLengths_.begin(), swapLengths_.end(),
+                                       transposedPoints_[k]));
             interpolators_.push_back(boost::shared_ptr<Interpolation2D>(
                 new FlatExtrapolator2D(interpolation)));
             interpolators_[k]->enableExtrapolation();
