@@ -20,64 +20,70 @@
 */
 
 #include <ql/termstructures/volatility/swaption/swaptionvolstructure.hpp>
-#include <ql/time/period.hpp>
+#include <ql/math/rounding.hpp>
 
 namespace QuantLib {
 
     SwaptionVolatilityStructure::SwaptionVolatilityStructure(
                                                     const Calendar& cal,
-                                                    const DayCounter& dc,
-                                                    BusinessDayConvention bdc)
+                                                    BusinessDayConvention bdc,
+                                                    const DayCounter& dc)
     : VolatilityTermStructure(cal, bdc, dc) {}
 
     SwaptionVolatilityStructure::SwaptionVolatilityStructure(
                                                 const Date& referenceDate,
                                                 const Calendar& calendar,
-                                                const DayCounter& dc,
-                                                BusinessDayConvention bdc)
+                                                BusinessDayConvention bdc,
+                                                const DayCounter& dc)
     : VolatilityTermStructure(referenceDate, calendar, bdc, dc) {}
 
     SwaptionVolatilityStructure::SwaptionVolatilityStructure(
                                                 Natural settlementDays,
                                                 const Calendar& calendar,
-                                                const DayCounter& dc,
-                                                BusinessDayConvention bdc)
+                                                BusinessDayConvention bdc,
+                                                const DayCounter& dc)
     : VolatilityTermStructure(settlementDays, calendar, bdc, dc) {}
 
 
-    Time SwaptionVolatilityStructure::maxSwapLength() const {
-        //Date d = referenceDate()+maxSwapTenor();
-        Date d = optionDateFromTenor(maxSwapTenor());
-        Time t = timeFromReference(d);
-        return t;
+    Time SwaptionVolatilityStructure::swapLength(const Period& p) const {
+        QL_REQUIRE(p.length()>0,
+                   "non-positive swap tenor (" << p << ") given");
+        /* while using the reference date is arbitrary it is coherent between
+           different swaption structures defined on the same reference date.
+        */
+        Date start = referenceDate();
+        Date end = start + p;
+        return swapLength(start, end);
     }
 
-    std::pair<Time,Time>
-    SwaptionVolatilityStructure::convertDates(const Date& optionDate,
-                                              const Period& swapTenor) const {
-        Date end = optionDate + swapTenor;
-        QL_REQUIRE(end>optionDate,
-                   "negative swap tenor (" << swapTenor << ") given");
-        Time optionTime = timeFromReference(optionDate);
-        Time timeLength = dayCounter().yearFraction(optionDate, end);
-        return std::make_pair(optionTime, timeLength);
+    Time SwaptionVolatilityStructure::swapLength(const Date& start,
+                                                 const Date& end) const {
+        QL_REQUIRE(end>start, "swap end date (" << end <<
+                   ") must be greater than start (" << start << ")");
+        Time result = (end-start)/365.25*24.0; // half a month unit
+        result = ClosestRounding(0)(result);
+        result /= 24.0; // year unit
+        return result;
     }
 
-    void SwaptionVolatilityStructure::checkRange(
-             const Date& optionDate, const Period& swapTenor,
-             Rate k, bool extrapolate) const {
-        TermStructure::checkRange(timeFromReference(optionDate),
-                                  extrapolate);
+    void SwaptionVolatilityStructure::checkSwapTenor(const Period& swapTenor,
+                                                     bool extrapolate) const {
         QL_REQUIRE(swapTenor.length() > 0,
-                   "negative swap tenor (" << swapTenor << ") given");
+                   "non-positive swap tenor (" << swapTenor << ") given");
         QL_REQUIRE(extrapolate || allowsExtrapolation() ||
                    swapTenor <= maxSwapTenor(),
                    "swap tenor (" << swapTenor << ") is past max tenor ("
                    << maxSwapTenor() << ")");
+    }
+
+    void SwaptionVolatilityStructure::checkSwapTenor(Time swapLength,
+                                                     bool extrapolate) const {
+        QL_REQUIRE(swapLength > 0.0,
+                   "non-positive swap length (" << swapLength << ") given");
         QL_REQUIRE(extrapolate || allowsExtrapolation() ||
-                   (k >= minStrike() && k <= maxStrike()),
-                   "strike (" << k << ") is outside the curve domain ["
-                   << minStrike() << "," << maxStrike()<< "]");
+                   swapLength <= maxSwapLength(),
+                   "swap tenor (" << swapLength << ") is past max tenor ("
+                   << maxSwapLength() << ")");
     }
 
 }
