@@ -2,7 +2,7 @@
 
 /*
  Copyright (C) 2002, 2003 Decillion Pty(Ltd)
- Copyright (C) 2005, 2006 StatPro Italia srl
+ Copyright (C) 2005, 2006, 2008 StatPro Italia srl
 
  This file is part of QuantLib, a free-software/open-source library
  for financial quantitative analysts and developers - http://quantlib.org/
@@ -28,6 +28,7 @@
 #include <ql/termstructures/yieldtermstructure.hpp>
 #include <ql/math/interpolations/loginterpolation.hpp>
 #include <ql/math/comparison.hpp>
+#include <boost/noncopyable.hpp>
 #include <vector>
 #include <utility>
 
@@ -36,7 +37,8 @@ namespace QuantLib {
     //! Term structure based on interpolation of discount factors
     /*! \ingroup yieldtermstructures */
     template <class Interpolator>
-    class InterpolatedDiscountCurve : public YieldTermStructure {
+    class InterpolatedDiscountCurve : public YieldTermStructure,
+                                      public boost::noncopyable {
       public:
         InterpolatedDiscountCurve(const std::vector<Date>& dates,
                                   const std::vector<DiscountFactor>& dfs,
@@ -116,6 +118,8 @@ namespace QuantLib {
         return results;
     }
 
+    #ifndef __DOXYGEN__
+
     template <class T>
     inline InterpolatedDiscountCurve<T>::InterpolatedDiscountCurve(
                                                  const DayCounter& dayCounter,
@@ -155,7 +159,7 @@ namespace QuantLib {
                                  const DayCounter& dayCounter,
                                  const Calendar& cal,
                                  const T& interpolator)
-    : YieldTermStructure(dates[0], cal, dayCounter),
+    : YieldTermStructure(dates.front(), cal, dayCounter),
       dates_(dates), data_(discounts),
       interpolator_(interpolator) {
         QL_REQUIRE(!dates_.empty(), "no input dates given");
@@ -185,6 +189,7 @@ namespace QuantLib {
         interpolation_.update();
     }
 
+    #endif
 }
 
 #endif
