@@ -22,6 +22,7 @@
 #include <ql/math/optimization/costfunction.hpp>
 #include <ql/math/optimization/constraint.hpp>
 #include <ql/cashflows/cashflows.hpp>
+#include <ql/utilities/dataformatters.hpp>
 
 namespace QuantLib {
 
@@ -118,11 +119,12 @@ namespace QuantLib {
         // double check bond quotes still valid and/or instruments not expired
         for (Size i=0; i<instruments_.size(); ++i) {
             QL_REQUIRE(instruments_[i]->quoteIsValid(),
-                       "instrument with invalid quote");
+                       io::ordinal(i+1) << " instrument has an invalid quote");
             instruments_[i]->setTermStructure(
                                   const_cast<FittedBondDiscountCurve*>(this));
             boost::shared_ptr<Bond> bond = instruments_[i]->bond();
-            QL_REQUIRE(!bond->isExpired(), "expired bond instrument");
+            QL_REQUIRE(!bond->isExpired(),
+                       io::ordinal(i+1) << " bond is expired");
         }
 
         maxDate_ = Date::minDate();
@@ -169,14 +171,14 @@ namespace QuantLib {
             Leg leg = bond->cashflows();
             Real cleanPrice = curve_->instruments_[k]->quoteValue();
             Rate ytm = bond->yield(cleanPrice,
-                                   curve_->instruments_[k]->dayCounter(),
+                                   curve_->instruments_[k]->bond()->dayCounter(),
                                    Compounded,
-                                   curve_->instruments_[k]->frequency(),
+                                   curve_->instruments_[k]->bond()->frequency(),
                                    today);
             InterestRate r(ytm,
-                           curve_->instruments_[k]->dayCounter(),
+                           curve_->instruments_[k]->bond()->dayCounter(),
                            Compounded,
-                           curve_->instruments_[k]->frequency());
+                           curve_->instruments_[k]->bond()->frequency());
 
             Date settlement = bond->settlementDate(today);
             Time duration =
@@ -267,7 +269,7 @@ namespace QuantLib {
             Real dirtyPrice = quotedPrice + bond->accruedAmount(settlement);
 
             DayCounter bondDayCount =
-                fittingMethod_->curve_->instruments_[i]->dayCounter();
+                fittingMethod_->curve_->instruments_[i]->bond()->dayCounter();
             Leg cf = bond->cashflows();
 
             // loop over cashFlows: P_j = sum( cf_i * d(t_i))
