@@ -2,7 +2,8 @@
 
 /*
  Copyright (C) 2000, 2001, 2002, 2003 RiskMap srl
- Copyright (C) 2003, 2004, 2005, 2006, 2007 StatPro Italia srl
+ Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008 StatPro Italia srl
+ Copyright (C) 2007, 2008 Ferdinando Ametrano
  Copyright (C) 2007 Roland Lichters
 
  This file is part of QuantLib, a free-software/open-source library
@@ -20,16 +21,17 @@
 */
 
 #include <ql/termstructures/yield/ratehelpers.hpp>
-#include <ql/cashflows/iborcoupon.hpp>
-#include <ql/time/daycounters/actual360.hpp>
 #include <ql/time/imm.hpp>
 #include <ql/instruments/makevanillaswap.hpp>
-#include <ql/instruments/bmaswap.hpp>
 #include <ql/pricingengines/swap/discountingswapengine.hpp>
-#include <ql/quotes/simplequote.hpp>
+#include <ql/quote.hpp>
 #include <ql/currency.hpp>
 #include <ql/indexes/swapindex.hpp>
-#include <ql/termstructures/yieldtermstructure.hpp>
+#ifdef QL_USE_INDEXED_COUPON
+    #include <ql/cashflows/floatingratecoupon.hpp>
+#endif
+
+using boost::shared_ptr;
 
 namespace QuantLib {
 
@@ -39,65 +41,77 @@ namespace QuantLib {
 
     FuturesRateHelper::FuturesRateHelper(const Handle<Quote>& price,
                                          const Date& immDate,
-                                         Size nMonths,
+                                         Natural lengthInMonths,
                                          const Calendar& calendar,
                                          BusinessDayConvention convention,
+                                         bool endOfMonth,
                                          const DayCounter& dayCounter,
-                                         const Handle<Quote>& convexityAdjustment)
-    : RateHelper(price),
-      convAdj_(convexityAdjustment) {
+                                         const Handle<Quote>& convAdj)
+    : RateHelper(price), convAdj_(convAdj) {
         QL_REQUIRE(IMM::isIMMdate(immDate, false),
-                   immDate << "is not a valid IMM date");
+                   immDate << " is not a valid IMM date");
         earliestDate_ = immDate;
-        latestDate_ =
-            calendar.advance(earliestDate_, nMonths, Months, convention);
+        latestDate_ = calendar.advance(immDate, lengthInMonths*Months,
+                                       convention, endOfMonth);
         yearFraction_ = dayCounter.yearFraction(earliestDate_, latestDate_);
-        registerWith(convexityAdjustment);
-    }
 
-    FuturesRateHelper::FuturesRateHelper(const Handle<Quote>& price,
-                                         const Date& immDate,
-                                         Size nMonths,
-                                         const Calendar& calendar,
-                                         BusinessDayConvention convention,
-                                         const DayCounter& dayCounter,
-                                         Rate convexityAdjustment)
-    : RateHelper(price),
-      convAdj_(Handle<Quote>(boost::shared_ptr<Quote>(new
-                                            SimpleQuote(convexityAdjustment))))
-    {
-        QL_REQUIRE(IMM::isIMMdate(immDate, false),
-                   immDate << "is not a valid IMM date");
-        earliestDate_ = immDate;
-        latestDate_ =
-            calendar.advance(earliestDate_, nMonths, Months, convention);
-        yearFraction_ = dayCounter.yearFraction(earliestDate_, latestDate_);
+        registerWith(convAdj_);
     }
 
     FuturesRateHelper::FuturesRateHelper(Real price,
                                          const Date& immDate,
-                                         Size nMonths,
+                                         Natural lengthInMonths,
                                          const Calendar& calendar,
                                          BusinessDayConvention convention,
+                                         bool endOfMonth,
                                          const DayCounter& dayCounter,
-                                         Rate convexityAdjustment)
+                                         Rate convAdj)
     : RateHelper(price),
-      convAdj_(Handle<Quote>(boost::shared_ptr<Quote>(new
-                                            SimpleQuote(convexityAdjustment))))
+      convAdj_(Handle<Quote>(shared_ptr<Quote>(new SimpleQuote(convAdj))))
     {
         QL_REQUIRE(IMM::isIMMdate(immDate, false),
                    immDate << "is not a valid IMM date");
         earliestDate_ = immDate;
-        latestDate_ =
-            calendar.advance(earliestDate_, nMonths, Months, convention);
+        latestDate_ = calendar.advance(immDate, lengthInMonths*Months,
+                                       convention, endOfMonth);
         yearFraction_ = dayCounter.yearFraction(earliestDate_, latestDate_);
     }
 
+    FuturesRateHelper::FuturesRateHelper(const Handle<Quote>& price,
+                                         const Date& immDate,
+                                         const shared_ptr<IborIndex>& i,
+                                         const Handle<Quote>& convAdj)
+    : RateHelper(price), convAdj_(convAdj) {
+        QL_REQUIRE(IMM::isIMMdate(immDate, false),
+                   immDate << "is not a valid IMM date");
+        earliestDate_ = immDate;
+        const Calendar& cal = i->fixingCalendar();
+        latestDate_=cal.advance(immDate,i->tenor(),i->businessDayConvention());
+        yearFraction_=i->dayCounter().yearFraction(earliestDate_, latestDate_);
+
+        registerWith(convAdj);
+    }
+
+    FuturesRateHelper::FuturesRateHelper(Real price,
+                                         const Date& immDate,
+                                         const shared_ptr<IborIndex>& i,
+                                         Rate convAdj)
+    : RateHelper(price),
+      convAdj_(Handle<Quote>(shared_ptr<Quote>(new SimpleQuote(convAdj))))
+    {
+        QL_REQUIRE(IMM::isIMMdate(immDate, false),
+                   immDate << "is not a valid IMM date");
+        earliestDate_ = immDate;
+        const Calendar& cal = i->fixingCalendar();
+        latestDate_=cal.advance(immDate,i->tenor(),i->businessDayConvention());
+        yearFraction_=i->dayCounter().yearFraction(earliestDate_, latestDate_);
+    }
+
     Real FuturesRateHelper::impliedQuote() const {
         QL_REQUIRE(termStructure_ != 0, "term structure not set");
         Rate forwardRate = (termStructure_->discount(earliestDate_) /
             termStructure_->discount(latestDate_)-1.0)/yearFraction_;
-        Rate convAdj = convAdj_->value();
+        Rate convAdj = convAdj_.empty() ? 0.0 : convAdj_->value();
         QL_ENSURE(convAdj >= 0.0,
                   "Negative (" << convAdj <<
                   ") futures convexity adjustment");
@@ -106,9 +120,19 @@ namespace QuantLib {
     }
 
     Real FuturesRateHelper::convexityAdjustment() const {
-        return convAdj_->value();
+        return convAdj_.empty() ? 0.0 : convAdj_->value();
     }
 
+    void FuturesRateHelper::accept(AcyclicVisitor& v) {
+        Visitor<FuturesRateHelper>* v1 =
+            dynamic_cast<Visitor<FuturesRateHelper>*>(&v);
+        if (v1 != 0)
+            v1->visit(*this);
+        else
+            RateHelper::accept(v);
+    }
+
+
     RelativeDateRateHelper::RelativeDateRateHelper(const Handle<Quote>& quote)
     : RateHelper(quote) {
         registerWith(Settings::instance().evaluationDate());
@@ -132,15 +156,15 @@ namespace QuantLib {
 
     DepositRateHelper::DepositRateHelper(const Handle<Quote>& rate,
                                          const Period& tenor,
-                                         Natural settlementDays,
+                                         Natural fixingDays,
                                          const Calendar& calendar,
                                          BusinessDayConvention convention,
                                          bool endOfMonth,
-                                         Natural fixingDays,
                                          const DayCounter& dayCounter)
-    : RelativeDateRateHelper(rate), settlementDays_(settlementDays) {
-        index_ = boost::shared_ptr<IborIndex>(new
-            IborIndex("dummy", tenor, fixingDays,
+    : RelativeDateRateHelper(rate) {
+        iborIndex_ = shared_ptr<IborIndex>(new
+            IborIndex("no-fix", // never take fixing into account
+                      tenor, fixingDays,
                       Currency(), calendar, convention,
                       endOfMonth, dayCounter, termStructureHandle_));
         initializeDates();
@@ -148,59 +172,85 @@ namespace QuantLib {
 
     DepositRateHelper::DepositRateHelper(Rate rate,
                                          const Period& tenor,
-                                         Natural settlementDays,
+                                         Natural fixingDays,
                                          const Calendar& calendar,
                                          BusinessDayConvention convention,
                                          bool endOfMonth,
-                                         Natural fixingDays,
                                          const DayCounter& dayCounter)
-    : RelativeDateRateHelper(rate), settlementDays_(settlementDays) {
-        index_ = boost::shared_ptr<IborIndex>(new
-            IborIndex("dummy", tenor, fixingDays,
+    : RelativeDateRateHelper(rate) {
+        iborIndex_ = shared_ptr<IborIndex>(new
+            IborIndex("no-fix", // never take fixing into account
+                      tenor, fixingDays,
                       Currency(), calendar, convention,
                       endOfMonth, dayCounter, termStructureHandle_));
         initializeDates();
     }
 
+    DepositRateHelper::DepositRateHelper(const Handle<Quote>& rate,
+                                         const shared_ptr<IborIndex>& i)
+    : RelativeDateRateHelper(rate) {
+        iborIndex_ = shared_ptr<IborIndex>(new
+            IborIndex("no-fix", // never take fixing into account
+                      i->tenor(), i->fixingDays(), Currency(),
+                      i->fixingCalendar(), i->businessDayConvention(),
+                      i->endOfMonth(), i->dayCounter(), termStructureHandle_));
+        initializeDates();
+    }
+
+    DepositRateHelper::DepositRateHelper(Rate rate,
+                                         const shared_ptr<IborIndex>& i)
+    : RelativeDateRateHelper(rate) {
+        iborIndex_ = shared_ptr<IborIndex>(new
+            IborIndex("no-fix", // never take fixing into account
+                      i->tenor(), i->fixingDays(), Currency(),
+                      i->fixingCalendar(), i->businessDayConvention(),
+                      i->endOfMonth(), i->dayCounter(), termStructureHandle_));
+        initializeDates();
+    }
+
     Real DepositRateHelper::impliedQuote() const {
         QL_REQUIRE(termStructure_ != 0, "term structure not set");
-        return index_->fixing(fixingDate_, true);
+        return iborIndex_->fixing(fixingDate_, true);
     }
 
     void DepositRateHelper::setTermStructure(YieldTermStructure* t) {
         // no need to register---the index is not lazy
         termStructureHandle_.linkTo(
-                         boost::shared_ptr<YieldTermStructure>(t,no_deletion),
+                         shared_ptr<YieldTermStructure>(t,no_deletion),
                          false);
         RelativeDateRateHelper::setTermStructure(t);
     }
 
     void DepositRateHelper::initializeDates() {
-        // why not using index_->fixingDays instead of settlementDays_
-        earliestDate_ = index_->fixingCalendar().advance(
-            evaluationDate_, settlementDays_, Days);
-        latestDate_ = index_->maturityDate(earliestDate_);
-        // why not using index_->fixingDate
-        fixingDate_ = index_->fixingCalendar().advance(earliestDate_,
-            -static_cast<Integer>(index_->fixingDays()), Days);
+        earliestDate_ = iborIndex_->fixingCalendar().advance(
+            evaluationDate_, iborIndex_->fixingDays()*Days);
+        latestDate_ = iborIndex_->maturityDate(earliestDate_);
+        fixingDate_ = iborIndex_->fixingDate(earliestDate_);
+    }
+
+    void DepositRateHelper::accept(AcyclicVisitor& v) {
+        Visitor<DepositRateHelper>* v1 =
+            dynamic_cast<Visitor<DepositRateHelper>*>(&v);
+        if (v1 != 0)
+            v1->visit(*this);
+        else
+            RateHelper::accept(v);
     }
 
 
     FraRateHelper::FraRateHelper(const Handle<Quote>& rate,
                                  Natural monthsToStart,
                                  Natural monthsToEnd,
-                                 Natural settlementDays,
+                                 Natural fixingDays,
                                  const Calendar& calendar,
                                  BusinessDayConvention convention,
                                  bool endOfMonth,
-                                 Natural fixingDays,
                                  const DayCounter& dayCounter)
-    : RelativeDateRateHelper(rate), monthsToStart_(monthsToStart),
-      settlementDays_(settlementDays) {
+    : RelativeDateRateHelper(rate), periodToStart_(monthsToStart*Months) {
         QL_REQUIRE(monthsToEnd>monthsToStart,
                    "monthsToEnd must be grater than monthsToStart");
-        index_ = boost::shared_ptr<IborIndex>(new
-            IborIndex("dummy",
+        iborIndex_ = shared_ptr<IborIndex>(new
+            IborIndex("no-fix", // never take fixing into account
                       (monthsToEnd-monthsToStart)*Months,
                       fixingDays,
                       Currency(), calendar, convention,
@@ -211,18 +261,16 @@ namespace QuantLib {
     FraRateHelper::FraRateHelper(Rate rate,
                                  Natural monthsToStart,
                                  Natural monthsToEnd,
-                                 Natural settlementDays,
+                                 Natural fixingDays,
                                  const Calendar& calendar,
                                  BusinessDayConvention convention,
                                  bool endOfMonth,
-                                 Natural fixingDays,
                                  const DayCounter& dayCounter)
-    : RelativeDateRateHelper(rate), monthsToStart_(monthsToStart),
-      settlementDays_(settlementDays) {
+    : RelativeDateRateHelper(rate),periodToStart_(monthsToStart*Months) {
         QL_REQUIRE(monthsToEnd>monthsToStart,
                    "monthsToEnd must be grater than monthsToStart");
-        index_ = boost::shared_ptr<IborIndex>(new
-            IborIndex("dummy",
+        iborIndex_ = shared_ptr<IborIndex>(new
+            IborIndex("no-fix", // never take fixing into account
                       (monthsToEnd-monthsToStart)*Months,
                       fixingDays,
                       Currency(), calendar, convention,
@@ -230,35 +278,127 @@ namespace QuantLib {
         initializeDates();
     }
 
+    FraRateHelper::FraRateHelper(const Handle<Quote>& rate,
+                                 Natural monthsToStart,
+                                 const shared_ptr<IborIndex>& i)
+    : RelativeDateRateHelper(rate), periodToStart_(monthsToStart*Months) {
+        iborIndex_ = shared_ptr<IborIndex>(new
+            IborIndex("no-fix", // never take fixing into account
+                      i->tenor(), i->fixingDays(), Currency(),
+                      i->fixingCalendar(), i->businessDayConvention(),
+                      i->endOfMonth(), i->dayCounter(), termStructureHandle_));
+        initializeDates();
+    }
+
+    FraRateHelper::FraRateHelper(Rate rate,
+                                 Natural monthsToStart,
+                                 const shared_ptr<IborIndex>& i)
+    : RelativeDateRateHelper(rate), periodToStart_(monthsToStart*Months) {
+        iborIndex_ = shared_ptr<IborIndex>(new
+            IborIndex("no-fix", // never take fixing into account
+                      i->tenor(), i->fixingDays(), Currency(),
+                      i->fixingCalendar(), i->businessDayConvention(),
+                      i->endOfMonth(), i->dayCounter(), termStructureHandle_));
+        initializeDates();
+    }
+
+    FraRateHelper::FraRateHelper(const Handle<Quote>& rate,
+                                 Period periodToStart,
+                                 Natural lengthInMonths,
+                                 Natural fixingDays,
+                                 const Calendar& calendar,
+                                 BusinessDayConvention convention,
+                                 bool endOfMonth,
+                                 const DayCounter& dayCounter)
+    : RelativeDateRateHelper(rate), periodToStart_(periodToStart) {
+        iborIndex_ = shared_ptr<IborIndex>(new
+            IborIndex("no-fix", // never take fixing into account
+                      lengthInMonths*Months,
+                      fixingDays,
+                      Currency(), calendar, convention,
+                      endOfMonth, dayCounter, termStructureHandle_));
+        initializeDates();
+    }
+
+    FraRateHelper::FraRateHelper(Rate rate,
+                                 Period periodToStart,
+                                 Natural lengthInMonths,
+                                 Natural fixingDays,
+                                 const Calendar& calendar,
+                                 BusinessDayConvention convention,
+                                 bool endOfMonth,
+                                 const DayCounter& dayCounter)
+    : RelativeDateRateHelper(rate),periodToStart_(periodToStart) {
+        iborIndex_ = shared_ptr<IborIndex>(new
+            IborIndex("no-fix", // never take fixing into account
+                      lengthInMonths*Months,
+                      fixingDays,
+                      Currency(), calendar, convention,
+                      endOfMonth, dayCounter, termStructureHandle_));
+        initializeDates();
+    }
+
+    FraRateHelper::FraRateHelper(const Handle<Quote>& rate,
+                                 Period periodToStart,
+                                 const shared_ptr<IborIndex>& i)
+    : RelativeDateRateHelper(rate), periodToStart_(periodToStart) {
+        iborIndex_ = shared_ptr<IborIndex>(new
+            IborIndex("no-fix", // never take fixing into account
+                      i->tenor(), i->fixingDays(), Currency(),
+                      i->fixingCalendar(), i->businessDayConvention(),
+                      i->endOfMonth(), i->dayCounter(), termStructureHandle_));
+        initializeDates();
+    }
+
+    FraRateHelper::FraRateHelper(Rate rate,
+                                 Period periodToStart,
+                                 const shared_ptr<IborIndex>& i)
+    : RelativeDateRateHelper(rate), periodToStart_(periodToStart) {
+        iborIndex_ = shared_ptr<IborIndex>(new
+            IborIndex("no-fix", // never take fixing into account
+                      i->tenor(), i->fixingDays(), Currency(),
+                      i->fixingCalendar(), i->businessDayConvention(),
+                      i->endOfMonth(), i->dayCounter(), termStructureHandle_));
+        initializeDates();
+    }
+
     Real FraRateHelper::impliedQuote() const {
         QL_REQUIRE(termStructure_ != 0, "term structure not set");
-        return index_->fixing(fixingDate_, true);
+        return iborIndex_->fixing(fixingDate_, true);
     }
 
     void FraRateHelper::setTermStructure(YieldTermStructure* t) {
         // no need to register---the index is not lazy
         termStructureHandle_.linkTo(
-                         boost::shared_ptr<YieldTermStructure>(t,no_deletion),
+                         shared_ptr<YieldTermStructure>(t,no_deletion),
                          false);
         RelativeDateRateHelper::setTermStructure(t);
     }
 
     void FraRateHelper::initializeDates() {
-        // why not using index_->fixingDays instead of settlementDays_
-        Date settlement = index_->fixingCalendar().advance(evaluationDate_,
-                                                           settlementDays_,
-                                                           Days);
-        earliestDate_ = index_->fixingCalendar().advance(
-                               settlement, monthsToStart_, Months,
-                               index_->businessDayConvention(),
-                               index_->endOfMonth());
-        latestDate_ = index_->maturityDate(earliestDate_);
-        fixingDate_ = index_->fixingDate(earliestDate_);
+        Date settlement = iborIndex_->fixingCalendar().advance(
+            evaluationDate_, iborIndex_->fixingDays()*Days);
+        earliestDate_ = iborIndex_->fixingCalendar().advance(
+                               settlement,
+                               periodToStart_,
+                               iborIndex_->businessDayConvention(),
+                               iborIndex_->endOfMonth());
+        latestDate_ = iborIndex_->maturityDate(earliestDate_);
+        fixingDate_ = iborIndex_->fixingDate(earliestDate_);
+    }
+
+    void FraRateHelper::accept(AcyclicVisitor& v) {
+        Visitor<FraRateHelper>* v1 =
+            dynamic_cast<Visitor<FraRateHelper>*>(&v);
+        if (v1 != 0)
+            v1->visit(*this);
+        else
+            RateHelper::accept(v);
     }
 
 
     SwapRateHelper::SwapRateHelper(const Handle<Quote>& rate,
-                                   const boost::shared_ptr<SwapIndex>& swapIndex,
+                                   const shared_ptr<SwapIndex>& swapIndex,
                                    const Handle<Quote>& spread,
                                    const Period& fwdStart)
     : RelativeDateRateHelper(rate),
@@ -280,12 +420,13 @@ namespace QuantLib {
                                    Frequency fixedFrequency,
                                    BusinessDayConvention fixedConvention,
                                    const DayCounter& fixedDayCount,
-                                   const boost::shared_ptr<IborIndex>& iborIndex,
+                                   const shared_ptr<IborIndex>& iborIndex,
                                    const Handle<Quote>& spread,
                                    const Period& fwdStart)
     : RelativeDateRateHelper(rate),
       tenor_(tenor),
-      calendar_(calendar), fixedConvention_(fixedConvention),
+      calendar_(calendar),
+      fixedConvention_(fixedConvention),
       fixedFrequency_(fixedFrequency),
       fixedDayCount_(fixedDayCount),
       iborIndex_(iborIndex),
@@ -301,12 +442,13 @@ namespace QuantLib {
                                    Frequency fixedFrequency,
                                    BusinessDayConvention fixedConvention,
                                    const DayCounter& fixedDayCount,
-                                   const boost::shared_ptr<IborIndex>& iborIndex,
+                                   const shared_ptr<IborIndex>& iborIndex,
                                    const Handle<Quote>& spread,
                                    const Period& fwdStart)
     : RelativeDateRateHelper(rate),
       tenor_(tenor),
-      calendar_(calendar), fixedConvention_(fixedConvention),
+      calendar_(calendar),
+      fixedConvention_(fixedConvention),
       fixedFrequency_(fixedFrequency),
       fixedDayCount_(fixedDayCount),
       iborIndex_(iborIndex),
@@ -316,19 +458,28 @@ namespace QuantLib {
         initializeDates();
     }
 
+    SwapRateHelper::SwapRateHelper(Rate rate,
+                                   const shared_ptr<SwapIndex>& swapIndex,
+                                   const Handle<Quote>& spread,
+                                   const Period& fwdStart)
+    : RelativeDateRateHelper(rate),
+      tenor_(swapIndex->tenor()),
+      calendar_(swapIndex->fixingCalendar()),
+      fixedConvention_(swapIndex->fixedLegConvention()),
+      fixedFrequency_(swapIndex->fixedLegTenor().frequency()),
+      fixedDayCount_(swapIndex->dayCounter()),
+      iborIndex_(swapIndex->iborIndex()),
+      spread_(spread), fwdStart_(fwdStart) {
+        registerWith(iborIndex_);
+        registerWith(spread_);
+        initializeDates();
+    }
+
     void SwapRateHelper::initializeDates() {
 
         // dummy ibor index with curve/swap arguments
-        boost::shared_ptr<IborIndex> clonedIborIndex(new
-            IborIndex(iborIndex_->familyName(),
-                      iborIndex_->tenor(),
-                      iborIndex_->fixingDays(),
-                      iborIndex_->currency(),
-                      iborIndex_->fixingCalendar(),
-                      iborIndex_->businessDayConvention(),
-                      iborIndex_->endOfMonth(),
-                      iborIndex_->dayCounter(),
-                      termStructureHandle_));
+        boost::shared_ptr<IborIndex> clonedIborIndex =
+            iborIndex_->clone(termStructureHandle_);
 
         // do not pass the spread here, as it might be a Quote
         // i.e. it can dinamically change
@@ -336,7 +487,9 @@ namespace QuantLib {
             .withFixedLegDayCount(fixedDayCount_)
             .withFixedLegTenor(Period(fixedFrequency_))
             .withFixedLegConvention(fixedConvention_)
-            .withFixedLegTerminationDateConvention(fixedConvention_);
+            .withFixedLegTerminationDateConvention(fixedConvention_)
+            .withFixedLegCalendar(calendar_)
+            .withFloatingLegCalendar(calendar_);
 
         earliestDate_ = swap_->startDate();
 
@@ -345,15 +498,12 @@ namespace QuantLib {
         // ...but due to adjustments, the last floating coupon might
         // need a later date for fixing
         #ifdef QL_USE_INDEXED_COUPON
-        boost::shared_ptr<FloatingRateCoupon> lastFloating =
+        shared_ptr<FloatingRateCoupon> lastFloating =
             boost::dynamic_pointer_cast<FloatingRateCoupon>(
                                                  swap_->floatingLeg().back());
-        Date fixingValueDate = calendar_.advance(lastFloating->fixingDate(),
-                                                 iborIndex_->fixingDays(),Days);
-        Date endValueDate = calendar_.advance(fixingValueDate,
-                                              iborIndex_->tenor(),
-                                              iborIndex_->businessDayConvention(),
-                                              iborIndex_->endOfMonth());
+        Date fixingValueDate =
+            iborIndex_->valueDate(lastFloating->fixingDate());
+        Date endValueDate = iborIndex_->maturityDate(fixingValueDate);
         latestDate_ = std::max(latestDate_, endValueDate);
         #endif
     }
@@ -362,7 +512,7 @@ namespace QuantLib {
         // do not set the relinkable handle as an observer -
         // force recalculation when needed
         termStructureHandle_.linkTo(
-                         boost::shared_ptr<YieldTermStructure>(t,no_deletion),
+                         shared_ptr<YieldTermStructure>(t,no_deletion),
                          false);
         RelativeDateRateHelper::setTermStructure(t);
     }
@@ -385,7 +535,7 @@ namespace QuantLib {
         return spread_.empty() ? 0.0 : spread_->value();
     }
 
-    boost::shared_ptr<VanillaSwap> SwapRateHelper::swap() const {
+    shared_ptr<VanillaSwap> SwapRateHelper::swap() const {
         return swap_;
     }
 
@@ -393,6 +543,16 @@ namespace QuantLib {
         return fwdStart_;
     }
 
+    void SwapRateHelper::accept(AcyclicVisitor& v) {
+        Visitor<SwapRateHelper>* v1 =
+            dynamic_cast<Visitor<SwapRateHelper>*>(&v);
+        if (v1 != 0)
+            v1->visit(*this);
+        else
+            RateHelper::accept(v);
+    }
+
+
     BMASwapRateHelper::BMASwapRateHelper(
                           const Handle<Quote>& liborFraction,
                           const Period& tenor,
@@ -402,9 +562,9 @@ namespace QuantLib {
                           const Period& bmaPeriod,
                           BusinessDayConvention bmaConvention,
                           const DayCounter& bmaDayCount,
-                          const boost::shared_ptr<BMAIndex>& bmaIndex,
+                          const shared_ptr<BMAIndex>& bmaIndex,
                           // libor leg
-                          const boost::shared_ptr<IborIndex>& index)
+                          const shared_ptr<IborIndex>& iborIndex)
     : RelativeDateRateHelper(liborFraction),
       tenor_(tenor), settlementDays_(settlementDays),
       calendar_(calendar),
@@ -412,8 +572,8 @@ namespace QuantLib {
       bmaConvention_(bmaConvention),
       bmaDayCount_(bmaDayCount),
       bmaIndex_(bmaIndex),
-      index_(index) {
-        registerWith(index_);
+      iborIndex_(iborIndex) {
+        registerWith(iborIndex_);
         registerWith(bmaIndex_);
         initializeDates();
     }
@@ -426,35 +586,34 @@ namespace QuantLib {
         Date maturity = earliestDate_ + tenor_;
 
         // dummy BMA index with curve/swap arguments
-        boost::shared_ptr<BMAIndex> clonedIndex(
-                                          new BMAIndex(termStructureHandle_));
+        shared_ptr<BMAIndex> clonedIndex(new BMAIndex(termStructureHandle_));
 
         Schedule bmaSchedule = MakeSchedule(earliestDate_,
                                             maturity,
                                             bmaPeriod_,
-                                            calendar_,
+                                            bmaIndex_->fixingCalendar(),
                                             bmaConvention_).backwards();
 
         Schedule liborSchedule =
             MakeSchedule(earliestDate_,
                          maturity,
-                         index_->tenor(),
-                         index_->fixingCalendar(),
-                         index_->businessDayConvention())
-            .endOfMonth(index_->endOfMonth())
+                         iborIndex_->tenor(),
+                         iborIndex_->fixingCalendar(),
+                         iborIndex_->businessDayConvention())
+            .endOfMonth(iborIndex_->endOfMonth())
             .backwards();
 
-        swap_ = boost::shared_ptr<BMASwap>(new BMASwap(BMASwap::Payer, 100.0,
-                                                       liborSchedule,
-                                                       0.75, // arbitrary
-                                                       0.0,
-                                                       index_,
-                                                       index_->dayCounter(),
-                                                       bmaSchedule,
-                                                       clonedIndex,
-                                                       bmaDayCount_));
-        swap_->setPricingEngine(boost::shared_ptr<PricingEngine>(
-                         new DiscountingSwapEngine(index_->termStructure())));
+        swap_ = shared_ptr<BMASwap>(new BMASwap(BMASwap::Payer, 100.0,
+                                                liborSchedule,
+                                                0.75, // arbitrary
+                                                0.0,
+                                                iborIndex_,
+                                                iborIndex_->dayCounter(),
+                                                bmaSchedule,
+                                                clonedIndex,
+                                                bmaDayCount_));
+        swap_->setPricingEngine(shared_ptr<PricingEngine>(new
+            DiscountingSwapEngine(iborIndex_->termStructure())));
 
         Date d = calendar_.adjust(swap_->maturityDate(), Following);
         Weekday w = d.weekday();
@@ -469,7 +628,7 @@ namespace QuantLib {
         // do not set the relinkable handle as an observer -
         // force recalculation when needed
         termStructureHandle_.linkTo(
-                         boost::shared_ptr<YieldTermStructure>(t,no_deletion),
+                         shared_ptr<YieldTermStructure>(t,no_deletion),
                          false);
         RelativeDateRateHelper::setTermStructure(t);
     }
@@ -481,5 +640,13 @@ namespace QuantLib {
         return swap_->fairLiborFraction();
     }
 
-}
+    void BMASwapRateHelper::accept(AcyclicVisitor& v) {
+        Visitor<BMASwapRateHelper>* v1 =
+            dynamic_cast<Visitor<BMASwapRateHelper>*>(&v);
+        if (v1 != 0)
+            v1->visit(*this);
+        else
+            RateHelper::accept(v);
+    }
 
+}
