@@ -2,7 +2,8 @@
 
 /*
  Copyright (C) 2000, 2001, 2002, 2003 RiskMap srl
- Copyright (C) 2003, 2004, 2005, 2006, 2007 StatPro Italia srl
+ Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008 StatPro Italia srl
+ Copyright (C) 2007, 2008 Ferdinando Ametrano
  Copyright (C) 2007 Roland Lichters
 
  This file is part of QuantLib, a free-software/open-source library
@@ -26,49 +27,46 @@
 #ifndef quantlib_ratehelpers_hpp
 #define quantlib_ratehelpers_hpp
 
-#include <ql/termstructures/yieldtermstructure.hpp>
 #include <ql/termstructures/bootstraphelper.hpp>
 #include <ql/instruments/vanillaswap.hpp>
 #include <ql/instruments/bmaswap.hpp>
 #include <ql/time/calendar.hpp>
 #include <ql/time/daycounter.hpp>
-#include <ql/quotes/simplequote.hpp>
 
 namespace QuantLib {
 
     class SwapIndex;
-    class BMAIndex;
+    class Quote;
 
     typedef BootstrapHelper<YieldTermStructure> RateHelper;
 
-    //! Rate helper for bootstrapping over interest-rate futures prices
-    /*! \todo implement/refactor constructors with:
-              Index instead of (nMonths, calendar, convention, dayCounter),
-              IMM code
-    */
+    //! Rate helper for bootstrapping over IborIndex futures prices
     class FuturesRateHelper : public RateHelper {
       public:
         FuturesRateHelper(const Handle<Quote>& price,
                           const Date& immDate,
-                          Size nMonths,
+                          Natural lengthInMonths,
                           const Calendar& calendar,
                           BusinessDayConvention convention,
+                          bool endOfMonth,
                           const DayCounter& dayCounter,
-                          const Handle<Quote>& convexityAdjustment);
-        FuturesRateHelper(const Handle<Quote>& price,
-                          const Date& immDate,
-                          Size nMonths,
-                          const Calendar& calendar,
-                          BusinessDayConvention convention,
-                          const DayCounter& dayCounter,
-                          Rate convexityAdjustment = 0.0);
+                          const Handle<Quote>& convexityAdjustment = Handle<Quote>());
         FuturesRateHelper(Real price,
                           const Date& immDate,
-                          Size nMonths,
+                          Natural lengthInMonths,
                           const Calendar& calendar,
                           BusinessDayConvention convention,
+                          bool endOfMonth,
                           const DayCounter& dayCounter,
                           Rate convexityAdjustment = 0.0);
+        FuturesRateHelper(const Handle<Quote>& price,
+                          const Date& immDate,
+                          const boost::shared_ptr<IborIndex>& iborIndex,
+                          const Handle<Quote>& convexityAdjustment = Handle<Quote>());
+        FuturesRateHelper(Real price,
+                          const Date& immDate,
+                          const boost::shared_ptr<IborIndex>& iborIndex,
+                          Rate convexityAdjustment = 0.0);
         //! \name RateHelper interface
         //@{
         Real impliedQuote() const;
@@ -77,6 +75,10 @@ namespace QuantLib {
         //@{
         Real convexityAdjustment() const;
         //@}
+        //! \name Visitability
+        //@{
+        void accept(AcyclicVisitor&);
+        //@}
       private:
         Time yearFraction_;
         Handle<Quote> convAdj_;
@@ -105,30 +107,35 @@ namespace QuantLib {
       public:
         DepositRateHelper(const Handle<Quote>& rate,
                           const Period& tenor,
-                          Natural settlementDays,
+                          Natural fixingDays,
                           const Calendar& calendar,
                           BusinessDayConvention convention,
                           bool endOfMonth,
-                          Natural fixingDays,
                           const DayCounter& dayCounter);
         DepositRateHelper(Rate rate,
                           const Period& tenor,
-                          Natural settlementDays,
+                          Natural fixingDays,
                           const Calendar& calendar,
                           BusinessDayConvention convention,
                           bool endOfMonth,
-                          Natural fixingDays,
                           const DayCounter& dayCounter);
+        DepositRateHelper(const Handle<Quote>& rate,
+                          const boost::shared_ptr<IborIndex>& iborIndex);
+        DepositRateHelper(Rate rate,
+                          const boost::shared_ptr<IborIndex>& iborIndex);
         //! \name RateHelper interface
         //@{
         Real impliedQuote() const;
         void setTermStructure(YieldTermStructure*);
         //@}
+        //! \name Visitability
+        //@{
+        void accept(AcyclicVisitor&);
+        //@}
       private:
         void initializeDates();
         Date fixingDate_;
-        Natural settlementDays_;
-        boost::shared_ptr<IborIndex> index_;
+        boost::shared_ptr<IborIndex> iborIndex_;
         RelinkableHandle<YieldTermStructure> termStructureHandle_;
     };
 
@@ -139,36 +146,66 @@ namespace QuantLib {
         FraRateHelper(const Handle<Quote>& rate,
                       Natural monthsToStart,
                       Natural monthsToEnd,
-                      Natural settlementDays,
+                      Natural fixingDays,
                       const Calendar& calendar,
                       BusinessDayConvention convention,
                       bool endOfMonth,
-                      Natural fixingDays,
                       const DayCounter& dayCounter);
         FraRateHelper(Rate rate,
                       Natural monthsToStart,
                       Natural monthsToEnd,
-                      Natural settlementDays,
+                      Natural fixingDays,
                       const Calendar& calendar,
                       BusinessDayConvention convention,
                       bool endOfMonth,
-                      Natural fixingDays,
                       const DayCounter& dayCounter);
+        FraRateHelper(const Handle<Quote>& rate,
+                      Natural monthsToStart,
+                      const boost::shared_ptr<IborIndex>& iborIndex);
+        FraRateHelper(Rate rate,
+                      Natural monthsToStart,
+                      const boost::shared_ptr<IborIndex>& iborIndex);
+        FraRateHelper(const Handle<Quote>& rate,
+                      Period periodToStart,
+                      Natural lengthInMonths,
+                      Natural fixingDays,
+                      const Calendar& calendar,
+                      BusinessDayConvention convention,
+                      bool endOfMonth,
+                      const DayCounter& dayCounter);
+        FraRateHelper(Rate rate,
+                      Period periodToStart,
+                      Natural lengthInMonths,
+                      Natural fixingDays,
+                      const Calendar& calendar,
+                      BusinessDayConvention convention,
+                      bool endOfMonth,
+                      const DayCounter& dayCounter);
+        FraRateHelper(const Handle<Quote>& rate,
+                      Period periodToStart,
+                      const boost::shared_ptr<IborIndex>& iborIndex);
+        FraRateHelper(Rate rate,
+                      Period periodToStart,
+                      const boost::shared_ptr<IborIndex>& iborIndex);
         //! \name RateHelper interface
         //@{
         Real impliedQuote() const;
         void setTermStructure(YieldTermStructure*);
         //@}
+        //! \name Visitability
+        //@{
+        void accept(AcyclicVisitor&);
+        //@}
       private:
         void initializeDates();
         Date fixingDate_;
-        Natural monthsToStart_;
-        Natural settlementDays_;
-        boost::shared_ptr<IborIndex> index_;
+        Period periodToStart_;
+        boost::shared_ptr<IborIndex> iborIndex_;
         RelinkableHandle<YieldTermStructure> termStructureHandle_;
     };
 
     //! Rate helper for bootstrapping over swap rates
+    /*! \todo use input SwapIndex to create the swap */
     class SwapRateHelper : public RelativeDateRateHelper {
       public:
         SwapRateHelper(const Handle<Quote>& rate,
@@ -197,6 +234,10 @@ namespace QuantLib {
                        const boost::shared_ptr<IborIndex>& iborIndex,
                        const Handle<Quote>& spread = Handle<Quote>(),
                        const Period& fwdStart = 0*Days);
+        SwapRateHelper(Rate rate,
+                       const boost::shared_ptr<SwapIndex>& swapIndex,
+                       const Handle<Quote>& spread = Handle<Quote>(),
+                       const Period& fwdStart = 0*Days);
         //! \name RateHelper interface
         //@{
         Real impliedQuote() const;
@@ -208,6 +249,10 @@ namespace QuantLib {
         boost::shared_ptr<VanillaSwap> swap() const;
         const Period& forwardStart() const;
         //@}
+        //! \name Visitability
+        //@{
+        void accept(AcyclicVisitor&);
+        //@}
       protected:
         void initializeDates();
         Period tenor_;
@@ -234,13 +279,17 @@ namespace QuantLib {
                           BusinessDayConvention bmaConvention,
                           const DayCounter& bmaDayCount,
                           const boost::shared_ptr<BMAIndex>& bmaIndex,
-                          // libor leg
+                          // ibor leg
                           const boost::shared_ptr<IborIndex>& index);
         //! \name RateHelper interface
         //@{
         Real impliedQuote() const;
         void setTermStructure(YieldTermStructure*);
         //@}
+        //! \name Visitability
+        //@{
+        void accept(AcyclicVisitor&);
+        //@}
     protected:
         void initializeDates();
         Period tenor_;
@@ -250,7 +299,7 @@ namespace QuantLib {
         BusinessDayConvention bmaConvention_;
         DayCounter bmaDayCount_;
         boost::shared_ptr<BMAIndex> bmaIndex_;
-        boost::shared_ptr<IborIndex> index_;
+        boost::shared_ptr<IborIndex> iborIndex_;
 
         boost::shared_ptr<BMASwap> swap_;
         RelinkableHandle<YieldTermStructure> termStructureHandle_;
