@@ -1,10 +1,10 @@
 /* -*- mode: c++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
 /*
- Copyright (C) 2004, 2005, 2006, 2007 Ferdinando Ametrano
+ Copyright (C) 2004, 2005, 2006, 2007, 2008 Ferdinando Ametrano
  Copyright (C) 2006 Katiuscia Manzoni
  Copyright (C) 2000, 2001, 2002, 2003 RiskMap srl
- Copyright (C) 2003, 2004, 2005, 2006 StatPro Italia srl
+ Copyright (C) 2003, 2004, 2005, 2006, 2008 StatPro Italia srl
 
  This file is part of QuantLib, a free-software/open-source library
  for financial quantitative analysts and developers - http://quantlib.org/
@@ -45,6 +45,7 @@ namespace QuantLib {
             units_ = Months;
             length_ = 12/f;
             break;
+          case EveryFourthWeek:
           case Biweekly:
           case Weekly:
             units_ = Weeks;
@@ -54,8 +55,10 @@ namespace QuantLib {
             units_ = Days;
             length_ = 1;
             break;
+          case OtherFrequency:
+            QL_FAIL("unknown frequency");  // no point in showing 999...
           default:
-            QL_FAIL("unknown frequency (" << Integer(f));
+            QL_FAIL("unknown frequency (" << Integer(f) << ")");
         }
     }
 
@@ -67,49 +70,55 @@ namespace QuantLib {
 
         switch (units_) {
           case Years:
-            QL_REQUIRE(length==1,
-                       "cannot instantiate a Frequency from " << *this);
-            return Annual;
+            if (length == 1)
+                return Annual;
+            else
+                return OtherFrequency;
           case Months:
-            QL_REQUIRE((12%length)==0 && length<=12,
-                       "cannot instantiate a Frequency from " << *this);
-            return Frequency(12/length);
+            if (12%length == 0 && length <= 12)
+                return Frequency(12/length);
+            else
+                return OtherFrequency;
           case Weeks:
             if (length==1)
                 return Weekly;
             else if (length==2)
                 return Biweekly;
+            else if (length==4)
+                return EveryFourthWeek;
             else
-                QL_FAIL("cannot instantiate a Frequency from " << *this);
+                return OtherFrequency;
           case Days:
-            QL_REQUIRE(length==1,
-                       "cannot instantiate a Frequency from " << *this);
-            return Daily;
+            if (length==1)
+                return Daily;
+            else
+                return OtherFrequency;
           default:
-            QL_FAIL("unknown time unit (" << Integer(units_));
+            QL_FAIL("unknown time unit (" << Integer(units_) << ")");
         }
     }
 
     void Period::normalize() {
-        switch (units_) {
-          case Days:
-            if (!(length_%7)) {
-                length_/=7;
-                units_ = Weeks;
+        if (length_!=0)
+            switch (units_) {
+              case Days:
+                if (!(length_%7)) {
+                    length_/=7;
+                    units_ = Weeks;
+                }
+                break;
+              case Months:
+                if (!(length_%12)) {
+                    length_/=12;
+                    units_ = Years;
+                }
+                break;
+              case Weeks:
+              case Years:
+                break;
+              default:
+                QL_FAIL("unknown time unit (" << Integer(units_) << ")");
             }
-            break;
-          case Months:
-            if (!(length_%12)) {
-                length_/=12;
-                units_ = Years;
-            }
-            break;
-          case Weeks:
-          case Years:
-            break;
-          default:
-            QL_FAIL("unknown time unit (" << Integer(units_));
-        }
     }
 
     Period& Period::operator+=(const Period& p) {
@@ -136,7 +145,7 @@ namespace QuantLib {
                                " and " << p);
                     break;
                   default:
-                    QL_FAIL("unknown units");
+                    QL_FAIL("unknown time unit (" << Integer(p.units()) << ")");
                 }
                 break;
 
@@ -152,7 +161,7 @@ namespace QuantLib {
                                " and " << p);
                     break;
                   default:
-                    QL_FAIL("unknown units");
+                    QL_FAIL("unknown time unit (" << Integer(p.units()) << ")");
                 }
                 break;
 
@@ -169,7 +178,7 @@ namespace QuantLib {
                                " and " << p);
                     break;
                   default:
-                    QL_FAIL("unknown units");
+                    QL_FAIL("unknown time unit (" << Integer(p.units()) << ")");
                 }
                 break;
 
@@ -185,12 +194,12 @@ namespace QuantLib {
                                " and " << p);
                     break;
                   default:
-                    QL_FAIL("unknown units");
+                    QL_FAIL("unknown time unit (" << Integer(p.units()) << ")");
                 }
                 break;
 
               default:
-                QL_FAIL("unknown units");
+                QL_FAIL("unknown time unit (" << Integer(units_) << ")");
             }
         }
 
@@ -251,12 +260,80 @@ namespace QuantLib {
               case Years:
                 return std::make_pair(365*p.length(), 366*p.length());
               default:
-                QL_FAIL("Unknown units");
+                QL_FAIL("unknown time unit (" << Integer(p.units()) << ")");
             }
         }
 
     }
 
+    Real years(const Period& p) {
+        if (p.length()==0) return 0.0;
+
+        switch (p.units()) {
+          case Days:
+            QL_FAIL("cannot convert Days into Years");
+          case Weeks:
+            QL_FAIL("cannot convert Weeks into Years");
+          case Months:
+              return p.length()/12.0;
+          case Years:
+              return p.length();
+          default:
+            QL_FAIL("unknown time unit (" << Integer(p.units()) << ")");
+        }
+    }
+
+    Real months(const Period& p) {
+        if (p.length()==0) return 0.0;
+
+        switch (p.units()) {
+          case Days:
+            QL_FAIL("cannot convert Days into Months");
+          case Weeks:
+            QL_FAIL("cannot convert Weeks into Months");
+          case Months:
+              return p.length();
+          case Years:
+              return p.length()*12.0;
+          default:
+            QL_FAIL("unknown time unit (" << Integer(p.units()) << ")");
+        }
+    }
+
+    Real weeks(const Period& p) {
+        if (p.length()==0) return 0.0;
+
+        switch (p.units()) {
+          case Days:
+              return p.length()/7.0;
+          case Weeks:
+              return p.length();
+          case Months:
+            QL_FAIL("cannot convert Months into Weeks");
+          case Years:
+            QL_FAIL("cannot convert Years into Weeks");
+          default:
+            QL_FAIL("unknown time unit (" << Integer(p.units()) << ")");
+        }
+    }
+
+    Real days(const Period& p) {
+        if (p.length()==0) return 0.0;
+
+        switch (p.units()) {
+          case Days:
+              return p.length();
+          case Weeks:
+              return p.length()*7.0;
+          case Months:
+            QL_FAIL("cannot convert Months into Days");
+          case Years:
+            QL_FAIL("cannot convert Years into Days");
+          default:
+            QL_FAIL("unknown time unit (" << Integer(p.units()) << ")");
+        }
+    }
+
     bool operator<(const Period& p1, const Period& p2) {
 
         // special cases
@@ -344,7 +421,7 @@ namespace QuantLib {
               case Years:
                 return out << n << (n == 1 ? " year" : " years");
               default:
-                QL_FAIL("unknown time unit");
+                QL_FAIL("unknown time unit (" << Integer(holder.p.units()) << ")");
             }
         }
 
@@ -378,7 +455,7 @@ namespace QuantLib {
               case Years:
                 return out << n << "Y";
               default:
-                QL_FAIL("unknown time unit");
+                QL_FAIL("unknown time unit (" << Integer(holder.p.units()) << ")");
             }
         }
 
