@@ -1,7 +1,7 @@
 /* -*- mode: c++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
 /*
- Copyright (C) 2006, 2007 Ferdinando Ametrano
+ Copyright (C) 2006, 2007, 2008 Ferdinando Ametrano
  Copyright (C) 2000, 2001, 2002, 2003 RiskMap srl
 
  This file is part of QuantLib, a free-software/open-source library
@@ -19,10 +19,30 @@
 */
 
 #include <ql/time/schedule.hpp>
+#include <ql/time/imm.hpp>
 #include <ql/settings.hpp>
 
 namespace QuantLib {
 
+    namespace {
+
+        Date nextTwentieth(const Date& d, DateGeneration::Rule rule) {
+            Date result = Date(20, d.month(), d.year());
+            if (result < d)
+                result += 1*Months;
+            if (rule == DateGeneration::TwentiethIMM) {
+                Month m = result.month();
+                if (m % 3 != 0) { // not a main IMM nmonth
+                    Integer skip = 3 - m%3;
+                    result += skip*Months;
+                }
+            }
+            return result;
+        }
+
+    }
+
+
     Schedule::Schedule(const std::vector<Date>& dates,
                        const Calendar& calendar,
                        BusinessDayConvention convention)
@@ -60,10 +80,10 @@ namespace QuantLib {
                    << ") later than or equal to termination date ("
                    << terminationDate << ")");
 
-        if (tenor.units()==0)
+        if (tenor.length()==0)
             rule_ = DateGeneration::Zero;
         else
-            QL_REQUIRE(tenor.units()>0,
+            QL_REQUIRE(tenor.length()>0,
                        "non positive tenor (" << tenor << ") not allowed");
 
 
@@ -71,13 +91,21 @@ namespace QuantLib {
             switch (rule_) {
               case DateGeneration::Backward:
               case DateGeneration::Forward:
-                QL_REQUIRE(firstDate > effectiveDate && firstDate < terminationDate,
-                           "first date (" << firstDate
-                           << ") out of range [effective date (" << effectiveDate
-                           << "), termination date (" << terminationDate << ")]");
+                QL_REQUIRE(firstDate > effectiveDate &&
+                           firstDate < terminationDate,
+                           "first date (" << firstDate <<
+                           ") out of [effective (" << effectiveDate <<
+                           "), termination (" << terminationDate <<
+                           ")] date range");
+                break;
+              case DateGeneration::ThirdWednesday:
+                  QL_REQUIRE(IMM::isIMMdate(firstDate, false),
+                             "first date (" << firstDate <<
+                             ") is not an IMM date");
                 break;
               case DateGeneration::Zero:
-              case DateGeneration::ThirdWednesday:
+              case DateGeneration::Twentieth:
+              case DateGeneration::TwentiethIMM:
                 QL_FAIL("first date incompatible with " << rule_ <<
                         " date generation rule");
               default:
@@ -88,13 +116,20 @@ namespace QuantLib {
             switch (rule_) {
               case DateGeneration::Backward:
               case DateGeneration::Forward:
-                QL_REQUIRE(nextToLastDate > effectiveDate && nextToLastDate < terminationDate,
-                           "next to last date (" << nextToLastDate
-                           << ") out of range [effective date (" << effectiveDate
-                           << "), termination date (" << terminationDate << ")]");
+                QL_REQUIRE(nextToLastDate > effectiveDate &&
+                           nextToLastDate < terminationDate,
+                           "next to last date (" << nextToLastDate <<
+                           ") out of [effective (" << effectiveDate <<
+                           "), termination (" << terminationDate <<
+                           ")] date range");
                 break;
-              case DateGeneration::Zero:
               case DateGeneration::ThirdWednesday:
+                  QL_REQUIRE(IMM::isIMMdate(nextToLastDate, false),
+                             "first date (" << firstDate <<
+                             ") is not an IMM date");
+              case DateGeneration::Zero:
+              case DateGeneration::Twentieth:
+              case DateGeneration::TwentiethIMM:
                 QL_FAIL("next to last date incompatible with " << rule_ <<
                         " date generation rule");
               default:
@@ -116,11 +151,6 @@ namespace QuantLib {
             isRegular_.push_back(true);
             break;
 
-          case DateGeneration::ThirdWednesday:
-            QL_REQUIRE(!endOfMonth,
-                       "endOfMonth convention incompatible with " << rule_ <<
-                        " date generation rule");
-          // fall through
           case DateGeneration::Backward:
 
             dates_.push_back(terminationDate);
@@ -163,20 +193,36 @@ namespace QuantLib {
             }
             break;
 
+          case DateGeneration::Twentieth:
+          case DateGeneration::TwentiethIMM:
+          case DateGeneration::ThirdWednesday:
+            QL_REQUIRE(!endOfMonth,
+                       "endOfMonth convention incompatible with " << rule_ <<
+                       " date generation rule");
+          // fall through
           case DateGeneration::Forward:
 
             dates_.push_back(effectiveDate);
 
             seed = effectiveDate;
+
             if (firstDate!=Date()) {
                 dates_.push_back(firstDate);
-                Date temp = nullCalendar.advance(seed,
-                    periods*tenor_, convention, endOfMonth);
+                Date temp = nullCalendar.advance(seed, periods*tenor_,
+                                                 convention, endOfMonth);
                 if (temp!=firstDate)
                     isRegular_.push_back(false);
                 else
                     isRegular_.push_back(true);
                 seed = firstDate;
+            } else if (rule_ == DateGeneration::Twentieth ||
+                       rule_ == DateGeneration::TwentiethIMM) {
+                Date next20th = nextTwentieth(effectiveDate, rule_);
+                if (next20th != effectiveDate) {
+                    dates_.push_back(next20th);
+                    isRegular_.push_back(false);
+                    seed = next20th;
+                }
             }
 
             exitDate = terminationDate;
@@ -184,11 +230,11 @@ namespace QuantLib {
                 exitDate = nextToLastDate;
 
             while (true) {
-                Date temp = nullCalendar.advance(seed,
-                    periods*tenor_, convention, endOfMonth);
-                if (temp > exitDate)
+                Date temp = nullCalendar.advance(seed, periods*tenor_,
+                                                 convention, endOfMonth);
+                if (temp > exitDate) {
                     break;
-                else {
+                } else {
                     dates_.push_back(temp);
                     isRegular_.push_back(true);
                     ++periods;
@@ -200,8 +246,14 @@ namespace QuantLib {
 
             if (calendar.adjust(dates_.back(),terminationDateConvention)!=
                 calendar.adjust(terminationDate, terminationDateConvention)) {
-                dates_.push_back(terminationDate);
-                isRegular_.push_back(false);
+                if (rule_ == DateGeneration::Twentieth ||
+                    rule_ == DateGeneration::TwentiethIMM) {
+                    dates_.push_back(nextTwentieth(terminationDate, rule_));
+                    isRegular_.push_back(true);
+                } else {
+                    dates_.push_back(terminationDate);
+                    isRegular_.push_back(false);
+                }
             }
 
             break;
@@ -220,9 +272,13 @@ namespace QuantLib {
         for (Size i=0; i<dates_.size()-1; ++i)
             dates_[i]=calendar.adjust(dates_[i], convention);
 
-        // termination date is NOT adjusted as per ISDA specifications,
-        // unless otherwise specified in the confirmation of the deal
-        if (terminationDateConvention!=Unadjusted) {
+        // termination date is NOT adjusted as per ISDA
+        // specifications, unless otherwise specified in the
+        // confirmation of the deal or unless we're creating a CDS
+        // schedule
+        if (terminationDateConvention != Unadjusted
+            || rule_ == DateGeneration::Twentieth
+            || rule_ == DateGeneration::TwentiethIMM) {
             dates_[dates_.size()-1]=calendar.adjust(dates_[dates_.size()-1],
                                                     terminationDateConvention);
         }
@@ -254,9 +310,9 @@ namespace QuantLib {
 
     bool Schedule::isRegular(Size i) const {
         QL_REQUIRE(fullInterface_, "full interface not available");
-        QL_REQUIRE(i<=isRegular_.size(),
-                   "i (" << i << ") must be less than or equal to "
-                   << isRegular_.size());
+        QL_REQUIRE(i<=isRegular_.size() && i>0,
+                   "index (" << i << ") must be in [1, " <<
+                   isRegular_.size() <<"]");
         return isRegular_[i-1];
     }
 
@@ -271,7 +327,7 @@ namespace QuantLib {
       tenor_(tenor),
       convention_(convention), terminationDateConvention_(convention),
       rule_(DateGeneration::Backward), endOfMonth_(false),
-      stubDate_(Date()), firstDate_(Date()), nextToLastDate_(Date()) {}
+      firstDate_(Date()), nextToLastDate_(Date()) {}
 
     MakeSchedule& MakeSchedule::withTerminationDateConvention(
                                                 BusinessDayConvention conv) {
@@ -310,26 +366,9 @@ namespace QuantLib {
     }
 
     MakeSchedule::operator Schedule() const {
-        Date firstDate, nextToLastDate;
-        if (stubDate_!=Date()) {
-            switch (rule_) {
-              case DateGeneration::Zero:
-              case DateGeneration::ThirdWednesday:
-              case DateGeneration::Backward:
-                firstDate = firstDate_;
-                nextToLastDate = stubDate_;
-                break;
-              case DateGeneration::Forward:
-                firstDate = stubDate_;
-                nextToLastDate = nextToLastDate_;
-                break;
-              default:
-                QL_FAIL("unknown DateGeneration::Rule (" << Integer(rule_) << ")");
-            }
-        }
         return Schedule(effectiveDate_, terminationDate_, tenor_, calendar_,
                         convention_, terminationDateConvention_,
-                        rule_, endOfMonth_, firstDate, nextToLastDate);
+                        rule_, endOfMonth_, firstDate_, nextToLastDate_);
     }
 
 }
