@@ -45,27 +45,27 @@
 using namespace QuantLib;
 using namespace boost::unit_test_framework;
 
-QL_BEGIN_TEST_LOCALS(BondTest)
+namespace {
 
-struct CommonVars {
-    // common data
-    Calendar calendar;
-    Date today;
-    Real faceAmount;
+    struct CommonVars {
+        // common data
+        Calendar calendar;
+        Date today;
+        Real faceAmount;
 
-    // cleanup
-    SavedSettings backup;
+        // cleanup
+        SavedSettings backup;
 
-    // setup
-    CommonVars() {
-        calendar = TARGET();
-        today = calendar.adjust(Date::todaysDate());
-        Settings::instance().evaluationDate() = today;
-        faceAmount = 1000000.0;
-    }
-};
+        // setup
+        CommonVars() {
+            calendar = TARGET();
+            today = calendar.adjust(Date::todaysDate());
+            Settings::instance().evaluationDate() = today;
+            faceAmount = 1000000.0;
+        }
+    };
 
-QL_END_TEST_LOCALS(BondTest)
+}
 
 
 void BondTest::testYield() {
@@ -860,13 +860,13 @@ void BondTest::testBrazilianCached() {
 test_suite* BondTest::suite() {
     test_suite* suite = BOOST_TEST_SUITE("Bond tests");
 
-    suite->add(BOOST_TEST_CASE(&BondTest::testYield));
-    suite->add(BOOST_TEST_CASE(&BondTest::testTheoretical));
-    suite->add(BOOST_TEST_CASE(&BondTest::testCached));
-    suite->add(BOOST_TEST_CASE(&BondTest::testCachedZero));
-    suite->add(BOOST_TEST_CASE(&BondTest::testCachedFixed));
-    suite->add(BOOST_TEST_CASE(&BondTest::testCachedFloating));
-    suite->add(BOOST_TEST_CASE(&BondTest::testBrazilianCached));
+    suite->add(QUANTLIB_TEST_CASE(&BondTest::testYield));
+    suite->add(QUANTLIB_TEST_CASE(&BondTest::testTheoretical));
+    suite->add(QUANTLIB_TEST_CASE(&BondTest::testCached));
+    suite->add(QUANTLIB_TEST_CASE(&BondTest::testCachedZero));
+    suite->add(QUANTLIB_TEST_CASE(&BondTest::testCachedFixed));
+    suite->add(QUANTLIB_TEST_CASE(&BondTest::testCachedFloating));
+    suite->add(QUANTLIB_TEST_CASE(&BondTest::testBrazilianCached));
     return suite;
 }
 
