@@ -2,7 +2,7 @@
 
 /*
  Copyright (C) 2003 RiskMap srl
- Copyright (C) 2004, 2005, 2006, 2007 StatPro Italia srl
+ Copyright (C) 2004, 2005, 2006, 2007, 2008 StatPro Italia srl
 
  This file is part of QuantLib, a free-software/open-source library
  for financial quantitative analysts and developers - http://quantlib.org/
@@ -40,101 +40,101 @@
 using namespace QuantLib;
 using namespace boost::unit_test_framework;
 
-QL_BEGIN_TEST_LOCALS(CapFloorTest)
+namespace {
 
-struct CommonVars {
-    // common data
-    Date settlement;
-    std::vector<Real> nominals;
-    BusinessDayConvention convention;
-    Frequency frequency;
-    boost::shared_ptr<IborIndex> index;
-    Calendar calendar;
-    Natural fixingDays;
-    RelinkableHandle<YieldTermStructure> termStructure;
+    struct CommonVars {
+        // common data
+        Date settlement;
+        std::vector<Real> nominals;
+        BusinessDayConvention convention;
+        Frequency frequency;
+        boost::shared_ptr<IborIndex> index;
+        Calendar calendar;
+        Natural fixingDays;
+        RelinkableHandle<YieldTermStructure> termStructure;
 
-    // cleanup
+        // cleanup
 
-    SavedSettings backup;
+        SavedSettings backup;
 
-    // setup
-    CommonVars() {
-        nominals = std::vector<Real>(1,100);
-        frequency = Semiannual;
-        index = boost::shared_ptr<IborIndex>(new Euribor6M(termStructure));
-        calendar = index->fixingCalendar();
-        convention = ModifiedFollowing;
-        Date today = calendar.adjust(Date::todaysDate());
-        Settings::instance().evaluationDate() = today;
-        Natural settlementDays = 2;
-        fixingDays = 2;
-        settlement = calendar.advance(today,settlementDays,Days);
-        termStructure.linkTo(flatRate(settlement,0.05,
-                                      ActualActual(ActualActual::ISDA)));
-    }
+        // setup
+        CommonVars() {
+            nominals = std::vector<Real>(1,100);
+            frequency = Semiannual;
+            index = boost::shared_ptr<IborIndex>(new Euribor6M(termStructure));
+            calendar = index->fixingCalendar();
+            convention = ModifiedFollowing;
+            Date today = calendar.adjust(Date::todaysDate());
+            Settings::instance().evaluationDate() = today;
+            Natural settlementDays = 2;
+            fixingDays = 2;
+            settlement = calendar.advance(today,settlementDays,Days);
+            termStructure.linkTo(flatRate(settlement,0.05,
+                                          ActualActual(ActualActual::ISDA)));
+        }
 
-    // utilities
-    Leg makeLeg(const Date& startDate, Integer length) {
-        Date endDate = calendar.advance(startDate,length*Years,convention);
-        Schedule schedule(startDate, endDate, Period(frequency), calendar,
-                          convention, convention,
-                          DateGeneration::Forward, false);
-        return IborLeg(schedule, index)
-            .withNotionals(nominals)
-            .withPaymentDayCounter(index->dayCounter())
-            .withPaymentAdjustment(convention)
-            .withFixingDays(fixingDays);
-    }
+        // utilities
+        Leg makeLeg(const Date& startDate, Integer length) {
+            Date endDate = calendar.advance(startDate,length*Years,convention);
+            Schedule schedule(startDate, endDate, Period(frequency), calendar,
+                              convention, convention,
+                              DateGeneration::Forward, false);
+            return IborLeg(schedule, index)
+                .withNotionals(nominals)
+                .withPaymentDayCounter(index->dayCounter())
+                .withPaymentAdjustment(convention)
+                .withFixingDays(fixingDays);
+        }
 
-    boost::shared_ptr<PricingEngine> makeEngine(Volatility volatility) {
-        Handle<Quote> vol(boost::shared_ptr<Quote>(
+        boost::shared_ptr<PricingEngine> makeEngine(Volatility volatility) {
+            Handle<Quote> vol(boost::shared_ptr<Quote>(
                                                 new SimpleQuote(volatility)));
-        return boost::shared_ptr<PricingEngine>(
+            return boost::shared_ptr<PricingEngine>(
                                 new BlackCapFloorEngine(termStructure, vol));
+        }
+
+        boost::shared_ptr<CapFloor> makeCapFloor(CapFloor::Type type,
+                                                 const Leg& leg,
+                                                 Rate strike,
+                                                 Volatility volatility) {
+            boost::shared_ptr<CapFloor> result;
+            switch (type) {
+              case CapFloor::Cap:
+                result = boost::shared_ptr<CapFloor>(
+                                  new Cap(leg, std::vector<Rate>(1, strike)));
+                break;
+              case CapFloor::Floor:
+                result = boost::shared_ptr<CapFloor>(
+                                new Floor(leg, std::vector<Rate>(1, strike)));
+                break;
+              default:
+                QL_FAIL("unknown cap/floor type");
+            }
+            result->setPricingEngine(makeEngine(volatility));
+            return result;
+        }
+
+    };
+
+    bool checkAbsError(Real x1, Real x2, Real tolerance){
+        return std::fabs(x1 - x2) < tolerance;
     }
 
-    boost::shared_ptr<CapFloor> makeCapFloor(CapFloor::Type type,
-                                             const Leg& leg,
-                                             Rate strike,
-                                             Volatility volatility) {
-        boost::shared_ptr<CapFloor> result;
+    std::string typeToString(CapFloor::Type type) {
         switch (type) {
           case CapFloor::Cap:
-            result = boost::shared_ptr<CapFloor>(
-                new Cap(leg, std::vector<Rate>(1, strike)));
-            break;
+            return "cap";
           case CapFloor::Floor:
-            result = boost::shared_ptr<CapFloor>(
-                new Floor(leg, std::vector<Rate>(1, strike)));
-            break;
+            return "floor";
+          case CapFloor::Collar:
+            return "collar";
           default:
             QL_FAIL("unknown cap/floor type");
         }
-        result->setPricingEngine(makeEngine(volatility));
-        return result;
     }
 
-};
-
-bool checkAbsError(Real x1, Real x2, Real tolerance){
-    return std::fabs(x1 - x2) < tolerance;
 }
 
-std::string typeToString(CapFloor::Type type) {
-    switch (type) {
-      case CapFloor::Cap:
-        return "cap";
-      case CapFloor::Floor:
-        return "floor";
-      case CapFloor::Collar:
-        return "collar";
-      default:
-        QL_FAIL("unknown cap/floor type");
-    }
-}
-
-QL_END_TEST_LOCALS(CapFloorTest)
-
 
 void CapFloorTest::testVega() {
 
@@ -272,10 +272,10 @@ void CapFloorTest::testConsistency() {
           for (Size l=0; l<LENGTH(vols); l++) {
 
               Leg leg = vars.makeLeg(startDate,lengths[i]);
-              boost::shared_ptr<Instrument> cap =
+              boost::shared_ptr<CapFloor> cap =
                   vars.makeCapFloor(CapFloor::Cap,leg,
                                     cap_rates[j],vols[l]);
-              boost::shared_ptr<Instrument> floor =
+              boost::shared_ptr<CapFloor> floor =
                   vars.makeCapFloor(CapFloor::Floor,leg,
                                     floor_rates[k],vols[l]);
               Collar collar(leg,std::vector<Rate>(1,cap_rates[j]),
@@ -292,6 +292,74 @@ void CapFloorTest::testConsistency() {
                     << "    floor value:  " << floor->NPV()
                     << " at strike: " << io::rate(floor_rates[k]) << "\n"
                     << "    collar value: " << collar.NPV());
+
+
+              // test re-composition by optionlets, N.B. two per year
+              Real capletsNPV = 0.0;
+              std::vector<boost::shared_ptr<CapFloor> > caplets;
+              for (Integer m=0; m<lengths[i]*2; m++) {
+                caplets.push_back(cap->optionlet(m));
+                caplets[m]->setPricingEngine(vars.makeEngine(vols[l]));
+                capletsNPV += caplets[m]->NPV();
+              }
+
+              if (std::fabs(cap->NPV() - capletsNPV) > 1e-10) {
+                BOOST_FAIL(
+                  "sum of caplet NPVs does not equal cap NPV:\n"
+                    << "    length:       " << lengths[i] << " years\n"
+                    << "    volatility:   " << io::volatility(vols[l]) << "\n"
+                    << "    cap value:    " << cap->NPV()
+                    << " at strike: " << io::rate(cap_rates[j]) << "\n"
+                    << "    sum of caplets value:  " << capletsNPV
+                    << " at strike (first): " << io::rate(caplets[0]->capRates()[0]) << "\n"
+                );
+              }
+
+              Real floorletsNPV = 0.0;
+              std::vector<boost::shared_ptr<CapFloor> > floorlets;
+              for (Integer m=0; m<lengths[i]*2; m++) {
+                floorlets.push_back(floor->optionlet(m));
+                floorlets[m]->setPricingEngine(vars.makeEngine(vols[l]));
+                floorletsNPV += floorlets[m]->NPV();
+              }
+
+              if (std::fabs(floor->NPV() - floorletsNPV) > 1e-10) {
+                BOOST_FAIL(
+                  "sum of floorlet NPVs does not equal floor NPV:\n"
+                    << "    length:       " << lengths[i] << " years\n"
+                    << "    volatility:   " << io::volatility(vols[l]) << "\n"
+                    << "    cap value:    " << floor->NPV()
+                    << " at strike: " << io::rate(floor_rates[j]) << "\n"
+                    << "    sum of floorlets value:  " << floorletsNPV
+                    << " at strike (first): " << io::rate(floorlets[0]->floorRates()[0]) << "\n"
+                );
+              }
+
+              Real collarletsNPV = 0.0;
+              std::vector<boost::shared_ptr<CapFloor> > collarlets;
+              for (Integer m=0; m<lengths[i]*2; m++) {
+                collarlets.push_back(collar.optionlet(m));
+                collarlets[m]->setPricingEngine(vars.makeEngine(vols[l]));
+                collarletsNPV += collarlets[m]->NPV();
+              }
+
+              if (std::fabs(collar.NPV() - collarletsNPV) > 1e-10) {
+                BOOST_FAIL(
+                  "sum of collarlet NPVs does not equal floor NPV:\n"
+                    << "    length:       " << lengths[i] << " years\n"
+                    << "    volatility:   " << io::volatility(vols[l]) << "\n"
+                    << "    cap value:    " << collar.NPV()
+                    << " at strike floor: " << io::rate(floor_rates[j])
+                    << " at strike cap: " << io::rate(cap_rates[j]) << "\n"
+                    << "    sum of collarlets value:  " << collarletsNPV
+                    << " at strike floor (first): " << io::rate(collarlets[0]->floorRates()[0])
+                    << " at strike cap (first): " << io::rate(collarlets[0]->capRates()[0]) << "\n"
+                );
+              }
+
+
+
+
               }
           }
         }
@@ -419,15 +487,15 @@ void CapFloorTest::testImpliedVolatility() {
     CommonVars vars;
 
     Size maxEvaluations = 100;
-    Real tolerance = 1.0e-6;
+    Real tolerance = 1.0e-8;
 
     CapFloor::Type types[] = { CapFloor::Cap, CapFloor::Floor };
     Rate strikes[] = { 0.02, 0.03, 0.04 };
     Integer lengths[] = { 1, 5, 10 };
 
     // test data
-    Rate rRates[] = { 0.02, 0.03, 0.04 };
-    Volatility vols[] = { 0.01, 0.20, 0.30, 0.70, 0.90 };
+    Rate rRates[] = { 0.02, 0.03, 0.04, 0.05, 0.06, 0.07 };
+    Volatility vols[] = { 0.01, 0.05, 0.10, 0.20, 0.30, 0.70, 0.90 };
 
     for (Size k=0; k<LENGTH(lengths); k++) {
         Leg leg = vars.makeLeg(vars.settlement, lengths[k]);
@@ -449,20 +517,30 @@ void CapFloorTest::testImpliedVolatility() {
 
                         Real value = capfloor->NPV();
                         Volatility implVol = 0.0;
-
                         try {
                             implVol =
                                 capfloor->impliedVolatility(value,
                                                             vars.termStructure,
+                                                            0.10,
                                                             tolerance,
                                                             maxEvaluations);
                         } catch (std::exception& e) {
-                            BOOST_FAIL(typeToString(types[i]) <<
-                                "\n  strike:     " << io::rate(strikes[j]) <<
-                                "\n  risk-free:  " << io::rate(r) <<
-                                "\n  length:     " << lengths[k] << "Y" <<
-                                "\n  volatility: " << io::volatility(v) <<
-                                "\n" << e.what());
+                            // couldn't bracket?
+                            capfloor->setPricingEngine(vars.makeEngine(0.0));
+                            Real value2 = capfloor->NPV();
+                            if (std::fabs(value-value2) < tolerance) {
+                                // ok, just skip:
+                                continue;
+                            }
+                            // otherwise, report error
+                            BOOST_ERROR("implied vol failure: " <<
+                                        typeToString(types[i]) <<
+                                        "\n  strike:     " << io::rate(strikes[j]) <<
+                                        "\n  risk-free:  " << io::rate(r) <<
+                                        "\n  length:     " << lengths[k] << "Y" <<
+                                        "\n  volatility: " << io::volatility(v) <<
+                                        "\n  price:      " << value <<
+                                        "\n" << e.what());
                         }
                         if (std::fabs(implVol-v) > tolerance) {
                             // the difference might not matter
@@ -470,21 +548,15 @@ void CapFloorTest::testImpliedVolatility() {
                                                     vars.makeEngine(implVol));
                             Real value2 = capfloor->NPV();
                             if (std::fabs(value-value2) > tolerance) {
-                                BOOST_FAIL(
-                                    typeToString(types[i]) << ":\n"
-                                    << "    strike:           "
-                                    << strikes[j] << "\n"
-                                    << "    risk-free rate:   "
-                                    << io::rate(r) << "\n"
-                                    << "    length:         "
-                                    << lengths[k] << " years\n\n"
-                                    << "    original volatility: "
-                                    << io::volatility(v) << "\n"
-                                    << "    price:               "
-                                    << value << "\n"
-                                    << "    implied volatility:  "
-                                    << io::volatility(implVol) << "\n"
-                                    << "    corresponding price: " << value2);
+                            BOOST_FAIL("implied vol failure: " <<
+                                       typeToString(types[i]) <<
+                                       "\n  strike:        " << io::rate(strikes[j]) <<
+                                       "\n  risk-free:     " << io::rate(r) <<
+                                       "\n  length:        " << lengths[k] << "Y" <<
+                                       "\n  volatility:    " << io::volatility(v) <<
+                                       "\n  price:         " << value <<
+                                       "\n  implied vol:   " << io::volatility(implVol) <<
+                                       "\n  implied price: " << value2);
                             }
                         }
                     }
@@ -538,14 +610,14 @@ void CapFloorTest::testCachedValue() {
 
 test_suite* CapFloorTest::suite() {
     test_suite* suite = BOOST_TEST_SUITE("Cap/floor tests");
-    suite->add(BOOST_TEST_CASE(&CapFloorTest::testStrikeDependency));
-    suite->add(BOOST_TEST_CASE(&CapFloorTest::testConsistency));
+    suite->add(QUANTLIB_TEST_CASE(&CapFloorTest::testStrikeDependency));
+    suite->add(QUANTLIB_TEST_CASE(&CapFloorTest::testConsistency));
     // FLOATING_POINT_EXCEPTION
-    suite->add(BOOST_TEST_CASE(&CapFloorTest::testParity));
-    suite->add(BOOST_TEST_CASE(&CapFloorTest::testVega));
-    suite->add(BOOST_TEST_CASE(&CapFloorTest::testATMRate));
-    suite->add(BOOST_TEST_CASE(&CapFloorTest::testImpliedVolatility));
-    suite->add(BOOST_TEST_CASE(&CapFloorTest::testCachedValue));
+    suite->add(QUANTLIB_TEST_CASE(&CapFloorTest::testParity));
+    suite->add(QUANTLIB_TEST_CASE(&CapFloorTest::testVega));
+    suite->add(QUANTLIB_TEST_CASE(&CapFloorTest::testATMRate));
+    suite->add(QUANTLIB_TEST_CASE(&CapFloorTest::testImpliedVolatility));
+    suite->add(QUANTLIB_TEST_CASE(&CapFloorTest::testCachedValue));
     return suite;
 }
 
