@@ -43,95 +43,100 @@
 using namespace QuantLib;
 using namespace boost::unit_test_framework;
 
-QL_BEGIN_TEST_LOCALS(CurveStatesTest)
-
-
 #define BEGIN(x) (x+0)
 #define END(x) (x+LENGTH(x))
 
+namespace {
 
-Date todaysDate, startDate, endDate;
-std::vector<Time> rateTimes, paymentTimes;
-std::vector<Real> accruals;
-Calendar calendar;
-DayCounter dayCounter;
-std::vector<Rate> todaysForwards, todaysCoterminalSwapRates;
-std::vector<Real> coterminalAnnuity;
-Spread displacement;
-std::vector<DiscountFactor> todaysDiscounts;
+    struct CommonVars {
+        // global data
+        Date todaysDate, startDate, endDate;
+        std::vector<Time> rateTimes, paymentTimes;
+        std::vector<Real> accruals;
+        Calendar calendar;
+        DayCounter dayCounter;
+        std::vector<Rate> todaysForwards, todaysCoterminalSwapRates;
+        std::vector<Real> coterminalAnnuity;
+        Spread displacement;
+        std::vector<DiscountFactor> todaysDiscounts;
 
-void setup() {
-    // Times
-    calendar = NullCalendar();
-    todaysDate = Settings::instance().evaluationDate();
-    //startDate = todaysDate + 5*Years;
-    endDate = todaysDate + 10*Years;
-    Schedule dates(todaysDate, endDate, Period(Semiannual),
-                   calendar, Following, Following, DateGeneration::Backward, false);
-    rateTimes = std::vector<Time>(dates.size()-1);
-    paymentTimes = std::vector<Time>(rateTimes.size()-1);
-    accruals = std::vector<Real>(rateTimes.size()-1);
-    dayCounter = SimpleDayCounter();
-    for (Size i=1; i<dates.size(); ++i)
-        rateTimes[i-1] = dayCounter.yearFraction(todaysDate, dates[i]);
-    std::copy(rateTimes.begin()+1, rateTimes.end(), paymentTimes.begin());
-    for (Size i=1; i<rateTimes.size(); ++i)
-        accruals[i-1] = rateTimes[i] - rateTimes[i-1];
+        // cleanup
+        SavedSettings backup;
 
-    // Rates & displacement
-    todaysForwards = std::vector<Rate>(paymentTimes.size());
-    displacement = 0.0;
-    for (Size i=0; i<todaysForwards.size(); ++i)
-        todaysForwards[i] = 0.03 + 0.0010*i;
+        CommonVars() {
+            // Times
+            calendar = NullCalendar();
+            todaysDate = Settings::instance().evaluationDate();
+            //startDate = todaysDate + 5*Years;
+            endDate = todaysDate + 10*Years;
+            Schedule dates(todaysDate, endDate, Period(Semiannual),
+                           calendar, Following, Following,
+                           DateGeneration::Backward, false);
+            rateTimes = std::vector<Time>(dates.size()-1);
+            paymentTimes = std::vector<Time>(rateTimes.size()-1);
+            accruals = std::vector<Real>(rateTimes.size()-1);
+            dayCounter = SimpleDayCounter();
+            for (Size i=1; i<dates.size(); ++i)
+                rateTimes[i-1] = dayCounter.yearFraction(todaysDate, dates[i]);
+            std::copy(rateTimes.begin()+1, rateTimes.end(),
+                      paymentTimes.begin());
+            for (Size i=1; i<rateTimes.size(); ++i)
+                accruals[i-1] = rateTimes[i] - rateTimes[i-1];
 
-    // Discounts
-    todaysDiscounts = std::vector<DiscountFactor>(rateTimes.size());
-    todaysDiscounts[0] = 0.95;
-    for (Size i=1; i<rateTimes.size(); ++i)
-        todaysDiscounts[i] = todaysDiscounts[i-1] /
-            (1.0+todaysForwards[i-1]*accruals[i-1]);
+            // Rates & displacement
+            todaysForwards = std::vector<Rate>(paymentTimes.size());
+            displacement = 0.0;
+            for (Size i=0; i<todaysForwards.size(); ++i)
+                todaysForwards[i] = 0.03 + 0.0010*i;
 
-    // Coterminal swap rates & annuities
-    Size N = todaysForwards.size();
-    todaysCoterminalSwapRates = std::vector<Rate>(N);
-    coterminalAnnuity = std::vector<Real>(N);
-    Real floatingLeg = 0.0;
-    for (Size i=1; i<=N; ++i) {
-        if (i==1) {
-            coterminalAnnuity[N-1] = accruals[N-1]*todaysDiscounts[N];
-        } else {
-            coterminalAnnuity[N-i] = coterminalAnnuity[N-i+1] +
-                                     accruals[N-i]*todaysDiscounts[N-i+1];
+            // Discounts
+            todaysDiscounts = std::vector<DiscountFactor>(rateTimes.size());
+            todaysDiscounts[0] = 0.95;
+            for (Size i=1; i<rateTimes.size(); ++i)
+                todaysDiscounts[i] = todaysDiscounts[i-1] /
+                    (1.0+todaysForwards[i-1]*accruals[i-1]);
+
+            // Coterminal swap rates & annuities
+            Size N = todaysForwards.size();
+            todaysCoterminalSwapRates = std::vector<Rate>(N);
+            coterminalAnnuity = std::vector<Real>(N);
+            Real floatingLeg = 0.0;
+            for (Size i=1; i<=N; ++i) {
+                if (i==1) {
+                    coterminalAnnuity[N-1] = accruals[N-1]*todaysDiscounts[N];
+                } else {
+                    coterminalAnnuity[N-i] = coterminalAnnuity[N-i+1] +
+                        accruals[N-i]*todaysDiscounts[N-i+1];
+                }
+                floatingLeg = todaysDiscounts[N-i]-todaysDiscounts[N];
+                todaysCoterminalSwapRates[N-i] =
+                    floatingLeg/coterminalAnnuity[N-i];
+            }
+
+            std::vector<Time> evolutionTimes(rateTimes.size()-1);
+            std::copy(rateTimes.begin(), rateTimes.end()-1,
+                      evolutionTimes.begin());
+            EvolutionDescription evolution(rateTimes,evolutionTimes);
+            std::vector<Real> rateTaus = evolution.rateTaus();
+            std::vector<Size> alive = evolution.firstAliveRate();
         }
-        floatingLeg = todaysDiscounts[N-i]-todaysDiscounts[N];
-        todaysCoterminalSwapRates[N-i] = floatingLeg/coterminalAnnuity[N-i];
-    }
+    };
 
-    std::vector<Time> evolutionTimes(rateTimes.size()-1);
-    std::copy(rateTimes.begin(), rateTimes.end()-1, evolutionTimes.begin());
-    EvolutionDescription evolution(rateTimes,evolutionTimes);
-    std::vector<Real> rateTaus = evolution.rateTaus();
-    std::vector<Size> alive = evolution.firstAliveRate();
 }
 
-QL_END_TEST_LOCALS(CurveStatesTest)
 
 void CurveStatesTest::testLMMCurveState() {
 
     BOOST_MESSAGE("Testing Libor-market-model curve state...");
 
-    SavedSettings backup;
-
-    setup();
+    CommonVars vars;
 }
 
 void CurveStatesTest::testCoterminalSwapCurveState() {
 
     BOOST_MESSAGE("Testing coterminal-swap-market-model curve state...");
 
-    SavedSettings backup;
-
-    setup();
+    CommonVars vars;
 }
 
 
@@ -139,11 +144,9 @@ void CurveStatesTest::testCMSwapCurveState() {
 
     BOOST_MESSAGE("Testing constant-maturity-swap-market-model curve state...");
 
-    SavedSettings backup;
+    CommonVars vars;
 
-    setup();
-
-    Size nbRates = todaysForwards.size();
+    Size nbRates = vars.todaysForwards.size();
     Size factors = nbRates;
     Matrix pseudo(nbRates, factors, 0.1);
     std::vector<Spread> displacements(nbRates, .0);
@@ -201,8 +204,8 @@ void CurveStatesTest::testCMSwapCurveState() {
 // --- Call the desired tests
 test_suite* CurveStatesTest::suite() {
     test_suite* suite = BOOST_TEST_SUITE("Curve States tests");
-    //suite->add(BOOST_TEST_CASE(&CurveStatesTest::testLMMCurveState));
-    //suite->add(BOOST_TEST_CASE(&CurveStatesTest::testCoterminalSwapCurveState));
-    suite->add(BOOST_TEST_CASE(&CurveStatesTest::testCMSwapCurveState));
+    //suite->add(QUANTLIB_TEST_CASE(&CurveStatesTest::testLMMCurveState));
+    //suite->add(QUANTLIB_TEST_CASE(&CurveStatesTest::testCoterminalSwapCurveState));
+    suite->add(QUANTLIB_TEST_CASE(&CurveStatesTest::testCMSwapCurveState));
     return suite;
 }
