@@ -25,6 +25,7 @@
 #include <ql/math/randomnumbers/rngtraits.hpp>
 #include <ql/pricingengines/vanilla/analyticeuropeanengine.hpp>
 #include <ql/pricingengines/vanilla/binomialengine.hpp>
+#include <ql/experimental/finitedifferences/fdblackscholesvanillaengine.hpp>
 #include <ql/pricingengines/vanilla/fdeuropeanengine.hpp>
 #include <ql/pricingengines/vanilla/mceuropeanengine.hpp>
 #include <ql/pricingengines/vanilla/integralengine.hpp>
@@ -54,158 +55,159 @@ using namespace boost::unit_test_framework;
                << "    error:            " << error << "\n" \
                << "    tolerance:        " << tolerance);
 
-QL_BEGIN_TEST_LOCALS(EuropeanOptionTest)
+namespace {
 
-// utilities
+    // utilities
 
-struct EuropeanOptionData {
-    Option::Type type;
-    Real strike;
-    Real s;        // spot
-    Rate q;        // dividend
-    Rate r;        // risk-free rate
-    Time t;        // time to maturity
-    Volatility v;  // volatility
-    Real result;   // expected result
-    Real tol;      // tolerance
-};
+    struct EuropeanOptionData {
+        Option::Type type;
+        Real strike;
+        Real s;        // spot
+        Rate q;        // dividend
+        Rate r;        // risk-free rate
+        Time t;        // time to maturity
+        Volatility v;  // volatility
+        Real result;   // expected result
+        Real tol;      // tolerance
+    };
 
-enum EngineType { Analytic,
-                  JR, CRR, EQP, TGEO, TIAN, LR, JOSHI,
-                  FiniteDifferences,
-                  Integral,
-                  PseudoMonteCarlo, QuasiMonteCarlo };
+    enum EngineType { Analytic,
+                      JR, CRR, EQP, TGEO, TIAN, LR, JOSHI,
+                      FiniteDifferences,
+                      Integral,
+                      PseudoMonteCarlo, QuasiMonteCarlo };
 
-boost::shared_ptr<GeneralizedBlackScholesProcess>
-makeProcess(const boost::shared_ptr<Quote>& u,
-            const boost::shared_ptr<YieldTermStructure>& q,
-            const boost::shared_ptr<YieldTermStructure>& r,
-            const boost::shared_ptr<BlackVolTermStructure>& vol) {
-    return boost::shared_ptr<BlackScholesMertonProcess>(
+    boost::shared_ptr<GeneralizedBlackScholesProcess>
+    makeProcess(const boost::shared_ptr<Quote>& u,
+                const boost::shared_ptr<YieldTermStructure>& q,
+                const boost::shared_ptr<YieldTermStructure>& r,
+                const boost::shared_ptr<BlackVolTermStructure>& vol) {
+        return boost::shared_ptr<BlackScholesMertonProcess>(
            new BlackScholesMertonProcess(Handle<Quote>(u),
                                          Handle<YieldTermStructure>(q),
                                          Handle<YieldTermStructure>(r),
                                          Handle<BlackVolTermStructure>(vol)));
-}
+    }
 
-boost::shared_ptr<VanillaOption>
-makeOption(const boost::shared_ptr<StrikedTypePayoff>& payoff,
-           const boost::shared_ptr<Exercise>& exercise,
-           const boost::shared_ptr<Quote>& u,
-           const boost::shared_ptr<YieldTermStructure>& q,
-           const boost::shared_ptr<YieldTermStructure>& r,
-           const boost::shared_ptr<BlackVolTermStructure>& vol,
-           EngineType engineType,
-           Size binomialSteps,
-           Size samples) {
+    boost::shared_ptr<VanillaOption>
+    makeOption(const boost::shared_ptr<StrikedTypePayoff>& payoff,
+               const boost::shared_ptr<Exercise>& exercise,
+               const boost::shared_ptr<Quote>& u,
+               const boost::shared_ptr<YieldTermStructure>& q,
+               const boost::shared_ptr<YieldTermStructure>& r,
+               const boost::shared_ptr<BlackVolTermStructure>& vol,
+               EngineType engineType,
+               Size binomialSteps,
+               Size samples) {
 
-    boost::shared_ptr<GeneralizedBlackScholesProcess> stochProcess =
-        makeProcess(u,q,r,vol);
+        boost::shared_ptr<GeneralizedBlackScholesProcess> stochProcess =
+            makeProcess(u,q,r,vol);
 
-    boost::shared_ptr<PricingEngine> engine;
-    switch (engineType) {
-      case Analytic:
-        engine = boost::shared_ptr<PricingEngine>(
+        boost::shared_ptr<PricingEngine> engine;
+        switch (engineType) {
+          case Analytic:
+            engine = boost::shared_ptr<PricingEngine>(
                                     new AnalyticEuropeanEngine(stochProcess));
-        break;
-      case JR:
-        engine = boost::shared_ptr<PricingEngine>(
+            break;
+          case JR:
+            engine = boost::shared_ptr<PricingEngine>(
                         new BinomialVanillaEngine<JarrowRudd>(stochProcess,
                                                               binomialSteps));
-        break;
-      case CRR:
-        engine = boost::shared_ptr<PricingEngine>(
+            break;
+          case CRR:
+            engine = boost::shared_ptr<PricingEngine>(
                  new BinomialVanillaEngine<CoxRossRubinstein>(stochProcess,
                                                               binomialSteps));
-      case EQP:
-        engine = boost::shared_ptr<PricingEngine>(
-           new BinomialVanillaEngine<AdditiveEQPBinomialTree>(stochProcess,
+          case EQP:
+            engine = boost::shared_ptr<PricingEngine>(
+                 new BinomialVanillaEngine<AdditiveEQPBinomialTree>(
+                                                              stochProcess,
                                                               binomialSteps));
-        break;
-      case TGEO:
-        engine = boost::shared_ptr<PricingEngine>(
+            break;
+          case TGEO:
+            engine = boost::shared_ptr<PricingEngine>(
                         new BinomialVanillaEngine<Trigeorgis>(stochProcess,
                                                               binomialSteps));
-        break;
-      case TIAN:
-        engine = boost::shared_ptr<PricingEngine>(
+            break;
+          case TIAN:
+            engine = boost::shared_ptr<PricingEngine>(
                 new BinomialVanillaEngine<Tian>(stochProcess, binomialSteps));
-        break;
-      case LR:
-        engine = boost::shared_ptr<PricingEngine>(
+            break;
+          case LR:
+            engine = boost::shared_ptr<PricingEngine>(
                       new BinomialVanillaEngine<LeisenReimer>(stochProcess,
                                                               binomialSteps));
-        break;
-      case JOSHI:
-        engine = boost::shared_ptr<PricingEngine>(
+            break;
+          case JOSHI:
+            engine = boost::shared_ptr<PricingEngine>(
               new BinomialVanillaEngine<Joshi4>(stochProcess, binomialSteps));
-        break;
-      case FiniteDifferences:
-        engine = boost::shared_ptr<PricingEngine>(
+            break;
+          case FiniteDifferences:
+            engine = boost::shared_ptr<PricingEngine>(
                     new FDEuropeanEngine(stochProcess,binomialSteps,samples));
-        break;
-      case Integral:
-          engine = boost::shared_ptr<PricingEngine>(
+            break;
+          case Integral:
+            engine = boost::shared_ptr<PricingEngine>(
                                             new IntegralEngine(stochProcess));
-          break;
-      case PseudoMonteCarlo:
-        engine = MakeMCEuropeanEngine<PseudoRandom>(stochProcess)
-            .withSteps(1)
-            .withSamples(samples)
-            .withSeed(42);
-        break;
-      case QuasiMonteCarlo:
-        engine = MakeMCEuropeanEngine<LowDiscrepancy>(stochProcess)
-            .withSteps(1)
-            .withSamples(samples);
-        break;
-      default:
-        QL_FAIL("unknown engine type");
-    }
+            break;
+          case PseudoMonteCarlo:
+            engine = MakeMCEuropeanEngine<PseudoRandom>(stochProcess)
+                .withSteps(1)
+                .withSamples(samples)
+                .withSeed(42);
+            break;
+          case QuasiMonteCarlo:
+            engine = MakeMCEuropeanEngine<LowDiscrepancy>(stochProcess)
+                .withSteps(1)
+                .withSamples(samples);
+            break;
+          default:
+            QL_FAIL("unknown engine type");
+        }
 
-    boost::shared_ptr<VanillaOption> option(
+        boost::shared_ptr<VanillaOption> option(
                                         new EuropeanOption(payoff, exercise));
-    option->setPricingEngine(engine);
-    return option;
-}
-
-std::string engineTypeToString(EngineType type) {
-    switch (type) {
-      case Analytic:
-        return "analytic";
-      case JR:
-        return "Jarrow-Rudd";
-      case CRR:
-        return "Cox-Ross-Rubinstein";
-      case EQP:
-        return "EQP";
-      case TGEO:
-        return "Trigeorgis";
-      case TIAN:
-        return "Tian";
-      case LR:
-        return "LeisenReimer";
-      case JOSHI:
-        return "Joshi";
-      case FiniteDifferences:
-        return "FiniteDifferences";
-    case Integral:
-        return "Integral";
-      case PseudoMonteCarlo:
-        return "MonteCarlo";
-      case QuasiMonteCarlo:
-        return "Quasi-MonteCarlo";
-      default:
-        QL_FAIL("unknown engine type");
+        option->setPricingEngine(engine);
+        return option;
     }
-}
 
-Integer timeToDays(Time t) {
-    // FLOATING_POINT_EXCEPTION
-    return Integer(t*360+0.5);
-}
+    std::string engineTypeToString(EngineType type) {
+        switch (type) {
+          case Analytic:
+            return "analytic";
+          case JR:
+            return "Jarrow-Rudd";
+          case CRR:
+            return "Cox-Ross-Rubinstein";
+          case EQP:
+            return "EQP";
+          case TGEO:
+            return "Trigeorgis";
+          case TIAN:
+            return "Tian";
+          case LR:
+            return "LeisenReimer";
+          case JOSHI:
+            return "Joshi";
+          case FiniteDifferences:
+            return "FiniteDifferences";
+          case Integral:
+            return "Integral";
+          case PseudoMonteCarlo:
+            return "MonteCarlo";
+          case QuasiMonteCarlo:
+            return "Quasi-MonteCarlo";
+          default:
+            QL_FAIL("unknown engine type");
+        }
+    }
 
-QL_END_TEST_LOCALS(EuropeanOptionTest)
+    Integer timeToDays(Time t) {
+        // FLOATING_POINT_EXCEPTION
+        return Integer(t*360+0.5);
+    }
+
+}
 
 
 void EuropeanOptionTest::testValues() {
@@ -310,6 +312,20 @@ void EuropeanOptionTest::testValues() {
                            values[i].v, values[i].result, calculated,
                            error, tolerance);
         }
+
+        engine = boost::shared_ptr<PricingEngine>(
+                    new FdBlackScholesVanillaEngine(stochProcess,200,400));
+        option.setPricingEngine(engine);
+
+        calculated = option.NPV();
+        error = std::fabs(calculated-values[i].result);
+        tolerance = 1.0e-3;
+        if (error>tolerance) {
+            REPORT_FAILURE("value", payoff, exercise, values[i].s,
+                           values[i].q, values[i].r, today,
+                           values[i].v, values[i].result, calculated,
+                           error, tolerance);
+        }
     }
 
 }
@@ -972,48 +988,48 @@ void EuropeanOptionTest::testImpliedVolContainment() {
 
 // different engines
 
-QL_BEGIN_TEST_LOCALS(EuropeanOptionTest)
+namespace {
 
-void testEngineConsistency(EngineType engine,
-                           Size binomialSteps,
-                           Size samples,
-                           std::map<std::string,Real> tolerance,
-                           bool testGreeks = false) {
+    void testEngineConsistency(EngineType engine,
+                               Size binomialSteps,
+                               Size samples,
+                               std::map<std::string,Real> tolerance,
+                               bool testGreeks = false) {
 
-    QL_TEST_START_TIMING
+        QL_TEST_START_TIMING
 
-    std::map<std::string,Real> calculated, expected;
+            std::map<std::string,Real> calculated, expected;
 
-    // test options
-    Option::Type types[] = { Option::Call, Option::Put };
-    Real strikes[] = { 75.0, 100.0, 125.0 };
-    Integer lengths[] = { 1 };
+        // test options
+        Option::Type types[] = { Option::Call, Option::Put };
+        Real strikes[] = { 75.0, 100.0, 125.0 };
+        Integer lengths[] = { 1 };
 
-    // test data
-    Real underlyings[] = { 100.0 };
-    Rate qRates[] = { 0.00, 0.05 };
-    Rate rRates[] = { 0.01, 0.05, 0.15 };
-    Volatility vols[] = { 0.11, 0.50, 1.20 };
+        // test data
+        Real underlyings[] = { 100.0 };
+        Rate qRates[] = { 0.00, 0.05 };
+        Rate rRates[] = { 0.01, 0.05, 0.15 };
+        Volatility vols[] = { 0.11, 0.50, 1.20 };
 
-    DayCounter dc = Actual360();
-    Date today = Date::todaysDate();
+        DayCounter dc = Actual360();
+        Date today = Date::todaysDate();
 
-    boost::shared_ptr<SimpleQuote> spot(new SimpleQuote(0.0));
-    boost::shared_ptr<SimpleQuote> vol(new SimpleQuote(0.0));
-    boost::shared_ptr<BlackVolTermStructure> volTS = flatVol(today,vol,dc);
-    boost::shared_ptr<SimpleQuote> qRate(new SimpleQuote(0.0));
-    boost::shared_ptr<YieldTermStructure> qTS = flatRate(today,qRate,dc);
-    boost::shared_ptr<SimpleQuote> rRate(new SimpleQuote(0.0));
-    boost::shared_ptr<YieldTermStructure> rTS = flatRate(today,rRate,dc);
+        boost::shared_ptr<SimpleQuote> spot(new SimpleQuote(0.0));
+        boost::shared_ptr<SimpleQuote> vol(new SimpleQuote(0.0));
+        boost::shared_ptr<BlackVolTermStructure> volTS = flatVol(today,vol,dc);
+        boost::shared_ptr<SimpleQuote> qRate(new SimpleQuote(0.0));
+        boost::shared_ptr<YieldTermStructure> qTS = flatRate(today,qRate,dc);
+        boost::shared_ptr<SimpleQuote> rRate(new SimpleQuote(0.0));
+        boost::shared_ptr<YieldTermStructure> rTS = flatRate(today,rRate,dc);
 
-    for (Size i=0; i<LENGTH(types); i++) {
-        for (Size j=0; j<LENGTH(strikes); j++) {
+        for (Size i=0; i<LENGTH(types); i++) {
+          for (Size j=0; j<LENGTH(strikes); j++) {
             for (Size k=0; k<LENGTH(lengths); k++) {
               Date exDate = today + lengths[k]*360;
               boost::shared_ptr<Exercise> exercise(
                                                 new EuropeanExercise(exDate));
               boost::shared_ptr<StrikedTypePayoff> payoff(new
-                  PlainVanillaPayoff(types[i], strikes[j]));
+                                    PlainVanillaPayoff(types[i], strikes[j]));
               // reference option
               boost::shared_ptr<VanillaOption> refOption =
                   makeOption(payoff, exercise, spot, qTS, rTS, volTS,
@@ -1070,11 +1086,11 @@ void testEngineConsistency(EngineType engine,
                 }
               }
             }
+          }
         }
     }
-}
 
-QL_END_TEST_LOCALS(EuropeanOptionTest)
+}
 
 
 void EuropeanOptionTest::testJRBinomialEngines() {
@@ -1364,26 +1380,29 @@ void EuropeanOptionTest::testPriceCurve() {
 test_suite* EuropeanOptionTest::suite() {
     test_suite* suite = BOOST_TEST_SUITE("European option tests");
 
-    suite->add(BOOST_TEST_CASE(&EuropeanOptionTest::testValues));
-    suite->add(BOOST_TEST_CASE(&EuropeanOptionTest::testGreekValues));
-    suite->add(BOOST_TEST_CASE(&EuropeanOptionTest::testGreeks));
-    suite->add(BOOST_TEST_CASE(&EuropeanOptionTest::testImpliedVol));
-    suite->add(BOOST_TEST_CASE(
+    suite->add(QUANTLIB_TEST_CASE(&EuropeanOptionTest::testValues));
+    suite->add(QUANTLIB_TEST_CASE(&EuropeanOptionTest::testGreekValues));
+    suite->add(QUANTLIB_TEST_CASE(&EuropeanOptionTest::testGreeks));
+    suite->add(QUANTLIB_TEST_CASE(&EuropeanOptionTest::testImpliedVol));
+    suite->add(QUANTLIB_TEST_CASE(
                            &EuropeanOptionTest::testImpliedVolContainment));
-    suite->add(BOOST_TEST_CASE(&EuropeanOptionTest::testJRBinomialEngines));
-    suite->add(BOOST_TEST_CASE(&EuropeanOptionTest::testCRRBinomialEngines));
-    suite->add(BOOST_TEST_CASE(&EuropeanOptionTest::testEQPBinomialEngines));
-    suite->add(BOOST_TEST_CASE(&EuropeanOptionTest::testTGEOBinomialEngines));
-    suite->add(BOOST_TEST_CASE(&EuropeanOptionTest::testTIANBinomialEngines));
-    suite->add(BOOST_TEST_CASE(&EuropeanOptionTest::testLRBinomialEngines));
-    suite->add(BOOST_TEST_CASE(&EuropeanOptionTest::testJOSHIBinomialEngines));
+    suite->add(QUANTLIB_TEST_CASE(&EuropeanOptionTest::testJRBinomialEngines));
+    suite->add(QUANTLIB_TEST_CASE(&EuropeanOptionTest::testCRRBinomialEngines));
+    suite->add(QUANTLIB_TEST_CASE(&EuropeanOptionTest::testEQPBinomialEngines));
+    suite->add(QUANTLIB_TEST_CASE(
+                               &EuropeanOptionTest::testTGEOBinomialEngines));
+    suite->add(QUANTLIB_TEST_CASE(
+                               &EuropeanOptionTest::testTIANBinomialEngines));
+    suite->add(QUANTLIB_TEST_CASE(&EuropeanOptionTest::testLRBinomialEngines));
+    suite->add(QUANTLIB_TEST_CASE(
+                              &EuropeanOptionTest::testJOSHIBinomialEngines));
     // FLOATING_POINT_EXCEPTION
-    suite->add(BOOST_TEST_CASE(&EuropeanOptionTest::testFdEngines));
-    suite->add(BOOST_TEST_CASE(&EuropeanOptionTest::testIntegralEngines));
-    suite->add(BOOST_TEST_CASE(&EuropeanOptionTest::testMcEngines));
-    suite->add(BOOST_TEST_CASE(&EuropeanOptionTest::testQmcEngines));
+    suite->add(QUANTLIB_TEST_CASE(&EuropeanOptionTest::testFdEngines));
+    suite->add(QUANTLIB_TEST_CASE(&EuropeanOptionTest::testIntegralEngines));
+    suite->add(QUANTLIB_TEST_CASE(&EuropeanOptionTest::testMcEngines));
+    suite->add(QUANTLIB_TEST_CASE(&EuropeanOptionTest::testQmcEngines));
     // FLOATING_POINT_EXCEPTION
-    suite->add(BOOST_TEST_CASE(&EuropeanOptionTest::testPriceCurve));
+    suite->add(QUANTLIB_TEST_CASE(&EuropeanOptionTest::testPriceCurve));
 
     return suite;
 }
