@@ -19,10 +19,20 @@
 
 #include "hestonmodel.hpp"
 #include "utilities.hpp"
+#include <ql/experimental/finitedifferences/dividendbarrieroption.hpp>
+#include <ql/instruments/dividendvanillaoption.hpp>
 #include <ql/processes/hestonprocess.hpp>
 #include <ql/models/equity/hestonmodel.hpp>
 #include <ql/models/equity/hestonmodelhelper.hpp>
+#include <ql/pricingengines/vanilla/analyticdividendeuropeanengine.hpp>
 #include <ql/pricingengines/vanilla/analytichestonengine.hpp>
+#include <ql/pricingengines/vanilla/fdamericanengine.hpp>
+#include <ql/pricingengines/vanilla/fddividendeuropeanengine.hpp>
+#include <ql/pricingengines/vanilla/fdeuropeanengine.hpp>
+#include <ql/experimental/finitedifferences/fdhestonbarrierengine.hpp>
+#include <ql/experimental/finitedifferences/fdblackscholesbarrierengine.hpp>
+#include <ql/experimental/finitedifferences/fdblackscholesvanillaengine.hpp>
+#include <ql/experimental/finitedifferences/fdhestonvanillaengine.hpp>
 #include <ql/pricingengines/vanilla/mceuropeanhestonengine.hpp>
 #include <ql/pricingengines/blackformula.hpp>
 #include <ql/time/calendars/target.hpp>
@@ -108,7 +118,8 @@ void HestonModelTest::testBlackCalibration() {
             options[i]->setPricingEngine(engine);
 
         LevenbergMarquardt om(1e-8, 1e-8, 1e-8);
-        model->calibrate(options, om, EndCriteria(400, 40, 1.0e-8, 1.0e-8, 1.0e-8));
+        model->calibrate(options, om, EndCriteria(400, 40, 1.0e-8,
+                                                  1.0e-8, 1.0e-8));
 
         Real tolerance = 3.0e-3;
 
@@ -284,11 +295,27 @@ void HestonModelTest::testAnalyticVsBlack() {
     Real error = std::fabs(calculated - expected);
     Real tolerance = 2.0e-7;
     if (error > tolerance) {
-        BOOST_FAIL("failed to reproduce Black price"
+        BOOST_FAIL("failed to reproduce Black price with AnalyticHestonEngine"
                    << "\n    calculated: " << calculated
                    << "\n    expected:   " << expected
                    << "\n    error:      " << QL_SCIENTIFIC << error);
     }
+
+    engine = boost::shared_ptr<PricingEngine>(new FdHestonVanillaEngine(
+              boost::shared_ptr<HestonModel>(new HestonModel(process)),
+              200,200,100));
+    option.setPricingEngine(engine);
+
+    calculated = option.NPV();
+    error = std::fabs(calculated - expected);
+    tolerance = 1.0e-4;
+    if (error > tolerance) {
+        BOOST_FAIL("failed to reproduce Black price with FdHestonVanillaEngine"
+                   << "\n    calculated: " << calculated
+                   << "\n    expected:   " << expected
+                   << "\n    error:      " << QL_SCIENTIFIC << error);
+    }
+
 }
 
 
@@ -442,10 +469,184 @@ void HestonModelTest::testMcVsCached() {
     }
 }
 
+void HestonModelTest::testFdBarrierVsCached() {
+    BOOST_MESSAGE("Testing FD Barrier Heston engine against cached values...");
+
+    SavedSettings backup;
+
+    DayCounter dc = Actual360();
+    Date today = Date::todaysDate();
+
+    Handle<Quote> s0(boost::shared_ptr<Quote>(new SimpleQuote(100.0)));
+    Handle<YieldTermStructure> rTS(flatRate(today, 0.08, dc));
+    Handle<YieldTermStructure> qTS(flatRate(today, 0.04, dc));
+
+    Date exDate = today + Integer(0.5*360+0.5);
+    boost::shared_ptr<Exercise> exercise(new EuropeanExercise(exDate));
+
+    boost::shared_ptr<StrikedTypePayoff> payoff(new
+            PlainVanillaPayoff(Option::Call, 90.0));
+
+    boost::shared_ptr<HestonProcess> process(new HestonProcess(
+            rTS, qTS, s0, 0.25*0.25, 1.0, 0.25*0.25, 0.001, 0.0));
+
+    boost::shared_ptr<PricingEngine> engine;
+    engine = boost::shared_ptr<PricingEngine>(new FdHestonBarrierEngine(
+                    boost::shared_ptr<HestonModel>(new HestonModel(process)),
+                    200,400,100));
+
+    BarrierOption option(Barrier::DownOut, 95.0, 3.0, payoff, exercise);
+    option.setPricingEngine(engine);
+
+    Real calculated = option.NPV();
+    Real expected = 9.0246;
+    Real error = std::fabs(calculated-expected);
+    if (error > 1.0e-3) {
+        BOOST_FAIL("failed to reproduce cached price with FD Barrier engine"
+                   << "\n    calculated: " << calculated
+                   << "\n    expected:   " << expected
+                   << "\n    error:      " << QL_SCIENTIFIC << error);
+    }
+
+    option = BarrierOption(Barrier::DownIn, 95.0, 3.0, payoff, exercise);
+    option.setPricingEngine(engine);
+
+    calculated = option.NPV();
+    expected = 7.7627;
+    error = std::fabs(calculated-expected);
+    if (error > 1.0e-3) {
+        BOOST_FAIL("failed to reproduce cached price with FD Barrier engine"
+                   << "\n    calculated: " << calculated
+                   << "\n    expected:   " << expected
+                   << "\n    error:      " << QL_SCIENTIFIC << error);
+    }
+}
+
+void HestonModelTest::testFdVanillaVsCached() {
+    BOOST_MESSAGE("Testing FD Vanilla Heston engine against cached values...");
+
+    SavedSettings backup;
+
+    Date settlementDate(27, December, 2004);
+    Settings::instance().evaluationDate() = settlementDate;
+
+    DayCounter dayCounter = ActualActual();
+    Date exerciseDate(28, March, 2005);
+
+    boost::shared_ptr<StrikedTypePayoff> payoff(
+                                   new PlainVanillaPayoff(Option::Put, 1.05));
+    boost::shared_ptr<Exercise> exercise(new EuropeanExercise(exerciseDate));
+
+    Handle<YieldTermStructure> riskFreeTS(flatRate(0.7, dayCounter));
+    Handle<YieldTermStructure> dividendTS(flatRate(0.4, dayCounter));
+
+    Handle<Quote> s0(boost::shared_ptr<Quote>(new SimpleQuote(1.05)));
+
+    VanillaOption option(payoff, exercise);
+
+    boost::shared_ptr<HestonProcess> process(new HestonProcess(
+                   riskFreeTS, dividendTS, s0, 0.3, 1.16, 0.2, 0.8, 0.8));
+
+    boost::shared_ptr<PricingEngine> engine;
+    engine = boost::shared_ptr<PricingEngine>(new FdHestonVanillaEngine(
+                    boost::shared_ptr<HestonModel>(new HestonModel(process)),
+                    100,200,100));
+    option.setPricingEngine(engine);
+
+    Real expected = 0.0632851308977151;
+    Real calculated = option.NPV();
+    Real error = std::fabs(calculated - expected);
+    Real tolerance = 1.0e-5;
+
+    if (error > tolerance) {
+        BOOST_FAIL("failed to reproduce cached price with FD engine"
+                   << "\n    calculated: " << calculated
+                   << "\n    expected:   " << expected
+                   << "\n    error:      " << QL_SCIENTIFIC << error);
+    }
+
+    BOOST_MESSAGE("Testing FD Vanilla Heston engine for discrete dividends ...");
+
+    payoff = boost::shared_ptr<StrikedTypePayoff>(
+                          new PlainVanillaPayoff(Option::Call, 95.0));
+    s0 = Handle<Quote>(boost::shared_ptr<Quote>(new SimpleQuote(100.0)));
+
+    riskFreeTS = Handle<YieldTermStructure>(flatRate(0.05, dayCounter));
+    dividendTS = Handle<YieldTermStructure>(flatRate(0.0, dayCounter));
+
+    exerciseDate = Date(28, March, 2006);
+    exercise = boost::shared_ptr<Exercise>(new EuropeanExercise(exerciseDate));
+
+    std::vector<Date> dividendDates;
+    std::vector<Real> dividends;
+    for (Date d = settlementDate + 3*Months;
+              d < exercise->lastDate();
+              d += 6*Months) {
+        dividendDates.push_back(d);
+        dividends.push_back(1.0);
+    }
+
+    DividendVanillaOption divOption(payoff, exercise,
+                                    dividendDates, dividends);
+    process = boost::shared_ptr<HestonProcess>(new HestonProcess(
+                   riskFreeTS, dividendTS, s0, 0.04, 1.0, 0.04, 0.001, 0.0));
+    engine = boost::shared_ptr<PricingEngine>(new FdHestonVanillaEngine(
+                    boost::shared_ptr<HestonModel>(new HestonModel(process)),
+                    200,400,100));
+    divOption.setPricingEngine(engine);
+    calculated = divOption.NPV();
+    // Value calculated with an independent FD framework, validated with
+    // an independent MC framework
+    expected = 12.946;
+    error = std::fabs(calculated - expected);
+    tolerance = 5.0e-3;
+
+    if (error > tolerance) {
+        BOOST_FAIL("failed to reproduce discrete dividend price with FD engine"
+                   << "\n    calculated: " << calculated
+                   << "\n    expected:   " << expected
+                   << "\n    error:      " << QL_SCIENTIFIC << error);
+    }
+
+    BOOST_MESSAGE("Testing FD Vanilla Heston engine for american exercise ...");
+
+    dividendTS = Handle<YieldTermStructure>(flatRate(0.03, dayCounter));
+    process = boost::shared_ptr<HestonProcess>(new HestonProcess(
+                   riskFreeTS, dividendTS, s0, 0.04, 1.0, 0.04, 0.001, 0.0));
+    engine = boost::shared_ptr<PricingEngine>(new FdHestonVanillaEngine(
+                    boost::shared_ptr<HestonModel>(new HestonModel(process)),
+                    200,400,100));
+    payoff = boost::shared_ptr<StrikedTypePayoff>(
+                          new PlainVanillaPayoff(Option::Put, 95.0));
+    exercise = boost::shared_ptr<Exercise>(new AmericanExercise(
+            settlementDate, exerciseDate));
+    option = VanillaOption(payoff, exercise);
+    option.setPricingEngine(engine);
+    calculated = option.NPV();
+
+    Handle<BlackVolTermStructure> volTS(flatVol(settlementDate, 0.2,
+                                                  dayCounter));
+    boost::shared_ptr<BlackScholesMertonProcess> ref_process(
+        new BlackScholesMertonProcess(s0, dividendTS, riskFreeTS, volTS));
+    boost::shared_ptr<PricingEngine> ref_engine(
+        new FDAmericanEngine(ref_process, 200, 400));
+    option.setPricingEngine(ref_engine);
+    expected = option.NPV();
+
+    error = std::fabs(calculated - expected);
+    tolerance = 1.0e-3;
+
+    if (error > tolerance) {
+        BOOST_FAIL("failed to reproduce american option price with FD engine"
+                   << "\n    calculated: " << calculated
+                   << "\n    expected:   " << expected
+                   << "\n    error:      " << QL_SCIENTIFIC << error);
+    }
+}
 
 void HestonModelTest::testKahlJaeckelCase() {
     BOOST_MESSAGE(
-          "Testing Monte Carlo Heston engine for the Kahl-Jäckel example...");
+          "Testing MC and FD Heston engine for the Kahl-Jaeckel example...");
 
     /* Example taken from Wilmott mag (Sept. 2005).
        "Not-so-complex logarithms in the Heston model",
@@ -476,7 +677,7 @@ void HestonModelTest::testKahlJaeckelCase() {
 
     VanillaOption option(payoff, exercise);
 
-    const Real tolerance = 0.1;
+    Real tolerance = 0.1;
 
     boost::shared_ptr<PricingEngine> engine =
         MakeMCEuropeanHestonEngine<PseudoRandom>(process)
@@ -484,113 +685,190 @@ void HestonModelTest::testKahlJaeckelCase() {
         .withAntitheticVariate()
         .withTolerance(tolerance)
         .withSeed(1234);
-
     option.setPricingEngine(engine);
 
     const Real expected = 4.95212;
-    const Real calculated = option.NPV();
+          Real calculated = option.NPV();
     const Real errorEstimate = option.errorEstimate();
 
     if (std::fabs(calculated - expected) > 2.34*errorEstimate) {
-        BOOST_ERROR("Failed to reproduce cached price"
+        BOOST_ERROR("Failed to reproduce cached price with MC engine"
                     << "\n    calculated: " << calculated
                     << "\n    expected:   " << expected
                     << " +/- " << errorEstimate);
     }
 
     if (errorEstimate > tolerance) {
-        BOOST_ERROR("failed to reproduce error estimate"
+        BOOST_ERROR("failed to reproduce error estimate with MC engine"
                     << "\n    calculated: " << errorEstimate
                     << "\n    expected:   " << tolerance);
     }
+
+    engine = boost::shared_ptr<PricingEngine>(new FdHestonVanillaEngine(
+                    boost::shared_ptr<HestonModel>(new HestonModel(process)),
+                    200,400,100));
+    option.setPricingEngine(engine);
+
+    calculated = option.NPV();
+    const Real error = std::fabs(calculated - expected);
+    tolerance = 2.0e-2;
+    if (error > tolerance) {
+        BOOST_FAIL("failed to reproduce cached price with FD engine"
+                   << "\n    calculated: " << calculated
+                   << "\n    expected:   " << expected
+                   << "\n    error:      " << QL_SCIENTIFIC << error);
+    }
 }
 
+namespace {
+    struct HestonParameter {
+        Real v0, kappa, theta, sigma, rho; };
+}
 
-void HestonModelTest::testEngines() {
+void HestonModelTest::testDifferentIntegrals() {
     BOOST_MESSAGE(
-       "Testing Monte Carlo Heston engine against analytic Heston engine...");
+       "Testing different numerical Heston integration algorithms...");
 
     SavedSettings backup;
 
-    Date settlementDate(27, December, 2004);
+    const Date settlementDate(27, December, 2004);
     Settings::instance().evaluationDate() = settlementDate;
 
-    DayCounter dayCounter = ActualActual();
-    Date exerciseDate(28, July, 2005);
+    const DayCounter dayCounter = ActualActual();
 
-    boost::shared_ptr<StrikedTypePayoff> payoff(
-                                   new PlainVanillaPayoff(Option::Put, 1.05));
-    boost::shared_ptr<Exercise> exercise(new EuropeanExercise(exerciseDate));
+    Handle<YieldTermStructure> riskFreeTS(flatRate(0.05, dayCounter));
+    Handle<YieldTermStructure> dividendTS(flatRate(0.03, dayCounter));
 
-    Handle<YieldTermStructure> riskFreeTS(flatRate(0.7, dayCounter));
-    Handle<YieldTermStructure> dividendTS(flatRate(0.4, dayCounter));
+    const Real strikes[] = { 0.5, 0.7, 1.0, 1.25, 1.5, 2.0 };
+    const Integer maturities[] = { 1, 2, 3, 12, 60, 120, 360};
+    const Option::Type types[] ={ Option::Put, Option::Call };
 
-    const Real v0 = 0.8;
-    const Real theta = 0.4;
-    const Real rho = -0.8;
+    const HestonParameter equityfx      = { 0.07, 2.0, 0.04, 0.55, -0.8 };
+    const HestonParameter highCorr      = { 0.07, 1.0, 0.04, 0.55,  0.995 };
+    const HestonParameter lowVolOfVol   = { 0.07, 1.0, 0.04, 0.025, -0.75 };
+    const HestonParameter highVolOfVol  = { 0.07, 1.0, 0.04, 5.0, -0.75 };
+    const HestonParameter kappaEqSigRho = { 0.07, 0.4, 0.04, 0.5, 0.8 };
 
-    for (Real s0 = 0.5; s0 < 1.6; s0 += 0.5) {
-        for (Real kappa = 1.0; kappa < 8.0; kappa += 2.0) {
-            for (Real sigma = 0.5; sigma < 7.0; sigma += 2.0) {
+    std::vector<HestonParameter> params;
+    params.push_back(equityfx);
+    params.push_back(highCorr);
+    params.push_back(lowVolOfVol);
+    params.push_back(highVolOfVol);
+    params.push_back(kappaEqSigRho);
 
-                BOOST_MESSAGE("s0 = " << s0
-                              << ", kappa = " << kappa
-                              << ", sigma = " << sigma);
+    const Real tol[] = { 1e-3, 1e-3, 0.2, 0.01, 1e-3 };
 
-                Handle<Quote> q(boost::shared_ptr<Quote>(new SimpleQuote(s0)));
-                boost::shared_ptr<HestonProcess> process(new HestonProcess(
-                    riskFreeTS, dividendTS, q, v0, kappa, theta, sigma, rho));
+    for (std::vector<HestonParameter>::const_iterator iter = params.begin();
+         iter != params.end(); ++iter) {
 
-                VanillaOption option(payoff, exercise);
+        Handle<Quote> s0(boost::shared_ptr<Quote>(new SimpleQuote(1.0)));
+        boost::shared_ptr<HestonProcess> process(new HestonProcess(
+            riskFreeTS, dividendTS,
+            s0, iter->v0, iter->kappa,
+            iter->theta, iter->sigma, iter->rho));
 
-                boost::shared_ptr<PricingEngine> engine1 =
-                    MakeMCEuropeanHestonEngine<PseudoRandom>(process)
-                    .withStepsPerYear(1825)
-                    .withAntitheticVariate()
-                    .withSamples(20000)
-                    .withSeed(1234);
+        boost::shared_ptr<HestonModel> model(new HestonModel(process));
 
-                boost::shared_ptr<PricingEngine> engine2(
-                    new AnalyticHestonEngine(boost::shared_ptr<HestonModel>(
-                                               new HestonModel(process)),192));
+        boost::shared_ptr<AnalyticHestonEngine> lobattoEngine(
+                              new AnalyticHestonEngine(model, 1e-10,
+                                                       1000000));
+        boost::shared_ptr<AnalyticHestonEngine> laguerreEngine(
+                                        new AnalyticHestonEngine(model, 192));
+        boost::shared_ptr<AnalyticHestonEngine> legendreEngine(
+            new AnalyticHestonEngine(
+                model, AnalyticHestonEngine::Gatheral,
+                AnalyticHestonEngine::Integration::gaussLegendre(512)));
+        boost::shared_ptr<AnalyticHestonEngine> chebyshevEngine(
+            new AnalyticHestonEngine(
+                model, AnalyticHestonEngine::Gatheral,
+                AnalyticHestonEngine::Integration::gaussChebyshev(512)));
+        boost::shared_ptr<AnalyticHestonEngine> chebyshev2thEngine(
+            new AnalyticHestonEngine(
+                model, AnalyticHestonEngine::Gatheral,
+                AnalyticHestonEngine::Integration::gaussChebyshev2th(512)));
 
-                option.setPricingEngine(engine1);
-                Real calculated = option.NPV();
-                Real errorEstimate = option.errorEstimate();
+        Real maxLegendreDiff    = 0.0;
+        Real maxChebyshevDiff   = 0.0;
+        Real maxChebyshev2thDiff= 0.0;
+        Real maxLaguerreDiff    = 0.0;
 
-                option.setPricingEngine(engine2);
-                Real expected = option.NPV();
-                Real tolerance = 7.5e-4;
+        for (Size i=0; i < LENGTH(maturities); ++i) {
+            boost::shared_ptr<Exercise> exercise(
+                new EuropeanExercise(settlementDate
+                                     + Period(maturities[i], Months)));
+
+            for (Size j=0; j < LENGTH(strikes); ++j) {
+                for (Size k=0; k < LENGTH(types); ++k) {
+
+                    boost::shared_ptr<StrikedTypePayoff> payoff(
+                        new PlainVanillaPayoff(types[k], strikes[j]));
+
+                    VanillaOption option(payoff, exercise);
+
+                    option.setPricingEngine(lobattoEngine);
+                    const Real lobattoNPV = option.NPV();
+
+                    option.setPricingEngine(laguerreEngine);
+                    const Real laguerre = option.NPV();
+
+                    option.setPricingEngine(legendreEngine);
+                    const Real legendre = option.NPV();
+
+                    option.setPricingEngine(chebyshevEngine);
+                    const Real chebyshev = option.NPV();
+
+                    option.setPricingEngine(chebyshev2thEngine);
+                    const Real chebyshev2th = option.NPV();
+
+                    maxLaguerreDiff
+                        = std::max(maxLaguerreDiff,
+                                   std::fabs(lobattoNPV-laguerre));
+                    maxLegendreDiff
+                        = std::max(maxLegendreDiff,
+                                   std::fabs(lobattoNPV-legendre));
+                    maxChebyshevDiff
+                        = std::max(maxChebyshevDiff,
+                                   std::fabs(lobattoNPV-chebyshev));
+                    maxChebyshev2thDiff
+                        = std::max(maxChebyshev2thDiff,
+                                   std::fabs(lobattoNPV-chebyshev2th));
 
-                if (std::fabs(calculated - expected) > 1.65*errorEstimate) {
-                    BOOST_ERROR("failed to match results from engines"
-                                << "\n    analytic:    " << expected
-                                << "\n    Monte Carlo: " << calculated
-                                << " +/- " << errorEstimate);
-                }
-                if (errorEstimate > 2.0*tolerance) {
-                    BOOST_ERROR("failed to reproduce error estimate"
-                                << "\n    calculated: " << errorEstimate
-                                << "\n    expected:   " << tolerance);
                 }
             }
         }
+        const Real maxDiff = std::max(std::max(
+            std::max(maxLaguerreDiff,maxLegendreDiff),
+                                     maxChebyshevDiff), maxChebyshev2thDiff);
+
+        const Real tr = tol[iter - params.begin()];
+        if (maxDiff > tr) {
+            BOOST_ERROR("Failed to reproduce Heston pricing values "
+                        "within given tolerance"
+                        << "\n    maxDifference: " << maxDiff
+                        << "\n    tolerance:     " << tr);
+        }
     }
 }
 
 test_suite* HestonModelTest::suite() {
     test_suite* suite = BOOST_TEST_SUITE("Heston model tests");
+
     // FLOATING_POINT_EXCEPTION
-    suite->add(BOOST_TEST_CASE(&HestonModelTest::testBlackCalibration));
+    suite->add(QUANTLIB_TEST_CASE(&HestonModelTest::testBlackCalibration));
     // FLOATING_POINT_EXCEPTION
-    suite->add(BOOST_TEST_CASE(&HestonModelTest::testDAXCalibration));
+    suite->add(QUANTLIB_TEST_CASE(&HestonModelTest::testDAXCalibration));
     // FLOATING_POINT_EXCEPTION
-    suite->add(BOOST_TEST_CASE(&HestonModelTest::testAnalyticVsBlack));
-    suite->add(BOOST_TEST_CASE(&HestonModelTest::testAnalyticVsCached));
-    suite->add(BOOST_TEST_CASE(&HestonModelTest::testKahlJaeckelCase));
+    suite->add(QUANTLIB_TEST_CASE(&HestonModelTest::testAnalyticVsBlack));
+    suite->add(QUANTLIB_TEST_CASE(&HestonModelTest::testAnalyticVsCached));
+    suite->add(QUANTLIB_TEST_CASE(&HestonModelTest::testKahlJaeckelCase));
+    suite->add(QUANTLIB_TEST_CASE(&HestonModelTest::testDifferentIntegrals));
+
     // this passes but takes way too long
-    // suite->add(BOOST_TEST_CASE(&HestonModelTest::testEngines));
-    suite->add(BOOST_TEST_CASE(&HestonModelTest::testMcVsCached));
+    //suite->add(QUANTLIB_TEST_CASE(&HestonModelTest::testEngines));
+    //suite->add(QUANTLIB_TEST_CASE(&HestonModelTest::testMcVsCached));
+    //suite->add(QUANTLIB_TEST_CASE(&HestonModelTest::testFdBarrierVsCached));
+    //suite->add(QUANTLIB_TEST_CASE(&HestonModelTest::testFdVanillaVsCached));
+
     return suite;
 }
 
