@@ -75,6 +75,8 @@ namespace {
 void InflationTest::testYoYTermStructure() {
     BOOST_MESSAGE("Testing year-on-year inflation term structure...");
 
+    SavedSettings backup;
+
     // try the YY UK
     Calendar calendar = UnitedKingdom();
     BusinessDayConvention bdc = ModifiedFollowing;
@@ -179,6 +181,8 @@ void InflationTest::testYoYTermStructure() {
 void InflationTest::testZeroTermStructure() {
     BOOST_MESSAGE("Testing zero inflation term structure...");
 
+    SavedSettings backup;
+
     // try the Zero UK
     Calendar calendar = UnitedKingdom();
     BusinessDayConvention bdc = ModifiedFollowing;
@@ -284,6 +288,8 @@ void InflationTest::testZeroTermStructure() {
 void InflationTest::testYYIndex() {
     BOOST_MESSAGE("Testing year-on-year inflation indices...");
 
+    SavedSettings backup;
+
     YYEUHICP yyeuhicp(Monthly, false, false);
     if (yyeuhicp.name() != "EU YY_HICP"
         || yyeuhicp.frequency() != Monthly
@@ -403,6 +409,8 @@ void InflationTest::testYYIndex() {
 void InflationTest::testZeroIndex() {
     BOOST_MESSAGE("Testing zero inflation indices...");
 
+    SavedSettings backup;
+
     EUHICP euhicp(Monthly, false, false);
     if (euhicp.name() != "EU HICP"
         || euhicp.frequency() != Monthly
@@ -483,10 +491,10 @@ test_suite* InflationTest::suite() {
 
     test_suite* suite = BOOST_TEST_SUITE("Inflation tests");
 
-    suite->add(BOOST_TEST_CASE(&InflationTest::testZeroIndex));
-    suite->add(BOOST_TEST_CASE(&InflationTest::testYYIndex));
-    suite->add(BOOST_TEST_CASE(&InflationTest::testZeroTermStructure));
-    suite->add(BOOST_TEST_CASE(&InflationTest::testYoYTermStructure));
+    suite->add(QUANTLIB_TEST_CASE(&InflationTest::testZeroIndex));
+    suite->add(QUANTLIB_TEST_CASE(&InflationTest::testYYIndex));
+    suite->add(QUANTLIB_TEST_CASE(&InflationTest::testZeroTermStructure));
+    suite->add(QUANTLIB_TEST_CASE(&InflationTest::testYoYTermStructure));
 
     return suite;
 }
