@@ -26,7 +26,7 @@
 #include <ql/math/interpolations/linearinterpolation.hpp>
 #include <ql/math/interpolations/backwardflatinterpolation.hpp>
 #include <ql/math/interpolations/forwardflatinterpolation.hpp>
-#include <ql/math/interpolations/cubicspline.hpp>
+#include <ql/math/interpolations/cubicinterpolation.hpp>
 #include <ql/math/interpolations/multicubicspline.hpp>
 #include <ql/math/interpolations/sabrinterpolation.hpp>
 #include <ql/math/integrals/simpsonintegral.hpp>
@@ -37,149 +37,149 @@
 using namespace QuantLib;
 using namespace boost::unit_test_framework;
 
-QL_BEGIN_TEST_LOCALS(InterpolationTest)
-
 #define BEGIN(x) (x+0)
 #define END(x) (x+LENGTH(x))
 
-std::vector<Real> xRange(Real start, Real finish, Size points) {
-    std::vector<Real> x(points);
-    Real dx = (finish-start)/(points-1);
-    for (Size i=0; i<points-1; i++)
-        x[i] = start+i*dx;
-    x[points-1] = finish;
-    return x;
-}
+namespace {
 
-std::vector<Real> gaussian(const std::vector<Real>& x) {
-    std::vector<Real> y(x.size());
-    for (Size i=0; i<x.size(); i++)
-        y[i] = std::exp(-x[i]*x[i]);
-    return y;
-}
+    std::vector<Real> xRange(Real start, Real finish, Size points) {
+        std::vector<Real> x(points);
+        Real dx = (finish-start)/(points-1);
+        for (Size i=0; i<points-1; i++)
+            x[i] = start+i*dx;
+        x[points-1] = finish;
+        return x;
+    }
 
-std::vector<Real> parabolic(const std::vector<Real>& x) {
-    std::vector<Real> y(x.size());
-    for (Size i=0; i<x.size(); i++)
-        y[i] = -x[i]*x[i];
-    return y;
-}
+    std::vector<Real> gaussian(const std::vector<Real>& x) {
+        std::vector<Real> y(x.size());
+        for (Size i=0; i<x.size(); i++)
+            y[i] = std::exp(-x[i]*x[i]);
+        return y;
+    }
 
-template <class I, class J>
-void checkValues(const char* type,
-                 const CubicSplineInterpolation& spline,
-                 I xBegin, I xEnd, J yBegin) {
-    Real tolerance = 2.0e-15;
-    while (xBegin != xEnd) {
-        Real interpolated = spline(*xBegin);
-        if (std::fabs(interpolated-*yBegin) > tolerance) {
-            BOOST_ERROR(type << " interpolation failed at x = " << *xBegin
-                        << QL_SCIENTIFIC
+    std::vector<Real> parabolic(const std::vector<Real>& x) {
+        std::vector<Real> y(x.size());
+        for (Size i=0; i<x.size(); i++)
+            y[i] = -x[i]*x[i];
+        return y;
+    }
+
+    template <class I, class J>
+    void checkValues(const char* type,
+                     const CubicInterpolation& cubic,
+                     I xBegin, I xEnd, J yBegin) {
+        Real tolerance = 2.0e-15;
+        while (xBegin != xEnd) {
+            Real interpolated = cubic(*xBegin);
+            if (std::fabs(interpolated-*yBegin) > tolerance) {
+                BOOST_ERROR(type << " interpolation failed at x = " << *xBegin
+                            << QL_SCIENTIFIC
+                            << "\n    interpolated value: " << interpolated
+                            << "\n    expected value:     " << *yBegin
+                            << "\n    error:              "
+                            << std::fabs(interpolated-*yBegin));
+            }
+            ++xBegin; ++yBegin;
+        }
+    }
+
+    void check1stDerivativeValue(const char* type,
+                                 const CubicInterpolation& cubic,
+                                 Real x,
+                                 Real value) {
+        Real tolerance = 1.0e-14;
+        Real interpolated = cubic.derivative(x);
+        Real error = std::fabs(interpolated-value);
+        if (error > tolerance) {
+            BOOST_ERROR(type << " interpolation first derivative failure\n"
+                        << "at x = " << x
                         << "\n    interpolated value: " << interpolated
-                        << "\n    expected value:     " << *yBegin
-                        << "\n    error:              "
-                        << std::fabs(interpolated-*yBegin));
-        }
-        ++xBegin; ++yBegin;
-    }
-}
-
-void check1stDerivativeValue(const char* type,
-                             const CubicSplineInterpolation& spline,
-                             Real x,
-                             Real value) {
-    Real tolerance = 1.0e-14;
-    Real interpolated = spline.derivative(x);
-    Real error = std::fabs(interpolated-value);
-    if (error > tolerance) {
-        BOOST_ERROR(type << " interpolation first derivative failure\n"
-                    << "at x = " << x
-                    << "\n    interpolated value: " << interpolated
-                    << "\n    expected value:     " << value
-                    << QL_SCIENTIFIC
-                    << "\n    error:              " << error);
-    }
-}
-
-void check2ndDerivativeValue(const char* type,
-                             const CubicSplineInterpolation& spline,
-                             Real x,
-                             Real value) {
-    Real tolerance = 1.0e-13;
-    Real interpolated = spline.secondDerivative(x);
-    Real error = std::fabs(interpolated-value);
-    if (error > tolerance) {
-        BOOST_ERROR(type << " interpolation second derivative failure\n"
-                    << "at x = " << x
-                    << "\n    interpolated value: " << interpolated
-                    << "\n    expected value:     " << value
-                    << QL_SCIENTIFIC
-                    << "\n    error:              " << error);
-    }
-}
-
-void checkNotAKnotCondition(const char* type,
-                            const CubicSplineInterpolation& spline) {
-    Real tolerance = 1.0e-14;
-    const std::vector<Real>& c = spline.cCoefficients();
-    if (std::fabs(c[0]-c[1]) > tolerance) {
-        BOOST_ERROR(type << " interpolation failure"
-                    << "\n    cubic coefficient of the first"
-                    << " polinomial is " << c[0]
-                    << "\n    cubic coefficient of the second"
-                    << " polinomial is " << c[1]);
-    }
-    Size n = c.size();
-    if (std::fabs(c[n-2]-c[n-1]) > tolerance) {
-        BOOST_ERROR(type << " interpolation failure"
-                    << "\n    cubic coefficient of the 2nd to last"
-                    << " polinomial is " << c[n-2]
-                    << "\n    cubic coefficient of the last"
-                    << " polinomial is " << c[n-1]);
-    }
-}
-
-void checkSymmetry(const char* type,
-                   const CubicSplineInterpolation& spline,
-                   Real xMin) {
-    Real tolerance = 1.0e-15;
-    for (Real x = xMin; x < 0.0; x += 0.1) {
-        Real y1 = spline(x), y2 = spline(-x);
-        if (std::fabs(y1-y2) > tolerance) {
-            BOOST_ERROR(type << " interpolation not symmetric"
-                        << "\n    x = " << x
-                        << "\n    g(x)  = " << y1
-                        << "\n    g(-x) = " << y2
-                        << "\n    error:  " << std::fabs(y1-y2));
+                        << "\n    expected value:     " << value
+                        << QL_SCIENTIFIC
+                        << "\n    error:              " << error);
         }
     }
-}
 
-template <class F>
-class errorFunction : public std::unary_function<Real,Real> {
-  public:
-    errorFunction(const F& f) : f_(f) {}
-    Real operator()(Real x) const {
-        Real temp = f_(x)-std::exp(-x*x);
-        return temp*temp;
+    void check2ndDerivativeValue(const char* type,
+                                 const CubicInterpolation& cubic,
+                                 Real x,
+                                 Real value) {
+        Real tolerance = 1.0e-13;
+        Real interpolated = cubic.secondDerivative(x);
+        Real error = std::fabs(interpolated-value);
+        if (error > tolerance) {
+            BOOST_ERROR(type << " interpolation second derivative failure\n"
+                        << "at x = " << x
+                        << "\n    interpolated value: " << interpolated
+                        << "\n    expected value:     " << value
+                        << QL_SCIENTIFIC
+                        << "\n    error:              " << error);
+        }
+    }
+
+    void checkNotAKnotCondition(const char* type,
+                                const CubicInterpolation& cubic) {
+        Real tolerance = 1.0e-14;
+        const std::vector<Real>& c = cubic.cCoefficients();
+        if (std::fabs(c[0]-c[1]) > tolerance) {
+            BOOST_ERROR(type << " interpolation failure"
+                        << "\n    cubic coefficient of the first"
+                        << " polinomial is " << c[0]
+                        << "\n    cubic coefficient of the second"
+                        << " polinomial is " << c[1]);
+        }
+        Size n = c.size();
+        if (std::fabs(c[n-2]-c[n-1]) > tolerance) {
+            BOOST_ERROR(type << " interpolation failure"
+                        << "\n    cubic coefficient of the 2nd to last"
+                        << " polinomial is " << c[n-2]
+                        << "\n    cubic coefficient of the last"
+                        << " polinomial is " << c[n-1]);
+        }
+    }
+
+    void checkSymmetry(const char* type,
+                       const CubicInterpolation& cubic,
+                       Real xMin) {
+        Real tolerance = 1.0e-15;
+        for (Real x = xMin; x < 0.0; x += 0.1) {
+            Real y1 = cubic(x), y2 = cubic(-x);
+            if (std::fabs(y1-y2) > tolerance) {
+                BOOST_ERROR(type << " interpolation not symmetric"
+                            << "\n    x = " << x
+                            << "\n    g(x)  = " << y1
+                            << "\n    g(-x) = " << y2
+                            << "\n    error:  " << std::fabs(y1-y2));
+            }
+        }
+    }
+
+    template <class F>
+    class errorFunction : public std::unary_function<Real,Real> {
+      public:
+        errorFunction(const F& f) : f_(f) {}
+        Real operator()(Real x) const {
+            Real temp = f_(x)-std::exp(-x*x);
+            return temp*temp;
+        }
+      private:
+        F f_;
+    };
+
+    template <class F>
+    errorFunction<F> make_error_function(const F& f) {
+        return errorFunction<F>(f);
+    }
+
+    Real multif(Real s, Real t, Real u, Real v, Real w) {
+        return std::sqrt(s * std::sinh(std::log(t)) +
+                         std::exp(std::sin(u) * std::sin(3 * v)) +
+                         std::sinh(std::log(v * w)));
     }
-  private:
-    F f_;
-};
 
-template <class F>
-errorFunction<F> make_error_function(const F& f) {
-    return errorFunction<F>(f);
 }
 
-Real multif(Real s, Real t, Real u, Real v, Real w) {
-    return std::sqrt(s * std::sinh(std::log(t)) +
-                     std::exp(std::sin(u) * std::sin(3 * v)) +
-                     std::sinh(std::log(v * w)));
-}
-
-QL_END_TEST_LOCALS(InterpolationTest)
-
 
 /* See J. M. Hyman, "Accurate monotonicity preserving cubic interpolation"
    SIAM J. of Scientific and Statistical Computing, v. 4, 1983, pp. 645-654.
@@ -215,10 +215,10 @@ void InterpolationTest::testSplineErrorOnGaussianValues() {
         std::vector<Real> y = gaussian(x);
 
         // Not-a-knot
-        CubicSplineInterpolation f(x.begin(), x.end(), y.begin(),
-                      CubicSplineInterpolation::NotAKnot, Null<Real>(),
-                      CubicSplineInterpolation::NotAKnot, Null<Real>(),
-                      false);
+        CubicInterpolation f(x.begin(), x.end(), y.begin(),
+                             CubicInterpolation::Spline, false,
+                             CubicInterpolation::NotAKnot, Null<Real>(),
+                             CubicInterpolation::NotAKnot, Null<Real>());
         f.update();
         Real result = std::sqrt(integral(make_error_function(f), -1.7, 1.9));
         result /= scaleFactor;
@@ -229,9 +229,10 @@ void InterpolationTest::testSplineErrorOnGaussianValues() {
                         << "\n    it should be:       " << tabulatedErrors[i]);
 
         // MC not-a-knot
-        f = MonotonicCubicSpline(x.begin(), x.end(), y.begin(),
-                                 CubicSplineInterpolation::NotAKnot, Null<Real>(),
-                                 CubicSplineInterpolation::NotAKnot, Null<Real>());
+        f = CubicInterpolation(x.begin(), x.end(), y.begin(),
+                               CubicInterpolation::Spline, true,
+                               CubicInterpolation::NotAKnot, Null<Real>(),
+                               CubicInterpolation::NotAKnot, Null<Real>());
         f.update();
         result = std::sqrt(integral(make_error_function(f), -1.7, 1.9));
         result /= scaleFactor;
@@ -264,10 +265,10 @@ void InterpolationTest::testSplineOnGaussianValues() {
         y = gaussian(x);
 
         // Not-a-knot spline
-        CubicSplineInterpolation f(x.begin(), x.end(), y.begin(),
-                      CubicSplineInterpolation::NotAKnot, Null<Real>(),
-                      CubicSplineInterpolation::NotAKnot, Null<Real>(),
-                      false);
+        CubicInterpolation f(x.begin(), x.end(), y.begin(),
+                             CubicInterpolation::Spline, false,
+                             CubicInterpolation::NotAKnot, Null<Real>(),
+                             CubicInterpolation::NotAKnot, Null<Real>());
         f.update();
         checkValues("Not-a-knot spline", f,
                     x.begin(), x.end(), y.begin());
@@ -286,9 +287,10 @@ void InterpolationTest::testSplineOnGaussianValues() {
         }
 
         // MC not-a-knot spline
-        f = MonotonicCubicSpline(x.begin(), x.end(), y.begin(),
-                                 CubicSplineInterpolation::NotAKnot, Null<Real>(),
-                                 CubicSplineInterpolation::NotAKnot, Null<Real>());
+        f = CubicInterpolation(x.begin(), x.end(), y.begin(),
+                               CubicInterpolation::Spline, true,
+                               CubicInterpolation::NotAKnot, Null<Real>(),
+                               CubicInterpolation::NotAKnot, Null<Real>());
         f.update();
         checkValues("MC not-a-knot spline", f,
                     x.begin(), x.end(), y.begin());
@@ -333,8 +335,12 @@ void InterpolationTest::testSplineOnRPN15AValues() {
     Real interpolated;
 
     // Natural spline
-    CubicSplineInterpolation f = NaturalCubicSpline(BEGIN(RPN15A_x), END(RPN15A_x),
-                                       BEGIN(RPN15A_y));
+    CubicInterpolation f = CubicInterpolation(
+                                    BEGIN(RPN15A_x), END(RPN15A_x),
+                                    BEGIN(RPN15A_y),
+                                    CubicInterpolation::Spline, false,
+                                    CubicInterpolation::SecondDerivative, 0.0,
+                                    CubicInterpolation::SecondDerivative, 0.0);
     f.update();
     checkValues("Natural spline", f,
                 BEGIN(RPN15A_x), END(RPN15A_x), BEGIN(RPN15A_y));
@@ -355,10 +361,10 @@ void InterpolationTest::testSplineOnRPN15AValues() {
 
 
     // Clamped spline
-    f = CubicSplineInterpolation(BEGIN(RPN15A_x), END(RPN15A_x), BEGIN(RPN15A_y),
-                    CubicSplineInterpolation::FirstDerivative, 0.0,
-                    CubicSplineInterpolation::FirstDerivative, 0.0,
-                    false);
+    f = CubicInterpolation(BEGIN(RPN15A_x), END(RPN15A_x), BEGIN(RPN15A_y),
+                           CubicInterpolation::Spline, false,
+                           CubicInterpolation::FirstDerivative, 0.0,
+                           CubicInterpolation::FirstDerivative, 0.0);
     f.update();
     checkValues("Clamped spline", f,
                 BEGIN(RPN15A_x), END(RPN15A_x), BEGIN(RPN15A_y));
@@ -378,10 +384,10 @@ void InterpolationTest::testSplineOnRPN15AValues() {
 
 
     // Not-a-knot spline
-    f = CubicSplineInterpolation(BEGIN(RPN15A_x), END(RPN15A_x), BEGIN(RPN15A_y),
-                    CubicSplineInterpolation::NotAKnot, Null<Real>(),
-                    CubicSplineInterpolation::NotAKnot, Null<Real>(),
-                    false);
+    f = CubicInterpolation(BEGIN(RPN15A_x), END(RPN15A_x), BEGIN(RPN15A_y),
+                           CubicInterpolation::Spline, false, 
+                           CubicInterpolation::NotAKnot, Null<Real>(),
+                           CubicInterpolation::NotAKnot, Null<Real>());
     f.update();
     checkValues("Not-a-knot spline", f,
                 BEGIN(RPN15A_x), END(RPN15A_x), BEGIN(RPN15A_y));
@@ -398,8 +404,11 @@ void InterpolationTest::testSplineOnRPN15AValues() {
 
 
     // MC natural spline values
-    f = NaturalMonotonicCubicSpline(BEGIN(RPN15A_x), END(RPN15A_x),
-                                    BEGIN(RPN15A_y));
+    f = CubicInterpolation(BEGIN(RPN15A_x), END(RPN15A_x),
+                           BEGIN(RPN15A_y),
+                           CubicInterpolation::Spline, true,
+                           CubicInterpolation::SecondDerivative, 0.0,
+                           CubicInterpolation::SecondDerivative, 0.0);
     f.update();
     checkValues("MC natural spline", f,
                 BEGIN(RPN15A_x), END(RPN15A_x), BEGIN(RPN15A_y));
@@ -415,9 +424,10 @@ void InterpolationTest::testSplineOnRPN15AValues() {
 
 
     // MC clamped spline values
-    f = MonotonicCubicSpline(BEGIN(RPN15A_x), END(RPN15A_x), BEGIN(RPN15A_y),
-                             CubicSplineInterpolation::FirstDerivative, 0.0,
-                             CubicSplineInterpolation::FirstDerivative, 0.0);
+    f = CubicInterpolation(BEGIN(RPN15A_x), END(RPN15A_x), BEGIN(RPN15A_y),
+                           CubicInterpolation::Spline, true,
+                           CubicInterpolation::FirstDerivative, 0.0,
+                           CubicInterpolation::FirstDerivative, 0.0);
     f.update();
     checkValues("MC clamped spline", f,
                 BEGIN(RPN15A_x), END(RPN15A_x), BEGIN(RPN15A_y));
@@ -437,9 +447,10 @@ void InterpolationTest::testSplineOnRPN15AValues() {
 
 
     // MC not-a-knot spline values
-    f = MonotonicCubicSpline(BEGIN(RPN15A_x), END(RPN15A_x), BEGIN(RPN15A_y),
-                             CubicSplineInterpolation::NotAKnot, Null<Real>(),
-                             CubicSplineInterpolation::NotAKnot, Null<Real>());
+    f = CubicInterpolation(BEGIN(RPN15A_x), END(RPN15A_x), BEGIN(RPN15A_y),
+                           CubicInterpolation::Spline, true,
+                           CubicInterpolation::NotAKnot, Null<Real>(),
+                           CubicInterpolation::NotAKnot, Null<Real>());
     f.update();
     checkValues("MC not-a-knot spline", f,
                 BEGIN(RPN15A_x), END(RPN15A_x), BEGIN(RPN15A_y));
@@ -471,13 +482,13 @@ void InterpolationTest::testSplineOnGenericValues() {
     std::vector<Real> x35(3);
 
     // Natural spline
-    CubicSplineInterpolation f(BEGIN(generic_x), END(generic_x),
-                  BEGIN(generic_y),
-                  CubicSplineInterpolation::SecondDerivative,
-                  generic_natural_y2[0],
-                  CubicSplineInterpolation::SecondDerivative,
-                  generic_natural_y2[n-1],
-                  false);
+    CubicInterpolation f(BEGIN(generic_x), END(generic_x),
+                         BEGIN(generic_y),
+                         CubicInterpolation::Spline, false,
+                         CubicInterpolation::SecondDerivative,
+                         generic_natural_y2[0],
+                         CubicInterpolation::SecondDerivative,
+                         generic_natural_y2[n-1]);
     f.update();
     checkValues("Natural spline", f,
                 BEGIN(generic_x), END(generic_x), BEGIN(generic_y));
@@ -498,10 +509,10 @@ void InterpolationTest::testSplineOnGenericValues() {
 
     // Clamped spline
     Real y1a = 0.0, y1b = 0.0;
-    f = CubicSplineInterpolation(BEGIN(generic_x), END(generic_x), BEGIN(generic_y),
-                    CubicSplineInterpolation::FirstDerivative, y1a,
-                    CubicSplineInterpolation::FirstDerivative, y1b,
-                    false);
+    f = CubicInterpolation(BEGIN(generic_x), END(generic_x), BEGIN(generic_y),
+                    CubicInterpolation::Spline, false,
+                    CubicInterpolation::FirstDerivative, y1a,
+                    CubicInterpolation::FirstDerivative, y1b);
     f.update();
     checkValues("Clamped spline", f,
                 BEGIN(generic_x), END(generic_x), BEGIN(generic_y));
@@ -513,10 +524,10 @@ void InterpolationTest::testSplineOnGenericValues() {
 
 
     // Not-a-knot spline
-    f = CubicSplineInterpolation(BEGIN(generic_x), END(generic_x), BEGIN(generic_y),
-                    CubicSplineInterpolation::NotAKnot, Null<Real>(),
-                    CubicSplineInterpolation::NotAKnot, Null<Real>(),
-                    false);
+    f = CubicInterpolation(BEGIN(generic_x), END(generic_x), BEGIN(generic_y),
+                           CubicInterpolation::Spline, false, 
+                           CubicInterpolation::NotAKnot, Null<Real>(),
+                           CubicInterpolation::NotAKnot, Null<Real>());
     f.update();
     checkValues("Not-a-knot spline", f,
                 BEGIN(generic_x), END(generic_x), BEGIN(generic_y));
@@ -547,10 +558,10 @@ void InterpolationTest::testSimmetricEndConditions() {
     y = gaussian(x);
 
     // Not-a-knot spline
-    CubicSplineInterpolation f(x.begin(), x.end(), y.begin(),
-                  CubicSplineInterpolation::NotAKnot, Null<Real>(),
-                  CubicSplineInterpolation::NotAKnot, Null<Real>(),
-                  false);
+    CubicInterpolation f(x.begin(), x.end(), y.begin(),
+                         CubicInterpolation::Spline, false, 
+                         CubicInterpolation::NotAKnot, Null<Real>(),
+                         CubicInterpolation::NotAKnot, Null<Real>());
     f.update();
     checkValues("Not-a-knot spline", f,
                 x.begin(), x.end(), y.begin());
@@ -559,9 +570,10 @@ void InterpolationTest::testSimmetricEndConditions() {
 
 
     // MC not-a-knot spline
-    f = MonotonicCubicSpline(x.begin(), x.end(), y.begin(),
-                             CubicSplineInterpolation::NotAKnot, Null<Real>(),
-                             CubicSplineInterpolation::NotAKnot, Null<Real>());
+    f = CubicInterpolation(x.begin(), x.end(), y.begin(),
+                           CubicInterpolation::Spline, true,
+                           CubicInterpolation::NotAKnot, Null<Real>(),
+                           CubicInterpolation::NotAKnot, Null<Real>());
     f.update();
     checkValues("MC not-a-knot spline", f,
                 x.begin(), x.end(), y.begin());
@@ -581,10 +593,10 @@ void InterpolationTest::testDerivativeEndConditions() {
     y = parabolic(x);
 
     // Not-a-knot spline
-    CubicSplineInterpolation f(x.begin(), x.end(), y.begin(),
-                  CubicSplineInterpolation::NotAKnot, Null<Real>(),
-                  CubicSplineInterpolation::NotAKnot, Null<Real>(),
-                  false);
+    CubicInterpolation f(x.begin(), x.end(), y.begin(),
+                         CubicInterpolation::Spline, false, 
+                         CubicInterpolation::NotAKnot, Null<Real>(),
+                         CubicInterpolation::NotAKnot, Null<Real>());
     f.update();
     checkValues("Not-a-knot spline", f,
                 x.begin(), x.end(), y.begin());
@@ -599,10 +611,10 @@ void InterpolationTest::testDerivativeEndConditions() {
 
 
     // Clamped spline
-    f = CubicSplineInterpolation(x.begin(), x.end(), y.begin(),
-                    CubicSplineInterpolation::FirstDerivative,  4.0,
-                    CubicSplineInterpolation::FirstDerivative, -4.0,
-                    false);
+    f = CubicInterpolation(x.begin(), x.end(), y.begin(),
+                           CubicInterpolation::Spline, false, 
+                           CubicInterpolation::FirstDerivative,  4.0,
+                           CubicInterpolation::FirstDerivative, -4.0);
     f.update();
     checkValues("Clamped spline", f,
                 x.begin(), x.end(), y.begin());
@@ -617,10 +629,10 @@ void InterpolationTest::testDerivativeEndConditions() {
 
 
     // SecondDerivative spline
-    f = CubicSplineInterpolation(x.begin(), x.end(), y.begin(),
-                    CubicSplineInterpolation::SecondDerivative, -2.0,
-                    CubicSplineInterpolation::SecondDerivative, -2.0,
-                    false);
+    f = CubicInterpolation(x.begin(), x.end(), y.begin(),
+                           CubicInterpolation::Spline, false, 
+                           CubicInterpolation::SecondDerivative, -2.0,
+                           CubicInterpolation::SecondDerivative, -2.0);
     f.update();
     checkValues("SecondDerivative spline", f,
                 x.begin(), x.end(), y.begin());
@@ -634,10 +646,10 @@ void InterpolationTest::testDerivativeEndConditions() {
                             x[n-1], -2.0);
 
     // MC Not-a-knot spline
-    f = CubicSplineInterpolation(x.begin(), x.end(), y.begin(),
-                    CubicSplineInterpolation::NotAKnot, Null<Real>(),
-                    CubicSplineInterpolation::NotAKnot, Null<Real>(),
-                    true);
+    f = CubicInterpolation(x.begin(), x.end(), y.begin(),
+                           CubicInterpolation::Spline, true, 
+                           CubicInterpolation::NotAKnot, Null<Real>(),
+                           CubicInterpolation::NotAKnot, Null<Real>());
     f.update();
     checkValues("MC Not-a-knot spline", f,
                 x.begin(), x.end(), y.begin());
@@ -652,10 +664,10 @@ void InterpolationTest::testDerivativeEndConditions() {
 
 
     // MC Clamped spline
-    f = CubicSplineInterpolation(x.begin(), x.end(), y.begin(),
-                    CubicSplineInterpolation::FirstDerivative,  4.0,
-                    CubicSplineInterpolation::FirstDerivative, -4.0,
-                    true);
+    f = CubicInterpolation(x.begin(), x.end(), y.begin(),
+                           CubicInterpolation::Spline, true, 
+                           CubicInterpolation::FirstDerivative,  4.0,
+                           CubicInterpolation::FirstDerivative, -4.0);
     f.update();
     checkValues("MC Clamped spline", f,
                 x.begin(), x.end(), y.begin());
@@ -670,10 +682,10 @@ void InterpolationTest::testDerivativeEndConditions() {
 
 
     // MC SecondDerivative spline
-    f = CubicSplineInterpolation(x.begin(), x.end(), y.begin(),
-                    CubicSplineInterpolation::SecondDerivative, -2.0,
-                    CubicSplineInterpolation::SecondDerivative, -2.0,
-                    true);
+    f = CubicInterpolation(x.begin(), x.end(), y.begin(),
+                           CubicInterpolation::Spline, true, 
+                           CubicInterpolation::SecondDerivative, -2.0,
+                           CubicInterpolation::SecondDerivative, -2.0);
     f.update();
     checkValues("MC SecondDerivative spline", f,
                 x.begin(), x.end(), y.begin());
@@ -706,10 +718,10 @@ void InterpolationTest::testNonRestrictiveHymanFilter() {
     Real zero=0.0, interpolated, expected=0.0;
 
     // MC Not-a-knot spline
-    CubicSplineInterpolation f(x.begin(), x.end(), y.begin(),
-                  CubicSplineInterpolation::NotAKnot, Null<Real>(),
-                  CubicSplineInterpolation::NotAKnot, Null<Real>(),
-                  true);
+    CubicInterpolation f(x.begin(), x.end(), y.begin(),
+                         CubicInterpolation::Spline, true,
+                         CubicInterpolation::NotAKnot, Null<Real>(),
+                         CubicInterpolation::NotAKnot, Null<Real>());
     f.update();
     interpolated = f(zero);
     if (std::fabs(interpolated-expected)>1e-15) {
@@ -723,10 +735,10 @@ void InterpolationTest::testNonRestrictiveHymanFilter() {
 
 
     // MC Clamped spline
-    f = CubicSplineInterpolation(x.begin(), x.end(), y.begin(),
-                    CubicSplineInterpolation::FirstDerivative,  4.0,
-                    CubicSplineInterpolation::FirstDerivative, -4.0,
-                    true);
+    f = CubicInterpolation(x.begin(), x.end(), y.begin(),
+                           CubicInterpolation::Spline, true,
+                           CubicInterpolation::FirstDerivative,  4.0,
+                           CubicInterpolation::FirstDerivative, -4.0);
     f.update();
     interpolated = f(zero);
     if (std::fabs(interpolated-expected)>1e-15) {
@@ -740,10 +752,10 @@ void InterpolationTest::testNonRestrictiveHymanFilter() {
 
 
     // MC SecondDerivative spline
-    f = CubicSplineInterpolation(x.begin(), x.end(), y.begin(),
-                    CubicSplineInterpolation::SecondDerivative, -2.0,
-                    CubicSplineInterpolation::SecondDerivative, -2.0,
-                    true);
+    f = CubicInterpolation(x.begin(), x.end(), y.begin(),
+                           CubicInterpolation::Spline, true,
+                           CubicInterpolation::SecondDerivative, -2.0,
+                           CubicInterpolation::SecondDerivative, -2.0);
     f.update();
     interpolated = f(zero);
     if (std::fabs(interpolated-expected)>1e-15) {
@@ -1179,7 +1191,7 @@ void InterpolationTest::testSabrInterpolation(){
     volatilities[24] = 0.958983602256592 ; volatilities[25] = 0.953860388001395 ; volatilities[26] = 0.948882997029509 ;
     volatilities[27] = 0.944043915545469 ; volatilities[28] = 0.939336183299237 ; volatilities[29] = 0.934753341079515 ;
     volatilities[30] = 0.930289384251337;
-    
+
     Time expiry = 1.0;
     Real forward = 0.039;
     // input SABR coefficients (corresponding to the vols above)
@@ -1225,13 +1237,13 @@ void InterpolationTest::testSabrInterpolation(){
     for (Size j=0; j<methods_.size(); ++j) {
       for (Size i=0; i<LENGTH(vegaWeighted); ++i) {
         for (Size k_a=0; k_a<LENGTH(isAlphaFixed); ++k_a) {
-          for (Size k_b=0; k_b<LENGTH(isBetaFixed); ++k_b) {            
+          for (Size k_b=0; k_b<LENGTH(isBetaFixed); ++k_b) {
             for (Size k_n=0; k_n<LENGTH(isNuFixed); ++k_n) {
               for (Size k_r=0; k_r<LENGTH(isRhoFixed); ++k_r) {
                 SABRInterpolation sabrInterpolation(strikes.begin(), strikes.end(),
                                                     volatilities.begin(), expiry, forward,
                                                     alphaGuess, betaGuess, nuGuess, rhoGuess,
-                                                    isAlphaFixed[k_a], isBetaFixed[k_b], 
+                                                    isAlphaFixed[k_a], isBetaFixed[k_b],
                                                     isNuFixed[k_n], isRhoFixed[k_r],
                                                     vegaWeighted[i],
                                                     endCriteria, methods_[j]);
@@ -1301,24 +1313,26 @@ void InterpolationTest::testSabrInterpolation(){
 
 test_suite* InterpolationTest::suite() {
     test_suite* suite = BOOST_TEST_SUITE("Interpolation tests");
-    
-    suite->add(BOOST_TEST_CASE(&InterpolationTest::testSplineOnGenericValues));
-    suite->add(BOOST_TEST_CASE(
+
+    suite->add(QUANTLIB_TEST_CASE(
+                        &InterpolationTest::testSplineOnGenericValues));
+    suite->add(QUANTLIB_TEST_CASE(
                         &InterpolationTest::testSimmetricEndConditions));
-    suite->add(BOOST_TEST_CASE(
+    suite->add(QUANTLIB_TEST_CASE(
                         &InterpolationTest::testDerivativeEndConditions));
-    suite->add(BOOST_TEST_CASE(
+    suite->add(QUANTLIB_TEST_CASE(
                         &InterpolationTest::testNonRestrictiveHymanFilter));
-    suite->add(BOOST_TEST_CASE(&InterpolationTest::testSplineOnRPN15AValues));
-    suite->add(BOOST_TEST_CASE(
+    suite->add(QUANTLIB_TEST_CASE(
+                        &InterpolationTest::testSplineOnRPN15AValues));
+    suite->add(QUANTLIB_TEST_CASE(
                         &InterpolationTest::testSplineOnGaussianValues));
-    suite->add(BOOST_TEST_CASE(
+    suite->add(QUANTLIB_TEST_CASE(
                         &InterpolationTest::testSplineErrorOnGaussianValues));
-    suite->add(BOOST_TEST_CASE(&InterpolationTest::testMultiSpline));
-    suite->add(BOOST_TEST_CASE(&InterpolationTest::testAsFunctor));
-    suite->add(BOOST_TEST_CASE(&InterpolationTest::testBackwardFlat));
-    suite->add(BOOST_TEST_CASE(&InterpolationTest::testForwardFlat));
-    suite->add(BOOST_TEST_CASE(&InterpolationTest::testSabrInterpolation));
+    suite->add(QUANTLIB_TEST_CASE(&InterpolationTest::testMultiSpline));
+    suite->add(QUANTLIB_TEST_CASE(&InterpolationTest::testAsFunctor));
+    suite->add(QUANTLIB_TEST_CASE(&InterpolationTest::testBackwardFlat));
+    suite->add(QUANTLIB_TEST_CASE(&InterpolationTest::testForwardFlat));
+    suite->add(QUANTLIB_TEST_CASE(&InterpolationTest::testSabrInterpolation));
     return suite;
 }
 
