@@ -101,7 +101,7 @@ test_suite* LinearLeastSquaresRegressionTest::suite() {
     test_suite* suite =
         BOOST_TEST_SUITE("linear least squares regression tests");
 
-    suite->add(BOOST_TEST_CASE(
+    suite->add(QUANTLIB_TEST_CASE(
         &LinearLeastSquaresRegressionTest::testRegression));
     return suite;
 }
