@@ -2,7 +2,7 @@
 
 /*
  Copyright (C) 2004, 2005, 2006, 2007 Ferdinando Ametrano
- Copyright (C) 2004, 2005, 2006, 2007 StatPro Italia srl
+ Copyright (C) 2004, 2005, 2006, 2007, 2008 StatPro Italia srl
 
  This file is part of QuantLib, a free-software/open-source library
  for financial quantitative analysts and developers - http://quantlib.org/
@@ -38,6 +38,8 @@
 //   namespace { unsigned int u = _controlfp(_EM_INEXACT, _MCW_EM); }
 
 #endif
+#include "utilities.hpp"
+
 #include "americanoption.hpp"
 #include "array.hpp"
 #include "asianoptions.hpp"
@@ -51,24 +53,33 @@
 #include "calendars.hpp"
 #include "capfloor.hpp"
 #include "capflooredcoupon.hpp"
+#include "cdo.hpp"
+#include "cdsoption.hpp"
 #include "cliquetoption.hpp"
 #include "cms.hpp"
 #include "compoundforward.hpp"
 #include "convertiblebonds.hpp"
 #include "covariance.hpp"
+#include "creditdefaultswap.hpp"
 #include "curvestates.hpp"
 #include "dates.hpp"
 #include "daycounters.hpp"
+#include "defaultprobabilitycurves.hpp"
 #include "digitalcoupon.hpp"
 #include "digitaloption.hpp"
 #include "distributions.hpp"
 #include "dividendoption.hpp"
 #include "europeanoption.hpp"
+#include "everestoption.hpp"
 #include "exchangerate.hpp"
+#include "extendedtrees.hpp"
 #include "factorial.hpp"
+#include "fdmlinearop.hpp"
 #include "forwardoption.hpp"
 #include "gaussianquadratures.hpp"
+#include "gjrgarchmodel.hpp"
 #include "hestonmodel.hpp"
+#include "himalayaoption.hpp"
 #include "hybridhestonhullwhiteprocess.hpp"
 #include "inflation.hpp"
 #include "instruments.hpp"
@@ -91,8 +102,11 @@
 #include "mclongstaffschwartzengine.hpp"
 #include "mersennetwister.hpp"
 #include "money.hpp"
+#include "nthtodefault.hpp"
 #include "operators.hpp"
 #include "optimizers.hpp"
+#include "optionletstripper.hpp"
+#include "pagodaoption.hpp"
 #include "pathgenerator.hpp"
 #include "period.hpp"
 #include "piecewiseyieldcurve.hpp"
@@ -115,11 +129,10 @@
 #include "tqreigendecomposition.hpp"
 #include "tracing.hpp"
 #include "transformedgrid.hpp"
+#include "varianceoption.hpp"
 #include "varianceswaps.hpp"
 #include "volatilitymodels.hpp"
 #include "swaptionvolatilitymatrix.hpp"
-// to be deprecated
-#include "old_pricers.hpp"
 
 #include <iostream>
 #include <iomanip>
@@ -175,39 +188,48 @@ test_suite* init_unit_test_suite(int, char* []) {
     BOOST_MESSAGE(rule);
     test_suite* test = BOOST_TEST_SUITE("QuantLib test suite");
 
-    test->add(BOOST_TEST_CASE(startTimer));
+    test->add(QUANTLIB_TEST_CASE(startTimer));
 
-    test->add(AmericanOptionTest::suite());   // FLOATING_POINT_EXCEPTION
+    test->add(AmericanOptionTest::suite());
     test->add(ArrayTest::suite());
     test->add(AsianOptionTest::suite());
     test->add(AssetSwapTest::suite()); // fails with QL_USE_INDEXED_COUPON
     test->add(BarrierOptionTest::suite());
-    test->add(BasketOptionTest::suite());   // FLOATING_POINT_EXCEPTION
-    test->add(BatesModelTest::suite());       // FLOATING_POINT_EXCEPTION
+    test->add(BasketOptionTest::suite());
+    test->add(BatesModelTest::suite());
     test->add(BermudanSwaptionTest::suite());
     test->add(BondTest::suite());
     test->add(BrownianBridgeTest::suite());
     test->add(CalendarTest::suite());
-    test->add(CapFloorTest::suite());       // FLOATING_POINT_EXCEPTION
+    test->add(CapFloorTest::suite());
     test->add(CapFlooredCouponTest::suite());
+    test->add(CdsOptionTest::suite());
+    test->add(CdoTest::suite());
     test->add(CliquetOptionTest::suite());
     test->add(CmsTest::suite());
     test->add(ConvertibleBondTest::suite());
     test->add(CovarianceTest::suite());
+    test->add(CreditDefaultSwapTest::suite());
     test->add(CurveStatesTest::suite());
     test->add(DateTest::suite());
     test->add(DayCounterTest::suite());
+    test->add(DefaultProbabilityCurveTest::suite());
     test->add(DigitalCouponTest::suite()); // might fail with QL_USE_INDEXED_COUPON
-    test->add(DigitalOptionTest::suite());  // FLOATING_POINT_EXCEPTION
+    test->add(DigitalOptionTest::suite());
     test->add(DistributionTest::suite());
-    test->add(DividendOptionTest::suite());   // FLOATING_POINT_EXCEPTION
-    test->add(EuropeanOptionTest::suite());   // FLOATING_POINT_EXCEPTION
+    test->add(DividendOptionTest::suite());
+    test->add(EuropeanOptionTest::suite());
+    test->add(EverestOptionTest::suite());
     test->add(ExchangeRateTest::suite());
+    test->add(ExtendedTreesTest::suite());
     test->add(FactorialTest::suite());
+    test->add(FdmLinearOpTest::suite());
     test->add(ForwardOptionTest::suite());
     test->add(GaussianQuadraturesTest::suite());
-    test->add(HestonModelTest::suite());        // FLOATING_POINT_EXCEPTION
-    test->add(HybridHestonHullWhiteProcessTest::suite());   // FLOATING_POINT_EXCEPTION
+    test->add(GJRGARCHModelTest::suite());
+    test->add(HestonModelTest::suite());
+    test->add(HimalayaOptionTest::suite());
+    test->add(HybridHestonHullWhiteProcessTest::suite());
     test->add(InflationTest::suite());
     test->add(InstrumentTest::suite());
     test->add(IntegralTest::suite());
@@ -217,19 +239,22 @@ test_suite* init_unit_test_suite(int, char* []) {
     test->add(LinearLeastSquaresRegressionTest::suite());
     test->add(LookbackOptionTest::suite());
     test->add(LowDiscrepancyTest::suite());
+    test->add(MarketModelTest::suite());
     test->add(MarketModelCmsTest::suite());
     test->add(MarketModelSmmTest::suite());
     test->add(MarketModelSmmCapletAlphaCalibrationTest::suite());
     test->add(MarketModelSmmCapletCalibrationTest::suite());
-    test->add(MarketModelSmmCapletHomoCalibrationTest::suite());    // FLOATING_POINT_EXCEPTION
-    test->add(MarketModelTest::suite());                          // FLOATING_POINT_EXCEPTION
+    test->add(MarketModelSmmCapletHomoCalibrationTest::suite());
     test->add(MatricesTest::suite());
-    test->add(MCLongstaffSchwartzEngineTest::suite());              // FLOATING_POINT_EXCEPTION
+    test->add(MCLongstaffSchwartzEngineTest::suite());
     test->add(MersenneTwisterTest::suite());
     test->add(MoneyTest::suite());
-    test->add(OperatorTest::suite());                             // FLOATING_POINT_EXCEPTION
+    test->add(NthToDefaultTest::suite());
+    test->add(OperatorTest::suite());
     test->add(OptimizersTest::suite());
-    test->add(PathGeneratorTest::suite());                          // FLOATING_POINT_EXCEPTION
+    test->add(OptionletStripperTest::suite());
+    test->add(PagodaOptionTest::suite());
+    test->add(PathGeneratorTest::suite());
     test->add(PeriodTest::suite());
     test->add(PiecewiseYieldCurveTest::suite());
     test->add(QuantoOptionTest::suite());
@@ -244,7 +269,7 @@ test_suite* init_unit_test_suite(int, char* []) {
     test->add(SurfaceTest::suite());
     test->add(SwapTest::suite());
     test->add(SwapForwardMappingsTest::suite());
-    test->add(SwaptionTest::suite());                               // FLOATING_POINT_EXCEPTION
+    test->add(SwaptionTest::suite());
     test->add(SwaptionVolatilityCubeTest::suite());
     test->add(SwaptionVolatilityMatrixTest::suite());
     test->add(TermStructureTest::suite());
@@ -252,16 +277,16 @@ test_suite* init_unit_test_suite(int, char* []) {
     test->add(TqrEigenDecompositionTest::suite());
     test->add(TracingTest::suite());
     test->add(TransformedGridTest::suite());
+    test->add(VarianceOptionTest::suite());
     test->add(VarianceSwapTest::suite());
     test->add(VolatilityModelsTest::suite());
 
-     //tests for deprecated classes
+    //tests for deprecated classes
     test->add(CompoundForwardTest::suite()); // fails with QL_USE_INDEXED_COUPON
     test->add(LiborMarketModelTest::suite());
     test->add(LiborMarketModelProcessTest::suite());
-    test->add(OldPricerTest::suite());                               // FLOATING_POINT_EXCEPTION
 
-    test->add(BOOST_TEST_CASE(stopTimer));
+    test->add(QUANTLIB_TEST_CASE(stopTimer));
 
     return test;
 }
