@@ -343,9 +343,10 @@ void RiskStatisticsTest::testResults() {
             expected = s.gaussianPotentialUpside(twoSigma);
             calculated = test.gaussianPotentialUpside(twoSigma);
             if (calculated!=expected)
-                BOOST_FAIL("GenericGaussianStatistics<StatsHolder> fails");
-
-
+                BOOST_FAIL("GenericGaussianStatistics<StatsHolder> fails"
+                           << std::setprecision(16)
+                           << "\n  calculated: " << calculated
+                           << "\n  expected: " << expected);
 
             // value-at-risk
             expected = -std::min<Real>(lower_tail,0.0);
@@ -607,7 +608,7 @@ void RiskStatisticsTest::testResults() {
 
 test_suite* RiskStatisticsTest::suite() {
     test_suite* suite = BOOST_TEST_SUITE("Risk statistics tests");
-    suite->add(BOOST_TEST_CASE(&RiskStatisticsTest::testResults));
+    suite->add(QUANTLIB_TEST_CASE(&RiskStatisticsTest::testResults));
     return suite;
 }
 
