@@ -2,7 +2,7 @@
 
 /*
  Copyright (C) 2006 Warren Chou
- Copyright (C) 2007 StatPro Italia srl
+ Copyright (C) 2007, 2008 StatPro Italia srl
 
  This file is part of QuantLib, a free-software/open-source library
  for financial quantitative analysts and developers - http://quantlib.org/
@@ -53,43 +53,43 @@ using namespace boost::unit_test_framework;
         << "    tolerance:        " << tolerance);
 
 
-QL_BEGIN_TEST_LOCALS(VarianceSwapTest)
+namespace {
 
-struct MCVarianceSwapData {
-    Position::Type type;
-    Real varStrike;
-    Real nominal;
-    Real s;         // spot
-    Rate q;         // dividend
-    Rate r;         // risk-free rate
-    Time t1;        // intermediate time
-    Time t;         // time to maturity
-    Volatility v1;  // volatility at t1
-    Volatility v;   // volatility at t
-    Real result;    // result
-    Real tol;       // tolerance
-};
+    struct MCVarianceSwapData {
+        Position::Type type;
+        Real varStrike;
+        Real nominal;
+        Real s;         // spot
+        Rate q;         // dividend
+        Rate r;         // risk-free rate
+        Time t1;        // intermediate time
+        Time t;         // time to maturity
+        Volatility v1;  // volatility at t1
+        Volatility v;   // volatility at t
+        Real result;    // result
+        Real tol;       // tolerance
+    };
 
-struct ReplicatingVarianceSwapData {
-    Position::Type type;
-    Real varStrike;
-    Real nominal;
-    Real s;         // spot
-    Rate q;         // dividend
-    Rate r;         // risk-free rate
-    Time t;         // time to maturity
-    Volatility v;   // volatility at t
-    Real result;    // result
-    Real tol;       // tolerance
-};
+    struct ReplicatingVarianceSwapData {
+        Position::Type type;
+        Real varStrike;
+        Real nominal;
+        Real s;         // spot
+        Rate q;         // dividend
+        Rate r;         // risk-free rate
+        Time t;         // time to maturity
+        Volatility v;   // volatility at t
+        Real result;    // result
+        Real tol;       // tolerance
+    };
 
-struct Datum {
-    Option::Type type;
-    Real strike;
-    Volatility v;
-};
+    struct Datum {
+        Option::Type type;
+        Real strike;
+        Volatility v;
+    };
 
-QL_END_TEST_LOCALS(VarianceSwapTest)
+}
 
 
 void VarianceSwapTest::testReplicatingVarianceSwap() {
@@ -196,18 +196,18 @@ void VarianceSwapTest::testReplicatingVarianceSwap() {
 
 
         boost::shared_ptr<PricingEngine> engine(
-                             new ReplicatingVarianceSwapEngine(5.0,
-                                                               callStrikes,
-                                                               putStrikes));
+                          new ReplicatingVarianceSwapEngine(stochProcess, 5.0,
+                                                            callStrikes,
+                                                            putStrikes));
 
         VarianceSwap varianceSwap(values[i].type,
                                   values[i].varStrike,
                                   values[i].nominal,
-                                  stochProcess,
-                                  exDate,
-                                  engine);
+                                  today,
+                                  exDate);
+        varianceSwap.setPricingEngine(engine);
 
-        Real calculated = varianceSwap.fairVariance();
+        Real calculated = varianceSwap.variance();
         Real expected = values[i].result;
         Real error = std::fabs(calculated-expected);
         if (error>values[i].tol)
@@ -275,18 +275,19 @@ void VarianceSwapTest::testMCVarianceSwap() {
 
         boost::shared_ptr<PricingEngine> engine;
         engine =
-            MakeMCVarianceSwapEngine<PseudoRandom>().withStepsPerYear(250)
-                                                    .withSamples(1023)
-                                                    .withSeed(42);
+            MakeMCVarianceSwapEngine<PseudoRandom>(stochProcess)
+            .withStepsPerYear(250)
+            .withSamples(1023)
+            .withSeed(42);
 
         VarianceSwap varianceSwap(values[i].type,
                                   values[i].varStrike,
                                   values[i].nominal,
-                                  stochProcess,
-                                  exDate,
-                                  engine);
+                                  today,
+                                  exDate);
+        varianceSwap.setPricingEngine(engine);
 
-        Real calculated = varianceSwap.fairVariance();
+        Real calculated = varianceSwap.variance();
         Real expected = values[i].result;
         Real error = std::fabs(calculated-expected);
         if (error>values[i].tol)
@@ -300,9 +301,9 @@ void VarianceSwapTest::testMCVarianceSwap() {
 test_suite* VarianceSwapTest::suite() {
     test_suite* suite = BOOST_TEST_SUITE("Variance swap tests");
 
-    suite->add(BOOST_TEST_CASE(
+    suite->add(QUANTLIB_TEST_CASE(
                              &VarianceSwapTest::testReplicatingVarianceSwap));
-    suite->add(BOOST_TEST_CASE(&VarianceSwapTest::testMCVarianceSwap));
+    suite->add(QUANTLIB_TEST_CASE(&VarianceSwapTest::testMCVarianceSwap));
     return suite;
 }
 
